Package: RQuantLib
Title: R Interface to the 'QuantLib' Library
Version: 0.4.27
Date: 2026-06-07
Authors@R: c(person("Dirk", "Eddelbuettel", role = c("aut", "cre"), email = "edd@debian.org",
                    comment = c(ORCID = "0000-0001-6419-907X")),
             person("Khanh", "Nguyen", role = "aut", comment = "2009-2010"),
             person("Terry", "Leitch", role = "aut",
                    comment = c(ORCID = "0000-0003-1706-4070", "since 2016")))
Description: The 'RQuantLib' package makes parts of 'QuantLib' accessible from R
 The 'QuantLib' project aims to provide a comprehensive software framework
 for quantitative finance. The goal is to provide a standard open source library
 for quantitative analysis, modeling, trading, and risk management of financial
 assets.
Suggests: tinytest, rgl, shiny
Imports: methods, Rcpp (>= 0.11.0), stats, graphics, zoo
LinkingTo: Rcpp
SystemRequirements: QuantLib library (>= 1.25) from
        https://quantlib.org, Boost library from https://www.boost.org
License: GPL (>= 2)
URL: https://github.com/eddelbuettel/rquantlib,
        https://dirk.eddelbuettel.com/code/rquantlib.html
BugReports: https://github.com/eddelbuettel/rquantlib/issues
RoxygenNote: 6.0.1
LazyLoad: true
NeedsCompilation: yes
Packaged: 2026-06-07 13:24:28 UTC; edd
Author: Dirk Eddelbuettel [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6419-907X>),
  Khanh Nguyen [aut] (2009-2010),
  Terry Leitch [aut] (ORCID: <https://orcid.org/0000-0003-1706-4070>,
    since 2016)
Maintainer: Dirk Eddelbuettel <edd@debian.org>
Repository: CRAN
Date/Publication: 2026-06-07 14:10:21 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-06-07 16:37:18 UTC; unix
Archs: RQuantLib.so.dSYM

Package: RQuantLib
Title: R Interface to the 'QuantLib' Library
Version: 0.4.27
Date: 2026-06-07
Authors@R: c(person("Dirk", "Eddelbuettel", role = c("aut", "cre"), email = "edd@debian.org",
                    comment = c(ORCID = "0000-0001-6419-907X")),
             person("Khanh", "Nguyen", role = "aut", comment = "2009-2010"),
             person("Terry", "Leitch", role = "aut",
                    comment = c(ORCID = "0000-0003-1706-4070", "since 2016")))
Description: The 'RQuantLib' package makes parts of 'QuantLib' accessible from R
 The 'QuantLib' project aims to provide a comprehensive software framework
 for quantitative finance. The goal is to provide a standard open source library
 for quantitative analysis, modeling, trading, and risk management of financial
 assets.
Suggests: tinytest, rgl, shiny
Imports: methods, Rcpp (>= 0.11.0), stats, graphics, zoo
LinkingTo: Rcpp
SystemRequirements: QuantLib library (>= 1.25) from
        https://quantlib.org, Boost library from https://www.boost.org
License: GPL (>= 2)
URL: https://github.com/eddelbuettel/rquantlib,
        https://dirk.eddelbuettel.com/code/rquantlib.html
BugReports: https://github.com/eddelbuettel/rquantlib/issues
RoxygenNote: 6.0.1
LazyLoad: true
NeedsCompilation: yes
Packaged: 2026-06-07 13:24:28 UTC; edd
Author: Dirk Eddelbuettel [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6419-907X>),
  Khanh Nguyen [aut] (2009-2010),
  Terry Leitch [aut] (ORCID: <https://orcid.org/0000-0003-1706-4070>,
    since 2016)
Maintainer: Dirk Eddelbuettel <edd@debian.org>
Repository: CRAN
Date/Publication: 2026-06-07 14:10:21 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-06-07 17:18:17 UTC; unix
Archs: RQuantLib.so.dSYM

## Emacs, make this -*- mode: R; -*-

import("methods")
importFrom("Rcpp", "Rcpp.plugin.maker", "sourceCpp")
importFrom("stats", "sd")
importFrom("graphics", "par", "plot", "lines", "legend")
importFrom("utils", "compareVersion", "packageVersion")
importFrom("zoo","zoo","na.spline")

useDynLib("RQuantLib", .registration=TRUE)

export(
    ##--arrays.R
    "oldEuropeanOptionArrays",
    "EuropeanOptionArrays",
    "plotOptionSurface",
    "matchParams",
    ##--asian.R
    "AsianOption",
    ##--bermudan.R
    "BermudanSwaption",
    "AffineSwaption",
    #"sabrengine",
    "SabrSwaption",
    "summary.G2Analytic",
    "summary.HWAnalytic",
    "summary.HWTree",
    "summary.BKTree",
    ##--bond.R
    "ZeroCouponBond",
    "ZeroPriceByYield",
    "ZeroYield",
    "FixedRateBond",
    "FixedRateBondYield",
    "FixedRateBondPriceByYield",
    "FloatingRateBond",
    "ConvertibleZeroCouponBond",
    "ConvertibleFixedCouponBond",
    "ConvertibleFloatingCouponBond",
    "CallableBond",
    "FittedBondCurve",
    ##--calendars.R
    "calendars",
    "isBusinessDay", "businessDay",
    "isHoliday",
    "isWeekend",
    "isEndOfMonth",
    "getEndOfMonth", "endOfMonth",
    "adjust",
    "advance",
    "businessDaysBetween",
    "getHolidayList", "holidayList",
    "getBusinessDayList", "businessDayList",
    "setCalendarContext",
    "addHolidays",
    "removeHolidays",
    ##--dayCounter.R
    "dayCount",
    "yearFraction",
    "setEvaluationDate",
    ##--dates.cpp
    "advanceDate",
    ##--discount.R
    "DiscountCurve",
    "plot.DiscountCurve",
    ##--implied.R
    "EuropeanOptionImpliedVolatility",
    "AmericanOptionImpliedVolatility",
    "BinaryOptionImpliedVolatility",
    ##--option.R
    "EuropeanOption",
    "AmericanOption",
    "BinaryOption",
    "BarrierOption",
    ##--schedule.R
    "Schedule",
    ##--utils.R
    "getQuantLibVersion",
    "getQuantLibCapabilities"
)

S3method("AmericanOption", "default")
S3method("AmericanOptionImpliedVolatility", "default")
S3method("AsianOption", "default")
S3method("BarrierOption", "default")
S3method("BermudanSwaption", "default")
S3method("AffineSwaption", "default")
S3method("SabrSwaption", "default")

S3method("BinaryOption", "default")
S3method("BinaryOptionImpliedVolatility", "default")
S3method("CallableBond", "default")
S3method("ConvertibleFixedCouponBond", "default")
S3method("ConvertibleFloatingCouponBond", "default")
S3method("ConvertibleZeroCouponBond", "default")
S3method("DiscountCurve", "default")
S3method("EuropeanOption", "default")
S3method("EuropeanOptionImpliedVolatility", "default")
S3method("FittedBondCurve", "default")
S3method("FixedRateBond", "default")
S3method("FixedRateBondPriceByYield", "default")
S3method("FixedRateBondYield", "default")
S3method("FloatingRateBond", "default")
S3method("plot", "DiscountCurve")
S3method("Schedule", "default")
S3method("summary", "BKTree")
S3method("summary", "G2Analytic")
S3method("summary", "HWAnalytic")
S3method("summary", "HWTree")
S3method("summary", "G2AnalyticAffineSwaption")
S3method("summary", "HWAnalyticAffineSwaption")
S3method("summary", "HWTreeAffineSwaption")
S3method("ZeroCouponBond", "default")
S3method("ZeroPriceByYield", "default")
S3method("ZeroYield", "default")

S3method("plot", "Option")
S3method("print", "Option")
S3method("summary", "Option")

S3method("plot", "Bond")
S3method("print", "Bond")
S3method("summary", "Bond")

S3method("print", "FixedRateBond")

S3method("print", "ImpliedVolatility")
S3method("summary", "ImpliedVolatility")

## RQuantLib: R Interface to the [QuantLib](https://www.quantlib.org) Library

[![CI](https://github.com/eddelbuettel/rquantlib/workflows/ci/badge.svg)](https://github.com/eddelbuettel/rquantlib/actions?query=workflow%3Aci)
[![License](https://img.shields.io/badge/license-GPL%20%28%3E=%202%29-brightgreen.svg?style=flat)](http://www.gnu.org/licenses/gpl-2.0.html)
[![CRAN](https://www.r-pkg.org/badges/version/RQuantLib)](https://cran.r-project.org/package=RQuantLib)
[![r-universe](https://eddelbuettel.r-universe.dev/badges/RQuantLib)](https://eddelbuettel.r-universe.dev/RQuantLib)
[![Debian package](https://img.shields.io/debian/v/r-cran-rquantlib/sid?color=brightgreen)](https://packages.debian.org/sid/r-cran-rquantlib)
[![Dependencies](https://tinyverse.netlify.app/badge/RQuantLib)](https://cran.r-project.org/package=RQuantLib)
[![Downloads](https://cranlogs.r-pkg.org/badges/RQuantLib?color=brightgreen)](https://www.r-pkg.org:443/pkg/RQuantLib)
[![Last Commit](https://img.shields.io/github/last-commit/eddelbuettel/rquantlib)](https://github.com/eddelbuettel/rquantlib)

### About

The RQuantLib package makes parts of
[QuantLib](https://github.com/lballabio/quantlib) visible to the R
user. Currently a number option pricing functions are included, both
vanilla and exotic, as well as a broad range of fixed-income
functions. Also included are general calendaring and holiday
utilities. Further software contributions are welcome.

The QuantLib project aims to provide a comprehensive software framework for
quantitative finance. The goal is to provide a standard open source library
for quantitative analysis, modeling, trading, and risk management of
financial assets.

### Status

The package is actively maintained, and is still being
extended. Contributions are welcome, and initial discussions via
[GitHub issue tickets](https://github.com/eddelbuettel/rquantlib/issues)
are encouraged as suggested in the
[Contributing guide](https://github.com/eddelbuettel/rquantlib/blob/master/Contributing.md).

The package is always tested against the most recent version of QuantLib itself, and
it generally updated (should a change be needed) when QuantLib releases updates.

### Installation

#### From Source

The package is on [CRAN](https://cran.r-project.org) and can be installed as usual:

```{r}
install.packages("RQuantLib")
```

Windows binary packages are available via [CRAN](https://cran.r-project.org) thanks to the work by
[Joshua Ulrich](https://about.me/joshuaulrich) and [Jeroen Ooms](https://github.com/jeroen)
providing a QuantLib binary for the CRAN builders. Similarly, binaries for macOS _can_ be provided
when a suitable macOS library of QuantLib is prepared, possibly via
[s-u/recipes](https://github.com/R-macos/recipes).  If and when these binary libraries may be outdated,
please raise the issue on the [rquantlib mailing list](http://rquantlib.groups.io).

For more OS-specific installation options, please see [the wiki](https://github.com/eddelbuettel/rquantlib/wiki/RQuantLib).

### Support

Come to the friendly and low-volume [rquantlib mailing list](http://rquantlib.groups.io) for help.

### See Also

The [qlcal](https://github.com/qlcal/qlcal-r) R package provides the calendaring functionality of
[QuantLib](https://github.com/lballabio/quantlib) in a standalone R package that does not require
[QuantLib](https://github.com/lballabio/quantlib).

### Authors

Dirk Eddelbuettel, Khanh Nguyen (during 2009-2010) and Terry Leitch (since 2016)

### License

GPL (>= 2)

## use shiny, if installed to show Shiny App installed with package
if (require("shiny")) {
    library(RQuantLib)
    runApp(system.file("shiny", "DiscountCurve", package="RQuantLib"),
           port=8765
           ## add   host="0.0.0.0"   if access from outside current machine needed
           )
}

library(shiny)
library(RQuantLib)

shinyServer(function(input, output) {

    ## This data is taken from sample code shipped with QuantLib 0.9.7
    ## from the file Examples/Swap/swapvaluation
    params <- list(tradeDate=as.Date('2004-09-20'),
                   settleDate=as.Date('2004-09-22'),
                   dt=.25,
                   interpWhat="discount",
                   interpHow="loglinear")
    setEvaluationDate(as.Date("2004-09-20"))

    ## We get numerical issue for the spline interpolation if we add
    ## any on of these three extra futures -- the original example
    ## creates different curves based on different deposit, fra, futures
    ## and swap data
    ## Removing s2y helps, as kindly pointed out by Luigi Ballabio
    tsQuotes <- list(d1w = 0.0382,
                     d1m = 0.0372,
                     d3m = 0.0363,
                     d6m = 0.0353,
                     d9m = 0.0348,
                     d1y = 0.0345,
                     fut1=96.2875,
                     fut2=96.7875,
                     fut3=96.9875,
                     fut4=96.6875,
                     fut5=96.4875,
                     fut6=96.3875,
                     fut7=96.2875,
                     fut8=96.0875,
                     ## s2y = 0.037125,
                     s3y = 0.0398,
                     s5y = 0.0443,
                     s10y = 0.05165,
                     s15y = 0.055175)

    times <- seq(0,10,.1)
   
    ## Reactive expression to generate the requested curves.
    data <- reactive({
        params$interpHow <- input$interpolation
        curve <- DiscountCurve(params, tsQuotes, times)
    })

    ## Generate a plot of the data.
    output$plot <- renderPlot({
        interp <- input$interpolation
        crv <- input$curve
        dat <- data()
        plot(dat[["times"]], dat[[crv]],
             type='l', main=paste(interp, crv),
             ylab=crv, xlab="time in years")
    })
  
    ## Generate a summary of the data
    output$summary <- renderPrint({
        dat <- data()
        cat("Return Object Structure\n")
        str(dat)
        cat("\n\nSummary of first four elements\n")
        summary(data.frame(dat[1:4]))
    })
  
    ## Generate an HTML table view of the data
    output$table <- renderTable({
        data.frame(x=data()[1:4])
    })
    
})

# This is the server logic for a Shiny web application.
# You can find out more about building applications with Shiny here:
#
# http://shiny.rstudio.com
#
library(rhandsontable)
library(lubridate)
library(reshape2)
library(zoo)
library(data.table)
data(vcube)
data(tsQuotes)
library(shiny)

assignInNamespace("volDF2CubeK",source("volDF2CubeK.R"),"RQuantLib")

# tsQuotes <- list(
#   d1w  =0.004,
#   d1y=.0091,
#   s2y=.0115,
#   s3y=.0138,
#   s4y =.011349,
#   s5y  =0.012375,
#   s6y =.0134,
#   s7y=.01446,
#   s10y =0.016829,
#   s15y =0.01944,
#   s30y=.02178,
#   s50y=.02205)

tsQuoteUp01=lapply(tsQuotes,"+",.0001)
tsQuoteDn01=lapply(tsQuotes,"-",.0001)

setEvaluationDate(as.Date("2016-4-30"))

params <- list(tradeDate=as.Date('2016-4-28'),
               settleDate=as.Date('2016-4-30'),
               payFixed=TRUE,
               #european=F,
               dt=.25,
               interpWhat="discount",
               interpHow="loglinear")

legparams=list(dayCounter="Thirty360",
               fixFreq="Annual",
               floatFreq="Semiannual")

times<-times <- seq(0,49.5,.25)

dcurve <- DiscountCurve(params, tsQuotes, times=times,legparams)
dcurveUp01 <- DiscountCurve(params, tsQuoteUp01, times=times,legparams)
dcurveDn01 <- DiscountCurve(params, tsQuoteDn01, times=times,legparams)

expiriesY=c(1/12,.25,.5,1,2,3,4,5,6,7,8,9,10)
expiriesM=c(1,3,6,0,0,0,0,0,0,0,0,0,0)
expLvl=c("1M","3M","6M","1Y" , "2Y", "3Y",  "4Y",  "5Y",  "6Y",  "7Y",  "8Y",  "9Y","10Y")

tenorLvl=c("1Y" , "2Y",  "5Y",  "10Y","15Y","20Y","30Y")
tenors=c(1,2,5,10,15,20,30)



shinyServer(function(input, output) {
    inputVol<-vcube
    output$table<- renderRHandsontable({
  
        tbl2=inputVol
        tbl2$Expiry<-factor(tbl2$Expiry,level=expLvl)
        tbl2$Tenor=factor(tbl2$Tenor,level=tenorLvl)
        tbl2=subset(tbl2,Expiry %in% input$expiries)
        tbl2=subset(tbl2,Tenor %in% input$tenors)

        tbl2=tbl2[with(tbl2,order(Expiry,Tenor,-Spread))]
        tbl2[,"payer"]<- -.0000001
        tbl2[,"payerDV01"]<- -.0000001
        tbl2[,"payerCnvx"]<- -.0000001
        tbl2[,"rcv"]<- -.0000001
        tbl2[,"rcvDV01"]<- -.0000001
        tbl2[,"rcvCnvx"]<- -.0000001
        tbl2[,"payerVega"]<- -.0000001
        tbl2[,"rcvVega"]<- -.0000001
        tbl2[,"sabrVol"]<- -.0000001
        tbl2[,"strike"]<- -.00000001
        for(expire in input$expiries){
          for(tenor in input$tenors){
            expIDX=match(expire,expLvl)
            expiry=params$settleDate;
            if(expIDX>3){
            year(expiry)=year(expiry)+expiriesY[expIDX]
            } else{
            expiry=expiry%m+% months(expiriesM[expIDX])
            }
            tenorIDX=match(tenor,tenorLvl)
            tenorVal=expiry;
            params$european=!input$bermudan
            year(tenorVal)=year(tenorVal)+tenors[tenorIDX]
            params$expiryDate=expiry
            if(params$european){params$startDate=expiry;}else{params$startDate=params$settleDate}
            params$maturity=tenorVal
            params$strike=.03
            tmp2<-SabrSwaption(params, dcurve, inputVol,legparams, vega=T)
            fwd1=tbl2[tbl2$Expiry==expire & tbl2$Tenor== tenor & tbl2$Spread==0,]$Strike
            fwdDiff=fwd1-tmp2$atmRate
            for(spread in unique(tbl2$Spread)){
              params$strike=tmp2$atmRate+spread/10000
              if(params$strike>0){
              tmp<-SabrSwaption(params, dcurve, inputVol,legparams,tsUp01=dcurveUp01,tsDn01=dcurveDn01,vega=T)
              tbl2[tbl2$Expiry==expire & tbl2$Tenor== tenor & tbl2$Spread==spread,rcv:=signif(tmp$rcv,digits=4)]
              tbl2[tbl2$Expiry==expire & tbl2$Tenor== tenor & tbl2$Spread==spread,payer:=signif(tmp$pay,digits=4)] 
              tbl2[tbl2$Expiry==expire & tbl2$Tenor== tenor & tbl2$Spread==spread,rcvDV01:=signif(tmp$rcvDV01,digits=6)]
              tbl2[tbl2$Expiry==expire & tbl2$Tenor== tenor & tbl2$Spread==spread,rcvCnvx:=signif(tmp$rcvCnvx,digits=6)]
              tbl2[tbl2$Expiry==expire & tbl2$Tenor== tenor & tbl2$Spread==spread,payerDV01:=signif(tmp$payDV01,digits=6)]
              tbl2[tbl2$Expiry==expire & tbl2$Tenor== tenor & tbl2$Spread==spread,payerCnvx:=signif(tmp$payCnvx,digits=6)]
              tbl2[tbl2$Expiry==expire & tbl2$Tenor== tenor & tbl2$Spread==spread,rcvVega:=signif(tmp$rcvVega,digits=4)]
              tbl2[tbl2$Expiry==expire & tbl2$Tenor== tenor & tbl2$Spread==spread,payerVega:=signif(tmp$payVega,digits=4)]
              tbl2[tbl2$Expiry==expire & tbl2$Tenor== tenor & tbl2$Spread==spread,sabrVol:=signif(tmp$sigma,digits=3)]
              tbl2[tbl2$Expiry==expire & tbl2$Tenor== tenor & tbl2$Spread==spread,strike:=signif(params$strike,digits=4)]
              }
            }
          }
        }
        tbl2=tbl2[,c("Expiry","Tenor","Spread","strike","LogNormalVol","sabrVol","payer","payerDV01","payerCnvx","payerVega","rcv","rcvDV01","rcvCnvx","rcvVega"),with=F]
        numRow=length(tbl2)
        rhandsontable(tbl2,readOnly = F,rowHeaders=NULL,selectCallback = TRUE,halign="htCenter")%>%
          hot_col("strike", format = "0.0000") %>%
          hot_col("LogNormalVol", format = "0.00") %>%
          hot_col("sabrVol", format = "0.00") %>%
          hot_col("payer", format = "0.0000") %>%
          hot_col("payerDV01", format = "0.000000") %>%
          hot_col("payerCnvx", format = "0.000000") %>%
          hot_col("payerVega", format = "0.0000") %>%
          hot_col("rcv", format = "0.000")%>%
          hot_col("rcvDV01", format = "0.00000")%>%
          hot_col("rcvCnvx", format = "0.00000")%>%
          hot_col("rcvVega", format = "0.0000")
    })
    


})

volDF2CubeK <- function(params, tbl, source = "CME") {
  strikes <- levels(tbl$Spread)<-c(-200,-150,-100,-75,-50,-25,0,25,50,75,100,150,200)
  
  matYears <- as.numeric(params$maturity-params$tradeDate)/365
  expYears <- as.numeric(params$expiryDate-params$tradeDate)/365
  expLvl <- c( "1M","3M","6M","1Y","2Y","3Y","4Y",  "5Y",  "6Y",  "7Y",  "8Y",  "9Y","10Y")
  tbl$Expiry <- factor(tbl$Expiry, levels  <-  expLvl)
  expiries <- c(1/12,.25,.5,1,2,3,4,5,6,7,8,9,10)
  tenorLvl <- c( "1Y", "2Y",  "5Y",  "10Y","15Y","20Y","30Y")
  tbl$Tenor <- factor(tbl$Tenor, levels = tenorLvl)
  tenors <- c(1,2,5,10,15,20,30)
  tenorIDX <- max(findInterval(matYears-expYears,tenors),1)
  tenorIDX <- min(tenorIDX,length(tenors)-3)
  expiryIDX <- findInterval(expYears,expiries)
  expiryIDX <- min(expiryIDX,length(expiries)-3)
  
  strikeIDX <- 1
  expire <- expLvl[expiryIDX]
  tenor <- tenorLvl[tenorIDX]
  
  for(strike in levels(tbl$Spread)){
    if(!is.na(tbl[tbl$Expiry==expire & tbl$Spread==strike &tbl$Tenor==tenor,]$LogNormalVol))break;
    strikeIDX <- strikeIDX+1
  }
  strikes <- strikes[strikeIDX:length(strikes)]
  expLvl <- expLvl[expiryIDX:length(expLvl)]
  expiries <- expiries[expiryIDX:length(expiries)]
  tenorLvl <- tenorLvl[tenorIDX:length(tenorLvl)]
  tenors <- tenors[tenorIDX:length(tenors)]
  tbl <- tbl[tbl$Expiry%in%expLvl,]
  tbl <- tbl[tbl$Tenor%in%tenorLvl,]
  tbl <- tbl[tbl$Spread%in%strikes,]
  tbl <- tbl[with(tbl,order(Expiry,Tenor,Spread)),]
  tbl3 <- tbl[tbl$Spread==0,]
  # atm vol matrix
  atmMat=acast(tbl3,Expiry~Tenor,value.var = "LogNormalVol")
  #atmMat=matrix(data=NA,nrow=length(expLvl),ncol=length(tenorLvl),dimnames=list(expLvl,tenorLvl))
  # for(i in 1:length(expLvl)){
  #   for(j in 1:length(tenorLvl)){
  #     atmMat[i,j]=tbl[tbl$Expiry==expLvl[i]& tbl$Tenor==tenorLvl[j] & tbl$Spread==0,]$LogNormalVol
  #   }
  # }
  smirk=matrix(ncol=length(strikes),nrow=length(expLvl)*length(tenorLvl))
  
  tmp3=acast(tbl,Expiry~Tenor~Spread,value.var="LogNormalVol")
  k=0
  for(i in 1:length(expLvl)){
    for(j in 1:length(tenorLvl)){
      k=k+1
      #for(n in 1:length(strikes)){
      smirk[k,]=tmp3[i,j,]-tmp3[i,j,"0"]
      #smirk[k,n]=tbl[tbl$Expiry==expLvl[i]& tbl$Tenor==tenorLvl[j] & tbl$Spread==strikes[n],]$LogNormalVol -
      # tbl[tbl$Expiry==expLvl[i]& tbl$Tenor==tenorLvl[j] & tbl$Spread==0,]$LogNormalVol
      #}
    }
  }
  
  
  smirk <- na.spline(smirk,method="natural")
  
  tmp <- list(atmVol=atmMat,tenors=tenors,expiries=expiries,smirk=smirk,strikes=strikes/10000)
  class(tmp) <- "volcube"
  
  return(tmp)
}

library(shiny)

shinyUI(fluidPage(
    
    ## Application title
    titlePanel("Simple DiscountCurve Example from RQuantLib"),
  
    ## Sidebar with controls to select parameters
    sidebarLayout(
        sidebarPanel(
            radioButtons("interpolation", "Interpolation type:",
                         c("loglinear" = "loglinear",
                           "linear" = "linear",
                           "spline" = "spline")),

            br(),

            radioButtons("curve", "Curve type:",
                         c("forwards" = "forwards",
                           "zero rates" = "zerorates",
                           "discounts" = "discounts"))
        ),
        
        ## Show a tabset that includes a plot, summary, and table view
        mainPanel(
            tabsetPanel(type = "tabs", 
                        tabPanel("Plot", plotOutput("plot")), 
                        tabPanel("Summary", verbatimTextOutput("summary")), 
                        tabPanel("Table", tableOutput("table"))
                        )
        )
    )
))

# This is the user-interface definition of a Shiny web application.
# You can find out more about building applications with Shiny here:
#
# http://shiny.rstudio.com
#

library(shiny)

shinyUI(fluidPage(

  # Application title
  titlePanel("SABR Vol Cube Tool"),
  checkboxGroupInput("expiries", label = h3("Option Expiries"), inline=T,
                     choices=list("1M"="1M","3M"="3M","6M"="6M","1Y"="1Y" , "2Y"="2Y", "3Y"="3Y",  
                                   "4Y"="4Y",  "5Y"="5Y",  "6Y"="6Y",  "7Y"="7Y",  "8Y"="8Y",  "9Y"="9Y","10Y"="10Y"),
                    # selected = c("1M","3M","6M","1Y" , "2Y", "3Y",  "4Y",  "5Y",  "6Y",  "7Y",  "8Y",  "9Y","10Y")),
  selected = c( "3Y",  "4Y","5Y")),
  checkboxGroupInput("tenors", label = h3("Swap Tenors"), inline=T,
                     choices=list("1Y"="1Y" , "2Y"="2Y",  "5Y"="5Y","10Y"="10Y","15Y"="15Y","20Y"="20Y","30Y"="30Y"),
                     #selected = c("1Y" , "2Y",  "5Y", "10Y","15Y","20Y","30Y")),
  selected = c( "5Y")),
  checkboxInput("bermudan", label = "Bermudan Exercise?", value = F), 
  

    # Show a plot of the generated distribution
    mainPanel(
        rHandsontableOutput(outputId="table")
    )
  
))

## RQuantLib Demo for (European) Option surfaces
## Dirk Eddelbuettel, September 2005
## $Id: OptionSurfaces.R,v 1.1 2005/10/12 03:42:45 edd Exp $

OptionSurface <- function(EOres, label, fov=60) {
    #
    # This can be removed when rgl 0.111.5 is released:
    if (packageVersion("rgl") < "0.111.5")
      surface3d <- rgl.surface
    # End of old workaround
    axis.col <- "black"
    text.col <- axis.col
    ylab <- label
    xlab <- "Underlying"
    zlab <- "Volatility"
    y <- EOres

    ## clear scene:
    clear3d()
    clear3d(type="bbox")
    clear3d(type="lights")

    ## setup env:
    ##  bg3d(color="#887777")
    bg3d(color="#DDDDDD")
    light3d()

    view3d(fov=fov)
    ##bg3d(col="white", fogtype="exp2")
    ##bg3d(col="black", fogtype="exp2")
    ##bg3d(col="black", fogtype="exp")
    ##bg3d(col="white", fogtype="exp")

    x <- (1:nrow(y))
    z <- (1:ncol(y))
    x <- (x-min(x))/(max(x)-min(x))
    y <- (y-min(y))/(max(y)-min(y))
    z <- (z-min(z))/(max(z)-min(z))
    surface3d(x = x, y = y, z = z, alpha=0.6, lit=TRUE, color="blue")
    lines3d(c(0,1), c(0,0), c(0,0), col=axis.col)
    lines3d(c(0,0), c(0,1), c(0,0), col=axis.col)
    lines3d(c(0,0),c(0,0), c(0,1), col=axis.col)
    text3d(1,0,0, xlab, adj=1, col=text.col)
    text3d(0,1,0, ylab, adj=1, col=text.col)
    text3d(0,0,1, zlab, adj=1, col=text.col)

    ## add grid (credit's to John Fox scatter3d)
    xgridind <- round(seq(1, nrow(y), length=25))
    zgridind <- round(seq(1, ncol(y), length=25))
    surface3d(x = x[xgridind], y = y[xgridind,zgridind], z = z[zgridind], 
              color="darkgray", alpha=0.5, lit=TRUE,
              front="lines", back="lines")

    ## animate (credit to view3d() example)
    start <- proc.time()[3]
    while ((i <- 36*(proc.time()[3]-start)) < 360) {
        view3d(i,i/8);
    }
}

RQuantLib.demo.OptionSurfaces <- function() {

    und.seq <- seq(10, 200, by = 2.5)
    vol.seq <- seq(0.05, 2, by = 0.025)

    cat("Calculating surface ...")
    EOarr <- EuropeanOptionArrays("call", underlying = und.seq, strike = 100,
                                  dividendYield = 0.01, riskFreeRate = 0.03,
                                  maturity = 1, volatility = vol.seq)
    cat(" done.\n")

    open3d()
    OptionSurface(EOarr$value, "Value")
    OptionSurface(EOarr$delta, "Delta")
    OptionSurface(EOarr$gamma, "Gamma")
    OptionSurface(EOarr$vega, "Vega")
    OptionSurface(EOarr$theta, "Theta")

}

require(rgl,quiet=TRUE)
require(RQuantLib,quiet=TRUE)

RQuantLib.demo.OptionSurfaces()

## doRUnit.R --- Run RUnit tests
##
## with credits to package fUtilities in RMetrics
## which credits Gregor Gojanc's example in CRAN package  'gdata'
## as per the R Wiki http://wiki.r-project.org/rwiki/doku.php?id=developers:runit
## and changed further by Martin Maechler
## and more changes by Murray Stokely in HistogramTools
## and then used adapted in RProtoBuf
## and now used in Rcpp* and here 
##
## (Initially) inAdapted for RQuantLib by Dirk Eddelbuettel, 29 Dec 2007

stopifnot(require(RUnit, quietly=TRUE))
stopifnot(require(RQuantLib, quietly=TRUE))

## Define tests
testSuite <- defineTestSuite(name="RQuantLib Unit Tests",
                             dirs=system.file("unitTests", package = "RQuantLib"),
                             testFuncRegexp = "^[Tt]est.+")

## without this, we get (or used to get) unit test failures
Sys.setenv("R_TESTS"="")

## Run tests
tests <- runTestSuite(testSuite)

## Print results
printTextProtocol(tests)

## Return success or failure to R CMD CHECK
if (getErrors(tests)$nFail > 0) {
    stop("TEST FAILED!")
}
if (getErrors(tests)$nErr > 0) {
    stop("TEST HAD ERRORS!")
}
if (getErrors(tests)$nTestFunc < 1) {
    stop("NO TEST FUNCTIONS RUN!")
}

##  RQuantLib -- R interface to the QuantLib libraries
##
##  Copyright (C) 2002-2026  Dirk Eddelbuettel <edd@debian.org>
##
##  This file is part of RQuantLib.
##
##  RQuantLib is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 2 of the License, or
##  (at your option) any later version.
##
##  RQuantLib is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with RQuantLib.  If not, see <http://www.gnu.org/licenses/>.

EuropeanOption <- function(type, underlying, strike, dividendYield,
                           riskFreeRate, maturity, volatility,
                           discreteDividends = NULL, discreteDividendsTimeUntil = NULL,
                           dayCounter = 0) {
    UseMethod("EuropeanOption")
}

EuropeanOption.default <- function(type, underlying, strike, dividendYield,
                                   riskFreeRate, maturity, volatility,
                                   discreteDividends = NULL, discreteDividendsTimeUntil = NULL,
                                   dayCounter = 0) {
    type <- match.arg(type, c("call", "put"))
    val <- europeanOptionEngine(type, underlying, strike, dividendYield, riskFreeRate,
                                if (inherits(maturity, "numeric")) maturity else NULL,
                                if (inherits(maturity, "Date")) maturity else NULL,
                                volatility, discreteDividends, discreteDividendsTimeUntil, dayCounter)
    class(val) <- c("EuropeanOption", "Option")
    val
}

AmericanOption <- function(type, underlying, strike, dividendYield,
                           riskFreeRate, maturity, volatility,
                           timeSteps=150, gridPoints=149, engine="BaroneAdesiWhaley",
                           discreteDividends = NULL, discreteDividendsTimeUntil = NULL,
                           dayCounter = 0) {
    UseMethod("AmericanOption")
}

AmericanOption.default <- function(type, underlying, strike, dividendYield,
                                   riskFreeRate, maturity, volatility,
                                   timeSteps=150, gridPoints=149, engine="BaroneAdesiWhaley",
                                   discreteDividends = NULL, discreteDividendsTimeUntil = NULL,
                                   dayCounter = 0) {
    type <- match.arg(type, c("call", "put"))
    engine <- match.arg(engine, c("BaroneAdesiWhaley", "CrankNicolson"))
    val <- americanOptionEngine(type, underlying, strike, dividendYield, riskFreeRate,
                                if (inherits(maturity, "numeric")) maturity else NULL,
                                if (inherits(maturity, "Date")) maturity else NULL,
                                volatility, timeSteps, gridPoints, engine,
                                discreteDividends, discreteDividendsTimeUntil, dayCounter)
    class(val) <- c("AmericanOption","Option")
    val
}

BinaryOption <- function(binType, type, excType, underlying, strike, dividendYield,
                         riskFreeRate, maturity, volatility, cashPayoff, dayCounter=0) {
    UseMethod("BinaryOption")
}

BinaryOption.default <- function(binType, type, excType, underlying, strike, dividendYield,
                                 riskFreeRate, maturity, volatility, cashPayoff, dayCounter=0) {
    type <- match.arg(type, c("call", "put"))
    binType <- match.arg(binType, c("cash", "asset", "gap"))
    excType <- match.arg(excType, c("american", "european"))
    val <- binaryOptionEngine(binType, type, excType, underlying,
                              strike, dividendYield, riskFreeRate,
                              if (inherits(maturity, "numeric")) maturity else NULL,
                              if (inherits(maturity, "Date")) maturity else NULL,
                              volatility, cashPayoff, dayCounter)
    class(val) <- c("BinaryOption", "Option")
    val
}

BarrierOption <- function(barrType, type, underlying, strike,
                          dividendYield, riskFreeRate, maturity,
                          volatility, barrier, rebate=0.0, dayCounter=0) {
    UseMethod("BarrierOption")
}

BarrierOption.default <- function(barrType, type, underlying, strike,
                                  dividendYield, riskFreeRate, maturity,
                                  volatility, barrier, rebate=0.0, dayCounter=0) {
    type <- match.arg(type, c("call", "put"))
    barrType <- match.arg(barrType, c("downin", "upin", "downout", "upout"))
    val <- barrierOptionEngine(barrType, type, underlying, strike, dividendYield, riskFreeRate,
                               if (inherits(maturity, "numeric")) maturity else NULL,
                               if (inherits(maturity, "Date")) maturity else NULL,
                               volatility, barrier, rebate, dayCounter)
    class(val) <- c("BarrierOption", "Option")
    val
}

plot.Option <- function(x, ...) {
    warning("No plotting available for class", class(x)[1],"\n")
    invisible(x)
}

print.Option <- function(x, digits=4, ...) {
    cat("Concise summary of valuation for", class(x)[1], "\n")
    print(round(unlist(x[1:7]), digits))
    invisible(x)
}

summary.Option <- function(object, digits=4, ...) {
    cat("Detailed summary of valuation for", class(object)[1], "\n")
    print(round(unlist(object[1:7]), digits))
    cat("with parameters\n")
    print(unlist(object[["parameters"]]))
    invisible(object)
}

library(RQuantLib)

## AsianOption() currently has issue of Windows
.onWindows <- .Platform$OS.type == "windows"

## American option tests based on Haug's book
#test.american.Haug <- function() {
## see QuantLib's test-suite/americanoption.cpp
#/* The data below are from
#   "Option pricing formulas", E.G. Haug, McGraw-Hill 1998
#   pag 24
#
#   The following values were replicated only up to the second digit
#   by the VB code provided by Haug, which was used as base for the
#   C++ implementation
#
#*/

AO <- AmericanOption                # shorthand
expect_equal(AO(type="call", strike= 100.00, underl= 90.00,  div= 0.10, riskF=0.10,  mat=0.10,  vol= 0.15)$value,  0.0206, tol=1.0e-2)
expect_equal(AO(type="call", strike= 100.00, underl= 100.00, div= 0.10, riskF= 0.10, mat= 0.10, vol= 0.15)$value,  1.8771, tol=1.0e-4)
expect_equal(AO(type="call", strike= 100.00, underl= 110.00, div= 0.10, riskF= 0.10, mat= 0.10, vol= 0.15)$value, 10.0089, tol=1.0e-3)
expect_equal(AO(type="call", strike= 100.00, underl=  90.00, div= 0.10, riskF= 0.10, mat= 0.10, vol= 0.25)$value,  0.3159, tol=1.0e-3)
expect_equal(AO(type="call", strike= 100.00, underl= 100.00, div= 0.10, riskF= 0.10, mat= 0.10, vol= 0.25)$value,  3.1280, tol=1.0e-3)
expect_equal(AO(type="call", strike= 100.00, underl= 110.00, div= 0.10, riskF= 0.10, mat= 0.10, vol= 0.25)$value, 10.3919, tol=1.0e-3)
expect_equal(AO(type="call", strike= 100.00, underl=  90.00, div= 0.10, riskF= 0.10, mat= 0.10, vol= 0.35)$value,  0.9495, tol=1.0e-3)
expect_equal(AO(type="call", strike= 100.00, underl= 100.00, div= 0.10, riskF= 0.10, mat= 0.10, vol= 0.35)$value,  4.3777, tol=1.0e-3)
expect_equal(AO(type="call", strike= 100.00, underl= 110.00, div= 0.10, riskF= 0.10, mat= 0.10, vol= 0.35)$value, 11.1679, tol=1.0e-3)
expect_equal(AO(type="call", strike= 100.00, underl=  90.00, div= 0.10, riskF= 0.10, mat= 0.50, vol= 0.15)$value,  0.8208, tol=1.0e-3)
expect_equal(AO(type="call", strike= 100.00, underl= 100.00, div= 0.10, riskF= 0.10, mat= 0.50, vol= 0.15)$value,  4.0842, tol=1.0e-3)
expect_equal(AO(type="call", strike= 100.00, underl= 110.00, div= 0.10, riskF= 0.10, mat= 0.50, vol= 0.15)$value, 10.8087, tol=1.0e-3)
expect_equal(AO(type="call", strike= 100.00, underl=  90.00, div= 0.10, riskF= 0.10, mat= 0.50, vol= 0.25)$value,  2.7437, tol=1.0e-3)
expect_equal(AO(type="call", strike= 100.00, underl= 100.00, div= 0.10, riskF= 0.10, mat= 0.50, vol= 0.25)$value,  6.8015, tol=1.0e-3)
expect_equal(AO(type="call", strike= 100.00, underl= 110.00, div= 0.10, riskF= 0.10, mat= 0.50, vol= 0.25)$value, 13.0170, tol=1.0e-3)
expect_equal(AO(type="call", strike= 100.00, underl=  90.00, div= 0.10, riskF= 0.10, mat= 0.50, vol= 0.35)$value,  5.0063, tol=1.0e-3)
expect_equal(AO(type="call", strike= 100.00, underl= 100.00, div= 0.10, riskF= 0.10, mat= 0.50, vol= 0.35)$value,  9.5106, tol=1.0e-3)
expect_equal(AO(type="call", strike= 100.00, underl= 110.00, div= 0.10, riskF= 0.10, mat= 0.50, vol= 0.35)$value, 15.5689, tol=1.0e-3)
expect_equal(AO(type="put", strike=  100.00, underl=  90.00, div= 0.10, riskF= 0.10, mat= 0.10, vol= 0.15)$value, 10.0000, tol=1.0e-3)
expect_equal(AO(type="put", strike=  100.00, underl= 100.00, div= 0.10, riskF= 0.10, mat= 0.10, vol= 0.15)$value,  1.8770, tol=1.0e-3)
expect_equal(AO(type="put", strike=  100.00, underl= 110.00, div= 0.10, riskF= 0.10, mat= 0.10, vol= 0.15)$value,  0.0410, tol=1.0e-3)
expect_equal(AO(type="put", strike=  100.00, underl=  90.00, div= 0.10, riskF= 0.10, mat= 0.10, vol= 0.25)$value, 10.2533, tol=1.0e-3)
expect_equal(AO(type="put", strike=  100.00, underl= 100.00, div= 0.10, riskF= 0.10, mat= 0.10, vol= 0.25)$value,  3.1277, tol=1.0e-3)
expect_equal(AO(type="put", strike=  100.00, underl= 110.00, div= 0.10, riskF= 0.10, mat= 0.10, vol= 0.25)$value,  0.4562, tol=1.0e-3)
expect_equal(AO(type="put", strike=  100.00, underl=  90.00, div= 0.10, riskF= 0.10, mat= 0.10, vol= 0.35)$value, 10.8787, tol=1.0e-3)
expect_equal(AO(type="put", strike=  100.00, underl= 100.00, div= 0.10, riskF= 0.10, mat= 0.10, vol= 0.35)$value,  4.3777, tol=1.0e-3)
expect_equal(AO(type="put", strike=  100.00, underl= 110.00, div= 0.10, riskF= 0.10, mat= 0.10, vol= 0.35)$value,  1.2402, tol=1.0e-3)
expect_equal(AO(type="put", strike=  100.00, underl=  90.00, div= 0.10, riskF= 0.10, mat= 0.50, vol= 0.15)$value, 10.5595, tol=1.0e-3)
expect_equal(AO(type="put", strike=  100.00, underl= 100.00, div= 0.10, riskF= 0.10, mat= 0.50, vol= 0.15)$value,  4.0842, tol=1.0e-3)
expect_equal(AO(type="put", strike=  100.00, underl= 110.00, div= 0.10, riskF= 0.10, mat= 0.50, vol= 0.15)$value,  1.0822, tol=1.0e-3)
expect_equal(AO(type="put", strike=  100.00, underl=  90.00, div= 0.10, riskF= 0.10, mat= 0.50, vol= 0.25)$value, 12.4419, tol=1.0e-3)
expect_equal(AO(type="put", strike=  100.00, underl= 100.00, div= 0.10, riskF= 0.10, mat= 0.50, vol= 0.25)$value,  6.8014, tol=1.0e-3)
expect_equal(AO(type="put", strike=  100.00, underl= 110.00, div= 0.10, riskF= 0.10, mat= 0.50, vol= 0.25)$value,  3.3226, tol=1.0e-3)
expect_equal(AO(type="put", strike=  100.00, underl=  90.00, div= 0.10, riskF= 0.10, mat= 0.50, vol= 0.35)$value, 14.6945, tol=1.0e-3)
expect_equal(AO(type="put", strike=  100.00, underl= 100.00, div= 0.10, riskF= 0.10, mat= 0.50, vol= 0.35)$value,  9.5104, tol=1.0e-3)
expect_equal(AO(type="put", strike=  100.00, underl= 110.00, div= 0.10, riskF= 0.10, mat= 0.50, vol= 0.35)$value,  5.8823, tol=1.0e-3)


## Barrier Options
#test.barrier <- function() {
expect_equal(BarrierOption(barrType="downin", type="call", underl=100, strike=100, div=0.02, riskF=0.03, mat=0.5, vol=0.4, barrier=90)$value, 3.738254414)
expect_equal(BarrierOption("downout", barrier=95, rebate=3, type="call", strike=90, underlying=100, div=0.04, riskF=0.08, mat=0.5, vol=0.25)$value, 9.024567695)
expect_equal(BarrierOption("downin", barrier=95, rebate=3, type="call", strike=90, underlying=100, div=0.04, riskF=0.08, mat=0.5, vol=0.25)$value, 7.76267021)


#test.barrier.Haug <- function() {
## see QuantLib's test-suite/barrieroption.cpp
#    /* The data below are from
#     * "Option pricing formulas", E.G. Haug, McGraw-Hill 1998 pag. 72
#     */
#
BO <- BarrierOption                 # shorthand
expect_equal(BO(barrType="downout", barrier=95.0,  rebate=3.0, type="call", strike=90,  underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  9.0246, tol=1.0e-4)
expect_equal(BO(barrType="downout", barrier=95.0,  rebate=3.0, type="call", strike=100, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  6.7924, tol=1.0e-4)
expect_equal(BO(barrType="downout", barrier=95.0,  rebate=3.0, type="call", strike=110, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  4.8759, tol=1.0e-4)
expect_equal(BO(barrType="downout", barrier=100.0, rebate=3.0, type="call", strike=90,  underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  3.0000, tol=1.0e-4)
expect_equal(BO(barrType="downout", barrier=100.0, rebate=3.0, type="call", strike=100, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  3.0000, tol=1.0e-4)
expect_equal(BO(barrType="downout", barrier=100.0, rebate=3.0, type="call", strike=110, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  3.0000, tol=1.0e-4)
expect_equal(BO(barrType="upout",   barrier=105.0, rebate=3.0, type="call", strike=90,  underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  2.6789, tol=1.0e-4)
expect_equal(BO(barrType="upout",   barrier=105.0, rebate=3.0, type="call", strike=100, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  2.3580, tol=1.0e-4)
expect_equal(BO(barrType="upout",   barrier=105.0, rebate=3.0, type="call", strike=110, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  2.3453, tol=1.0e-4)
expect_equal(BO(barrType="downin",  barrier=95.0,  rebate=3.0, type="call", strike=90,  underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  7.7627, tol=1.0e-4)
expect_equal(BO(barrType="downin",  barrier=95.0,  rebate=3.0, type="call", strike=100, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  4.0109, tol=1.0e-4)
expect_equal(BO(barrType="downin",  barrier=95.0,  rebate=3.0, type="call", strike=110, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  2.0576, tol=1.0e-4)
expect_equal(BO(barrType="downin",  barrier=100.0, rebate=3.0, type="call", strike=90,  underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value, 13.8333, tol=1.0e-4)
expect_equal(BO(barrType="downin",  barrier=100.0, rebate=3.0, type="call", strike=100, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  7.8494, tol=1.0e-4)
expect_equal(BO(barrType="downin",  barrier=100.0, rebate=3.0, type="call", strike=110, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  3.9795, tol=1.0e-4)
expect_equal(BO(barrType="upin",    barrier=105.0, rebate=3.0, type="call", strike=90,  underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value, 14.1112, tol=1.0e-4)
expect_equal(BO(barrType="upin",    barrier=105.0, rebate=3.0, type="call", strike=100, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  8.4482, tol=1.0e-4)
expect_equal(BO(barrType="upin",    barrier=105.0, rebate=3.0, type="call", strike=110, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  4.5910, tol=1.0e-4)
expect_equal(BO(barrType="downout", barrier=95.0,  rebate=3.0, type="call", strike=90,  underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  8.8334, tol=1.0e-4)
expect_equal(BO(barrType="downout", barrier=95.0,  rebate=3.0, type="call", strike=100, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  7.0285, tol=1.0e-4)
expect_equal(BO(barrType="downout", barrier=95.0,  rebate=3.0, type="call", strike=110, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  5.4137, tol=1.0e-4)
expect_equal(BO(barrType="downout", barrier=100.0, rebate=3.0, type="call", strike=90,  underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  3.0000, tol=1.0e-4)
expect_equal(BO(barrType="downout", barrier=100.0, rebate=3.0, type="call", strike=100, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  3.0000, tol=1.0e-4)
expect_equal(BO(barrType="downout", barrier=100.0, rebate=3.0, type="call", strike=110, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  3.0000, tol=1.0e-4)
expect_equal(BO(barrType="upout",   barrier=105.0, rebate=3.0, type="call", strike=90,  underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  2.6341, tol=1.0e-4)
expect_equal(BO(barrType="upout",   barrier=105.0, rebate=3.0, type="call", strike=100, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  2.4389, tol=1.0e-4)
expect_equal(BO(barrType="upout",   barrier=105.0, rebate=3.0, type="call", strike=110, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  2.4315, tol=1.0e-4)
expect_equal(BO(barrType="downin",  barrier=95.0,  rebate=3.0, type="call", strike=90,  underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  9.0093, tol=1.0e-4)
expect_equal(BO(barrType="downin",  barrier=95.0,  rebate=3.0, type="call", strike=100, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  5.1370, tol=1.0e-4)
expect_equal(BO(barrType="downin",  barrier=95.0,  rebate=3.0, type="call", strike=110, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  2.8517, tol=1.0e-4)
expect_equal(BO(barrType="downin",  barrier=100.0, rebate=3.0, type="call", strike=90,  underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value, 14.8816, tol=1.0e-4)
expect_equal(BO(barrType="downin",  barrier=100.0, rebate=3.0, type="call", strike=100, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  9.2045, tol=1.0e-4)
expect_equal(BO(barrType="downin",  barrier=100.0, rebate=3.0, type="call", strike=110, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  5.3043, tol=1.0e-4)
expect_equal(BO(barrType="upin",    barrier=105.0, rebate=3.0, type="call", strike=90,  underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value, 15.2098, tol=1.0e-4)
expect_equal(BO(barrType="upin",    barrier=105.0, rebate=3.0, type="call", strike=100, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  9.7278, tol=1.0e-4)
expect_equal(BO(barrType="upin",    barrier=105.0, rebate=3.0, type="call", strike=110, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  5.8350, tol=1.0e-4)
expect_equal(BO(barrType="downout", barrier=95.0,  rebate=3.0, type="put",  strike=90,  underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  2.2798, tol=1.0e-4)
expect_equal(BO(barrType="downout", barrier=95.0,  rebate=3.0, type="put",  strike=100, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  2.2947, tol=1.0e-4)
expect_equal(BO(barrType="downout", barrier=95.0,  rebate=3.0, type="put",  strike=110, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  2.6252, tol=1.0e-4)
expect_equal(BO(barrType="downout", barrier=100.0, rebate=3.0, type="put",  strike=90,  underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  3.0000, tol=1.0e-4)
expect_equal(BO(barrType="downout", barrier=100.0, rebate=3.0, type="put",  strike=100, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  3.0000, tol=1.0e-4)
expect_equal(BO(barrType="downout", barrier=100.0, rebate=3.0, type="put",  strike=110, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  3.0000, tol=1.0e-4)
expect_equal(BO(barrType="upout",   barrier=105.0, rebate=3.0, type="put",  strike=90,  underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  3.7760, tol=1.0e-4)
expect_equal(BO(barrType="upout",   barrier=105.0, rebate=3.0, type="put",  strike=100, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  5.4932, tol=1.0e-4)
expect_equal(BO(barrType="upout",   barrier=105.0, rebate=3.0, type="put",  strike=110, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  7.5187, tol=1.0e-4)
expect_equal(BO(barrType="downin",  barrier=95.0,  rebate=3.0, type="put",  strike=90,  underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  2.9586, tol=1.0e-4)
expect_equal(BO(barrType="downin",  barrier=95.0,  rebate=3.0, type="put",  strike=100, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  6.5677, tol=1.0e-4)
expect_equal(BO(barrType="downin",  barrier=95.0,  rebate=3.0, type="put",  strike=110, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value, 11.9752, tol=1.0e-4)
expect_equal(BO(barrType="downin",  barrier=100.0, rebate=3.0, type="put",  strike=90,  underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  2.2845, tol=1.0e-4)
expect_equal(BO(barrType="downin",  barrier=100.0, rebate=3.0, type="put",  strike=100, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  5.9085, tol=1.0e-4)
expect_equal(BO(barrType="downin",  barrier=100.0, rebate=3.0, type="put",  strike=110, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value, 11.6465, tol=1.0e-4)
expect_equal(BO(barrType="upin",    barrier=105.0, rebate=3.0, type="put",  strike=90,  underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  1.4653, tol=1.0e-4)
expect_equal(BO(barrType="upin",    barrier=105.0, rebate=3.0, type="put",  strike=100, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  3.3721, tol=1.0e-4)
expect_equal(BO(barrType="upin",    barrier=105.0, rebate=3.0, type="put",  strike=110, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.25)$value,  7.0846, tol=1.0e-4)
expect_equal(BO(barrType="downout", barrier=95.0,  rebate=3.0, type="put",  strike=90,  underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  2.4170, tol=1.0e-4)
expect_equal(BO(barrType="downout", barrier=95.0,  rebate=3.0, type="put",  strike=100, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  2.4258, tol=1.0e-4)
expect_equal(BO(barrType="downout", barrier=95.0,  rebate=3.0, type="put",  strike=110, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  2.6246, tol=1.0e-4)
expect_equal(BO(barrType="downout", barrier=100.0, rebate=3.0, type="put",  strike=90,  underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  3.0000, tol=1.0e-4)
expect_equal(BO(barrType="downout", barrier=100.0, rebate=3.0, type="put",  strike=100, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  3.0000, tol=1.0e-4)
expect_equal(BO(barrType="downout", barrier=100.0, rebate=3.0, type="put",  strike=110, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  3.0000, tol=1.0e-4)
expect_equal(BO(barrType="upout",   barrier=105.0, rebate=3.0, type="put",  strike=90,  underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  4.2293, tol=1.0e-4)
expect_equal(BO(barrType="upout",   barrier=105.0, rebate=3.0, type="put",  strike=100, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  5.8032, tol=1.0e-4)
expect_equal(BO(barrType="upout",   barrier=105.0, rebate=3.0, type="put",  strike=110, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  7.5649, tol=1.0e-4)
expect_equal(BO(barrType="downin",  barrier=95.0,  rebate=3.0, type="put",  strike=90,  underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  3.8769, tol=1.0e-4)
expect_equal(BO(barrType="downin",  barrier=95.0,  rebate=3.0, type="put",  strike=100, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  7.7989, tol=1.0e-4)
expect_equal(BO(barrType="downin",  barrier=95.0,  rebate=3.0, type="put",  strike=110, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value, 13.3078, tol=1.0e-4)
expect_equal(BO(barrType="downin",  barrier=100.0, rebate=3.0, type="put",  strike=90,  underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  3.3328, tol=1.0e-4)
expect_equal(BO(barrType="downin",  barrier=100.0, rebate=3.0, type="put",  strike=100, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  7.2636, tol=1.0e-4)
expect_equal(BO(barrType="downin",  barrier=100.0, rebate=3.0, type="put",  strike=110, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value, 12.9713, tol=1.0e-4)
expect_equal(BO(barrType="upin",    barrier=105.0, rebate=3.0, type="put",  strike=90,  underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  2.0658, tol=1.0e-4)
expect_equal(BO(barrType="upin",    barrier=105.0, rebate=3.0, type="put",  strike=100, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  4.4226, tol=1.0e-4)
expect_equal(BO(barrType="upin",    barrier=105.0, rebate=3.0, type="put",  strike=110, underl=100.0, div=0.04, riskF=0.08, mat=0.50, vol=0.30)$value,  8.3686, tol=1.0e-4)


## Binary Options aka Digitals
#test.binary <- function() {
## cash or nothing European
rc <- BinaryOption(binType="cash", type="put", excType="european", strike=80, underl=100, div=0.06, r=0.06, mat=0.75, vol=0.35, cash=10)
expect_equal(rc$value, 2.671045684)
expect_equal(rc[1:7],
            list(value=2.671045, delta=-0.1060594, gamma=0.00310624, vega=8.153881, theta=-1.742309, rho=-9.9577, divRho=7.9544),
            tol=1.0e-5)


## European option tests based on Haug's book
#test.european.Haug <- function() {
## see QuantLib's test-suite/europeanoption.cpp
#/* The data below are from
#   "Option pricing formulas", E.G. Haug, McGraw-Hill 1998
#*/
#  // pag 2-8, pg 24, p 27
#
Lines <- "type  strike  spot  q  r  t  v  value tol"
EO <- EuropeanOption                # shorthand
expect_equal(EO(type="call", strike=65.00,  underl=60.00,  div= 0.00, riskF=0.08, mat=0.25, vol=0.30)$value,  2.1334, tol=1.0e-3)
expect_equal(EO(type="put",  strike=95.00,  underl=100.00, div= 0.05, riskF=0.10, mat=0.50, vol=0.20)$value,  2.4648, tol=1.0e-3)
expect_equal(EO(type="put",  strike=19.00,  underl=19.00,  div= 0.10, riskF=0.10, mat=0.75, vol=0.28)$value,  1.7011, tol=1.0e-3)
expect_equal(EO(type="call", strike=19.00,  underl=19.00,  div= 0.10, riskF=0.10, mat=0.75, vol=0.28)$value,  1.7011, tol=1.0e-3)
expect_equal(EO(type="call", strike=1.60,   underl=1.56,   div= 0.08, riskF=0.06, mat=0.50, vol=0.12)$value,  0.0291, tol=1.0e-3)
expect_equal(EO(type="put",  strike=70.00,  underl=75.00,  div= 0.05, riskF=0.10, mat=0.50, vol=0.35)$value,  4.0870, tol=1.0e-3)
expect_equal(EO(type="call", strike=100.00, underl=90.00,  div= 0.10, riskF=0.10, mat=0.10, vol=0.15)$value,  0.0205, tol=1.0e-3)
expect_equal(EO(type="call", strike=100.00, underl=100.00, div= 0.10, riskF=0.10, mat=0.10, vol=0.15)$value,  1.8734, tol=1.0e-3)
expect_equal(EO(type="call", strike=100.00, underl=110.00, div= 0.10, riskF=0.10, mat=0.10, vol=0.15)$value,  9.9413, tol=1.0e-3)
expect_equal(EO(type="call", strike=100.00, underl=90.00,  div= 0.10, riskF=0.10, mat=0.10, vol=0.25)$value,  0.3150, tol=1.0e-3)
expect_equal(EO(type="call", strike=100.00, underl=100.00, div= 0.10, riskF=0.10, mat=0.10, vol=0.25)$value,  3.1217, tol=1.0e-3)
expect_equal(EO(type="call", strike=100.00, underl=110.00, div= 0.10, riskF=0.10, mat=0.10, vol=0.25)$value, 10.3556, tol=1.0e-3)
expect_equal(EO(type="call", strike=100.00, underl=90.00,  div= 0.10, riskF=0.10, mat=0.10, vol=0.35)$value,  0.9474, tol=1.0e-3)
expect_equal(EO(type="call", strike=100.00, underl=100.00, div= 0.10, riskF=0.10, mat=0.10, vol=0.35)$value,  4.3693, tol=1.0e-3)
expect_equal(EO(type="call", strike=100.00, underl=110.00, div= 0.10, riskF=0.10, mat=0.10, vol=0.35)$value, 11.1381, tol=1.0e-3)
expect_equal(EO(type="call", strike=100.00, underl=90.00,  div= 0.10, riskF=0.10, mat=0.50, vol=0.15)$value,  0.8069, tol=1.0e-3)
expect_equal(EO(type="call", strike=100.00, underl=100.00, div= 0.10, riskF=0.10, mat=0.50, vol=0.15)$value,  4.0232, tol=1.0e-3)
expect_equal(EO(type="call", strike=100.00, underl=110.00, div= 0.10, riskF=0.10, mat=0.50, vol=0.15)$value, 10.5769, tol=1.0e-3)
expect_equal(EO(type="call", strike=100.00, underl=90.00,  div= 0.10, riskF=0.10, mat=0.50, vol=0.25)$value,  2.7026, tol=1.0e-3)
expect_equal(EO(type="call", strike=100.00, underl=100.00, div= 0.10, riskF=0.10, mat=0.50, vol=0.25)$value,  6.6997, tol=1.0e-3)
expect_equal(EO(type="call", strike=100.00, underl=110.00, div= 0.10, riskF=0.10, mat=0.50, vol=0.25)$value, 12.7857, tol=1.0e-3)
expect_equal(EO(type="call", strike=100.00, underl=90.00,  div= 0.10, riskF=0.10, mat=0.50, vol=0.35)$value,  4.9329, tol=1.0e-3)
expect_equal(EO(type="call", strike=100.00, underl=100.00, div= 0.10, riskF=0.10, mat=0.50, vol=0.35)$value,  9.3679, tol=1.0e-3)
expect_equal(EO(type="call", strike=100.00, underl=110.00, div= 0.10, riskF=0.10, mat=0.50, vol=0.35)$value, 15.3086, tol=1.0e-3)
expect_equal(EO(type="put",  strike=100.00, underl=90.00,  div= 0.10, riskF=0.10, mat=0.10, vol=0.15)$value,  9.9210, tol=1.0e-3)
expect_equal(EO(type="put",  strike=100.00, underl=100.00, div= 0.10, riskF=0.10, mat=0.10, vol=0.15)$value,  1.8734, tol=1.0e-3)
expect_equal(EO(type="put",  strike=100.00, underl=110.00, div= 0.10, riskF=0.10, mat=0.10, vol=0.15)$value,  0.0408, tol=1.0e-3)
expect_equal(EO(type="put",  strike=100.00, underl=90.00,  div= 0.10, riskF=0.10, mat=0.10, vol=0.25)$value, 10.2155, tol=1.0e-3)
expect_equal(EO(type="put",  strike=100.00, underl=100.00, div= 0.10, riskF=0.10, mat=0.10, vol=0.25)$value,  3.1217, tol=1.0e-3)
expect_equal(EO(type="put",  strike=100.00, underl=110.00, div= 0.10, riskF=0.10, mat=0.10, vol=0.25)$value,  0.4551, tol=1.0e-3)
expect_equal(EO(type="put",  strike=100.00, underl=90.00,  div= 0.10, riskF=0.10, mat=0.10, vol=0.35)$value, 10.8479, tol=1.0e-3)
expect_equal(EO(type="put",  strike=100.00, underl=100.00, div= 0.10, riskF=0.10, mat=0.10, vol=0.35)$value,  4.3693, tol=1.0e-3)
expect_equal(EO(type="put",  strike=100.00, underl=110.00, div= 0.10, riskF=0.10, mat=0.10, vol=0.35)$value,  1.2376, tol=1.0e-3)
expect_equal(EO(type="put",  strike=100.00, underl=90.00,  div= 0.10, riskF=0.10, mat=0.50, vol=0.15)$value, 10.3192, tol=1.0e-3)
expect_equal(EO(type="put",  strike=100.00, underl=100.00, div= 0.10, riskF=0.10, mat=0.50, vol=0.15)$value,  4.0232, tol=1.0e-3)
expect_equal(EO(type="put",  strike=100.00, underl=110.00, div= 0.10, riskF=0.10, mat=0.50, vol=0.15)$value,  1.0646, tol=1.0e-3)
expect_equal(EO(type="put",  strike=100.00, underl=90.00,  div= 0.10, riskF=0.10, mat=0.50, vol=0.25)$value, 12.2149, tol=1.0e-3)
expect_equal(EO(type="put",  strike=100.00, underl=100.00, div= 0.10, riskF=0.10, mat=0.50, vol=0.25)$value,  6.6997, tol=1.0e-3)
expect_equal(EO(type="put",  strike=100.00, underl=110.00, div= 0.10, riskF=0.10, mat=0.50, vol=0.25)$value,  3.2734, tol=1.0e-3)
expect_equal(EO(type="put",  strike=100.00, underl=90.00,  div= 0.10, riskF=0.10, mat=0.50, vol=0.35)$value, 14.4452, tol=1.0e-3)
expect_equal(EO(type="put",  strike=100.00, underl=100.00, div= 0.10, riskF=0.10, mat=0.50, vol=0.35)$value,  9.3679, tol=1.0e-3)
expect_equal(EO(type="put",  strike=100.00, underl=110.00, div= 0.10, riskF=0.10, mat=0.50, vol=0.35)$value,  5.7963, tol=1.0e-3)
expect_equal(EO(type="call", strike=40.00,  underl=42.00,  div= 0.08, riskF=0.04, mat=0.75, vol=0.35)$value,  5.0975, tol=1.0e-3)

## European and American option tests with discrete dividends based on Hull's book
#test.discreteDividend.Hull <- function() {
AO <- AmericanOption                # shorthand
EO <- EuropeanOption                # shorthand
## Reference p. 253 - Hull 5th ed. Exercise 12.8 - From QuantLib tests
expect_equal(EO(type="call", underlying=40, strike=40, div=0, riskFree=0.09, maturity=0.5, vol=0.3,
               discreteDividends = c(0.5, 0.5), discreteDividendsTimeUntil = c(2/12, 5/12))$value, 3.67, tol=1.0e-2)
## Reference p. 256 - Hull 5th ed. Exercise 12.9 using (flawed) Roll, Geske and Whaley formula
expect_equal(AO(type="call", underlying=40, strike=40, div=0, riskFree=0.09, maturity=0.5, vol=0.3, engine = "CrankNicolson",
               discreteDividends = c(0.5, 0.5), discreteDividendsTimeUntil = c(2/12, 5/12))$value, 3.72, tol=1.0e-1)

## Asian option tests
#test.asian <- function() {
## see QuantLib's test-suite/asianoptions.cpp

AO <- AsianOption   	             # shorthand

expect_equal(AO(averageType="geometric", typ="put", strike=85, underl=80, div=-0.03, riskF=0.05, mat=0.25, vol=0.20)$value, 4.6922, tol=1.0e-4)

if (.onWindows) exit_file("Skipping Asian options on Windows.")

## data from "Asian Option", Levy, 1997
## in "Exotic Options: The State of the Art",
## edited by Clewlow, Strickland
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=0.0, length=11.0/12.0, fixings=2, vol=0.13)$value, 1.3942835683, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=0.0, length=11.0/12.0, fixings=4, vol=0.13)$value, 1.5852442983, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=0.0, length=11.0/12.0, fixings=8, vol=0.13)$value, 1.66970673, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=0.0, length=11.0/12.0, fixings=12, vol=0.13)$value, 1.6980019214, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=0.0, length=11.0/12.0, fixings=26, vol=0.13)$value, 1.7255070456, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=0.0, length=11.0/12.0, fixings=52, vol=0.13)$value, 1.7401553533, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=0.0, length=11.0/12.0, fixings=100, vol=0.13)$value, 1.7478303712, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=0.0, length=11.0/12.0, fixings=250, vol=0.13)$value, 1.7490291943, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=0.0, length=11.0/12.0, fixings=500, vol=0.13)$value, 1.7515113291, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=0.0, length=11.0/12.0, fixings=1000, vol=0.13)$value, 1.7537344885, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=1.0/12.0, length=11.0/12.0, fixings=2, vol=0.13)$value, 1.8496053697, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=1.0/12.0, length=11.0/12.0, fixings=4, vol=0.13)$value, 2.0111495205, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=1.0/12.0, length=11.0/12.0, fixings=8, vol=0.13)$value, 2.0852138818, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=1.0/12.0, length=11.0/12.0, fixings=12, vol=0.13)$value, 2.1105094397, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=1.0/12.0, length=11.0/12.0, fixings=26, vol=0.13)$value, 2.1346526695, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=1.0/12.0, length=11.0/12.0, fixings=52, vol=0.13)$value, 2.147489651, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=1.0/12.0, length=11.0/12.0, fixings=100, vol=0.13)$value, 2.154728109, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=1.0/12.0, length=11.0/12.0, fixings=250, vol=0.13)$value, 2.1564276565, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=1.0/12.0, length=11.0/12.0, fixings=500, vol=0.13)$value, 2.1594238588, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=1.0/12.0, length=11.0/12.0, fixings=1000, vol=0.13)$value, 2.1595367326, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=3.0/12.0, length=11.0/12.0, fixings=2, vol=0.13)$value, 2.63315092584, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=3.0/12.0, length=11.0/12.0, fixings=4, vol=0.13)$value, 2.76723962361, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=3.0/12.0, length=11.0/12.0, fixings=8, vol=0.13)$value, 2.83124836881, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=3.0/12.0, length=11.0/12.0, fixings=12, vol=0.13)$value, 2.84290301412, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=3.0/12.0, length=11.0/12.0, fixings=26, vol=0.13)$value, 2.88179560417, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=3.0/12.0, length=11.0/12.0, fixings=52, vol=0.13)$value, 2.88447044543, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=3.0/12.0, length=11.0/12.0, fixings=100, vol=0.13)$value, 2.89985329603, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=3.0/12.0, length=11.0/12.0, fixings=250, vol=0.13)$value, 2.90047296063, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=3.0/12.0, length=11.0/12.0, fixings=500, vol=0.13)$value, 2.89813412160, tol=1.0e-2)
expect_equal(AO("arithmetic", "put", underl=90.0, strike=87.0, div=0.06, riskF=0.025, first=3.0/12.0, length=11.0/12.0, fixings=1000, vol=0.13)$value, 2.8970336243, tol=1.0e-2)

## RQuantLib -- R interface to the QuantLib libraries
##
## Copyright (C) 2002 - 2021 Dirk Eddelbuettel
## Copyright (C) 2010        Dirk Eddelbuettel and Khanh Nguyen
##
## This file is part of the RQuantLib library for GNU R.
## It is made available under the terms of the GNU General Public
## License, version 2, or at your option, any later version,
## incorporated herein by reference.
##
## This program is distributed in the hope that it will be
## useful, but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with this program; if not, write to the Free
## Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
## MA 02111-1307, USA

calendars <- c("TARGET",
               "UnitedStates",
               "UnitedStates/LiborImpact",
               "UnitedStates/NYSE",
               "UnitedStates/GovernmentBond",
               "UnitedStates/NERC",
               "UnitedStates/FederalReserve",
               "Argentina",
               "Australia",
               "Austria",
               "Austria/Exchange",
               "Bespoke",
               "Botswana",
               "Brazil",
               "Brazil/Exchange",
               "Canada",
               "Canada/TSX",
               "Chile",
               "China",
               "China/IB",
               "CzechRepublic",
               "Denmark",
               "Finland",
               "France",
               "France/Exchange",
               "Germany",
               "Germany/FrankfurtStockExchange",
               "Germany/Xetra",
               "Germany/Eurex",
               "Germany/Euwax",
               "HongKong",
               "Hungary",
               "Iceland",
               "India",
               "Indonesia",
               "Israel",
               "Italy",
               "Italy/Exchange",
               "Japan",
               "Mexico",
               "NewZealand",
               "Norway",
               "Null",
               "Poland",
               "Romania",
               "Russia",
               "SaudiArabia",
               "Singapore",
               "Slovakia",
               "SouthAfrica",
               "SouthKorea",
               "SouthKorea/KRX",
               "Sweden",
               "Switzerland",
               "Taiwan",
               "Thailand",
               "Turkey",
               "Ukraine",
               "UnitedKingdom",
               "UnitedKingdom/Exchange",
               "UnitedKingdom/Metals",
               "WeekendsOnly")

## isBusinessDay <- function(calendar="TARGET", dates=Sys.Date()) {
##     stopifnot(is.character(calendar))
##     stopifnot(class(dates)=="Date")
##     val <- .Call("isBusinessDay", calendar, dates, PACKAGE="RQuantLib")
##     val <- as.logical(val)
##     names(val) <- dates
##     val
## }

businessDay <- function(calendar="TARGET", dates=Sys.Date()) {  ## may get deprecated one day
    isBusinessDay(calendar, dates)
}

## isHoliday <- function(calendar="TARGET", dates=Sys.Date()) {
##     stopifnot(is.character(calendar))
##     stopifnot(class(dates)=="Date")
##     val <- .Call("isHoliday", calendar, dates, PACKAGE="RQuantLib")
##     val <- as.logical(val)
##     names(val) <- dates
##     val
## }

## isWeekend <- function(calendar="TARGET", dates=Sys.Date()) {
##     stopifnot(is.character(calendar))
##     stopifnot(class(dates)=="Date")
##     val <- .Call("isWeekend", calendar, dates, PACKAGE="RQuantLib")
##     val <- as.logical(val)
##     names(val) <- dates
##     val
## }

## isEndOfMonth <- function(calendar="TARGET", dates=Sys.Date()) {
##     stopifnot(is.character(calendar))
##     stopifnot(class(dates)=="Date")
##     val <- .Call("isEndOfMonth", calendar, dates, PACKAGE="RQuantLib")
##     val <- as.logical(val)
##     names(val) <- dates
##     val
## }

## getEndOfMonth <- function(calendar="TARGET", dates=Sys.Date()) {
##     stopifnot(is.character(calendar))
##     stopifnot(class(dates)=="Date")
##     val <- .Call("endOfMonth", calendar, dates, PACKAGE="RQuantLib")
##     names(val) <- dates
##     val
## }

endOfMonth <- function(calendar="TARGET", dates=Sys.Date()) {
    getEndOfMonth(calendar, dates)
}

## adjust <- function(calendar="TARGET", dates=Sys.Date(), bdc = 0 ) {
##     stopifnot(is.character(calendar))
##     stopifnot(class(dates)=="Date")
##     val <- .Call("adjust", calendar, as.double(bdc), dates, PACKAGE="RQuantLib")
##     names(val) <- dates
##     val
## }

advance <- function(calendar="TARGET", dates=Sys.Date(),
                    n, timeUnit, # call 1
                    period,      # call 2
                    bdc = 0, emr = 0) {
    stopifnot(is.character(calendar))
    stopifnot(class(dates)=="Date")
    call1 <- missing(period) && !missing(n) && !missing(timeUnit)
    call2 <- !missing(period) && missing(n) && missing(timeUnit)
    stopifnot(call1 | call2)
    val <- NULL
    if (call1) {
        ## val <- .Call("advance1",
        ##              calendar,
        ##              list(amount = as.double(n),
        ##                   unit = as.double(timeUnit),
        ##                   bdc = as.double(bdc),
        ##                   emr = as.double(emr)),
        ##              dates,
        ##              PACKAGE="RQuantLib")
        val <- advance1(calendar, n, timeUnit, bdc, emr, dates)
    }
    if (call2) {
        ## val <- .Call("advance2",
        ##              calendar,
        ##              list(period = as.double(period),
        ##                   bdc = as.double(bdc),
        ##                   emr = as.double(emr)),
        ##              dates,
        ##              PACKAGE="RQuantLib")
        val <- advance2(calendar, period, bdc, emr, dates)
    }
    stopifnot( !is.null(val) )
    val
}

## businessDaysBetween  <- function(calendar="TARGET",
##                                  from=Sys.Date(),
##                                  to = Sys.Date() + 5,
##                                  includeFirst = 1,
##                                  includeLast = 0
##                                  ) {
##     stopifnot(is.character(calendar))
##     stopifnot(class(from)=="Date")
##     stopifnot(class(to)=="Date")
##     val <- .Call("businessDaysBetween",
##                  calendar,
##                  list(includeFirst = as.double(includeFirst),
##                       includeLast = as.double(includeLast)),
##                  from, to,
##                  PACKAGE="RQuantLib")
##     val <- val
##     val
## }

## getHolidayList <- function(calendar="TARGET",
##                            from=Sys.Date(),
##                            to=Sys.Date() + 5,
##                            includeWeekends=0) {
##     stopifnot(is.character(calendar))
##     stopifnot(class(from)=="Date")
##     stopifnot(class(to)=="Date")
##     val <- .Call("holidayList",
##                  calendar,
##                  list(includeWeekends=as.double(includeWeekends), from=from, to=to),
##                  PACKAGE="RQuantLib")
##     val
## }

holidayList <- function(calendar="TARGET", from=Sys.Date(), to=Sys.Date() + 5, includeWeekends=FALSE) {
    getHolidayList(calendar, from, to, includeWeekends)
}

businessDayList <- function(calendar="TARGET", from=Sys.Date(), to=Sys.Date() + 5) {
    getBusinessDayList(calendar, from, to)
}

#setCalendarContext <- function(calendar="TARGET",
#                               fixingDays = 2,
#                               settleDate = Sys.Date() + 2) {
#    val <- .Call("setContext",
#                 list(calendar = calendar,
#                      fixingDays = fixingDays,
#                      settleDate = settleDate),
#                 PACKAGE="RQuantLib")
#}

library(RQuantLib)

#test.Nearest <- function(){
expect_equal(advance("SouthAfrica", as.Date("2015-02-03"), 1, 2, bdc=6), as.Date("2015-03-03"))
expect_equal(advance("SouthAfrica", as.Date("2015-02-03"), 4, 0, bdc=6), as.Date("2015-02-09"))
expect_equal(advance("SouthAfrica", as.Date("2015-04-16"), 1, 2, bdc=6), as.Date("2015-05-15"))
expect_equal(advance("SouthAfrica", as.Date("2015-04-17"), 1, 2, bdc=6), as.Date("2015-05-18"))
expect_equal(advance("SouthAfrica", as.Date("2015-03-04"), 1, 2, bdc=6), as.Date("2015-04-02"))
expect_equal(advance("SouthAfrica", as.Date("2015-04-02"), 1, 2, bdc=6), as.Date("2015-05-04"))


#test.HalfMonthModifiedFollowing <- function() {
expect_equal(advance("SouthAfrica", as.Date("2015-02-03"), 1, 2, bdc=5), as.Date("2015-03-03"))
expect_equal(advance("SouthAfrica", as.Date("2015-02-03"), 4, 0, bdc=5), as.Date("2015-02-09"))
expect_equal(advance("SouthAfrica", as.Date("2015-01-31"), 1, 2, emr=1, bdc=5), as.Date("2015-02-27"))
expect_equal(advance("SouthAfrica", as.Date("2015-01-31"), 1, 2, bdc=5), as.Date("2015-02-27"))
expect_equal(advance("SouthAfrica", as.Date("2015-01-3"), 1, 1, bdc=5), as.Date("2015-01-12"))
expect_equal(advance("SouthAfrica", as.Date("2015-03-21"), 1, 1, bdc=5), as.Date("2015-03-30"))
expect_equal(advance("SouthAfrica", as.Date("2015-02-07"), 1, 2, bdc=5), as.Date("2015-03-09"))

## RQuantLib -- R interface to the QuantLib libraries
##
## Copyright (C) 2002 - 2024  Dirk Eddelbuettel <edd@debian.org>
##
## This file is part of the RQuantLib library for GNU R.
## It is made available under the terms of the GNU General Public
## License, version 2, or at your option, any later version,
## incorporated herein by reference.
##
## This program is distributed in the hope that it will be
## useful, but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with this program; if not, write to the Free
## Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
## MA 02111-1307, USA

#.First.lib <- function(lib, pkg) {
#  #cat("This is a pre-release. The interface might change...\n")
#  library.dynam("RQuantLib", pkg, lib )
#}

.onAttach <- function(libname, pkgname) {

    ## if it is not interactive (as eg in testing or cronjobs), do nothing
    if (!interactive()) return(invisible(NULL))

    ## else let's test the QuantLib version, and then the intraday capability
    qlver <- gsub("-[a-z]+", "", getQuantLibVersion())
    if (compareVersion(qlver, "1.7") < 0) {
        packageStartupMessage("QuantLib version ", qlver, " detected which is older than 1.7.")
        packageStartupMessage("Intra-daily options analytics unavailable with that version.")
    } else if (!getQuantLibCapabilities()[["intradayDate"]]) {
        packageStartupMessage("Sufficient QuantLib version with insufficient configuration.")
        packageStartupMessage("Try configuring your build with the --enable-intraday argument.")
    }
    packageStartupMessage("RQuantLib ", packageVersion("RQuantLib"), " built with QuantLib version ",
                          qlver, ". See https://www.quantlib.org for more on QuantLib.")
}

stopifnot(require(RQuantLib))

## values from Quantlib's test-suite
## Reference: Haug, Option Pricing Formulas, McGraw-Hill, 1998
##
## and generally sourced from the code in the test-suite/
## directory of the QuantLib distribution

## europeanoption.cpp:  call value == 2.1334
print(EuropeanOption("call", underlying=60, strike=65, div=0, riskFree=0.08,
                     maturity=0.25, vol=0.3), digits=5)
## europeanoption.cpp:  put value == 2.4648
print(EuropeanOption("put", underlying=100, strike=95, div=0.05, riskFree=0.1,
                     maturity=0.5, vol=0.2), digits=5)

## europeanoption.cpp:  call delta == 0.5946
print(EuropeanOption("call", underlying=105, strike=100,div=0.1,riskFree=0.1,
                     maturity=0.5, vol=0.36), digits=4)
## europeanoption.cpp:  put delta == -0.3566
print(EuropeanOption("put", underlying=105, strike=100,div=0.1,riskFree=0.1,
                     maturity=0.5, vol=0.36), digits=4)

## europeanoption.cpp:  call gamma == 0.0278
print(EuropeanOption("call", underlying=55, strike=60,div=0.0,riskFree=0.1,
                     maturity=0.75, vol=0.30), digits=4)
## europeanoption.cpp:  put gamma == 0.0278
print(EuropeanOption("put", underlying=55, strike=60,div=0.0,riskFree=0.1,
                     maturity=0.75, vol=0.30), digits=4)

## europeanoption.cpp:  call vega == 18.9358
print(EuropeanOption("call", underlying=55, strike=60,div=0.0,riskFree=0.1,
                     maturity=0.75, vol=0.30), digits=4)
## europeanoption.cpp:  put vega == 18.9358
print(EuropeanOption("put", underlying=55, strike=60,div=0.0,riskFree=0.1,
                     maturity=0.75, vol=0.30), digits=4)


## americanoption.cpp:  call value == 10.0089 -- we show 10.00606
print(AmericanOption("call", underlying=110, strike=100, div=0.1, riskFree=0.1,
                     maturity=0.1, vol=0.15), digits=5)
## americanoption.cpp:  put value == 0.3159
print(AmericanOption("call", underlying=90, strike=100, div=0.1, riskFree=0.1,
                     maturity=0.1, vol=0.25), digits=5)


# Discrete dividend
## europeanoption.cpp:  call value ==  3.67
## Reference pg. 253 - Hull 5th ed Exercise 12.8 - From QuantLib tests
print(EuropeanOption("call", underlying=40, strike=40, div=0, riskFree=0.09,
                     maturity=0.5, vol=0.3,
                     discreteDividends = c(0.5, 0.5),
                     discreteDividendsTimeUntil = c(2/12, 5/12)), digits=5)

## americanoption.cpp:  call value == 3.72 (Hull) -- we show 3.75
## Reference p. 256 - Hull 5th ed. Exercise 12.9 using (flawed) Roll, Geske, Whaley formula
print(AmericanOption("call", underlying=40, strike=40, div=0, riskFree=0.09,
                     maturity=0.5, vol=0.3,
                     engine = "CrankNicolson",
                     discreteDividends = c(0.5, 0.5),
                     discreteDividendsTimeUntil = c(2/12, 5/12)), digits=5)

## barrier: down and out call == 9.0246
print(BarrierOption("downout", barrier=95, rebate=3, type="call",
                    strike=90, underlying=100, div=0.04, riskF=0.08,
                    mat=0.5, vol=0.25), digits=4)
## barrier: down and in call == 7.7627
print(BarrierOption("downin", barrier=95, rebate=3, type="call",
                    strike=90, underlying=100, div=0.04, riskF=0.08,
                    mat=0.5, vol=0.25), digits=4)


## binary aka digital: put == 2.6710
print(BinaryOption(binType="cash", type="put", excType="european",
                   strike=80, underl=100, div=0.06, r=0.06,
                   mat=0.75, vol=0.35, cash=10), digits=4)

## asianoption.cpp:  put == 4.6922 (from testAnalyticContinuousGeometricAveragePrice())
print( AsianOption("geometric", "put", underlying=80, strike=85, div=-0.03, riskFree=0.05, maturity=0.25, vol=0.2))

#.onWindows <- .Platform$OS.type == "windows"

## simple call with unnamed parameters

bond <- list(faceAmount=100,issueDate=as.Date("2004-11-30"),
             maturityDate=as.Date("2008-11-30"), redemption=100 )

dateparams <-list(settlementDays=1, calendar="UnitedStates/GovernmentBond", businessDayConvention='Unadjusted')


discountCurve.param <- list(tradeDate=as.Date('2002-2-15'),
                            settleDate=as.Date('2002-2-15'),
                            dt=0.25,
                            interpWhat='discount', interpHow='loglinear')
discountCurve <- DiscountCurve(discountCurve.param, list(flat=0.05))

ZeroCouponBond(bond, discountCurve, dateparams)


## bond.cpp: examples from Fixed Income page of Matlab
ZeroYield(95, 100, as.Date("1993-6-24"), as.Date("1993-11-1"))

## bond.cpp: test theoretical price of bond by its yield
ZeroPriceByYield(0.1478, 100, as.Date("1993-6-24"), as.Date("1993-11-1"))

## bond.cpp: test theoretical yield of a fixed rate bond, = 0.0307
FixedRateBondYield(,99.282, 100000, as.Date("2004-11-30"), as.Date("2008-11-30"), 3, , c(0.02875), , , , ,as.Date("2004-11-30"))

## bond.cpp: test theoretical price of a fixed rate bond  = 99.2708
FixedRateBondPriceByYield(,0.0307, 100000, as.Date("2004-11-30"), as.Date("2008-11-30"), 3, , c(0.02875), , , , ,as.Date("2004-11-30"))

## bond.cpp

## Simple call with a flat curve
bond <- list(settlementDays=1,
             issueDate=as.Date("2004-11-30"),
             faceAmount=100,
             dayCounter='Thirty360',
             paymentConvention='Unadjusted')
schedule <- list(effectiveDate=as.Date("2004-11-30"),
                 maturityDate=as.Date("2008-11-30"),
                 period='Semiannual',
                 calendar='UnitedStates/GovernmentBond',
                 businessDayConvention='Unadjusted',
                 terminationDateConvention='Unadjusted',
                 dateGeneration='Forward',
                 endOfMonth=1)
calc=list(dayCounter='Actual360',
          compounding='Compounded',
          freq='Annual',
          durationType='Modified')
coupon.rate <- c(0.02875)
params <- list(tradeDate=as.Date('2002-2-15'),
               settleDate=as.Date('2002-2-19'),
               dt=.25,
               interpWhat="discount",
               interpHow="loglinear")
discountCurve.flat <- DiscountCurve(params, list(flat=0.05))
FixedRateBond(bond,
              coupon.rate,
              schedule,
              calc,
              discountCurve=discountCurve.flat)

## Same bond calculated from yield rather than from the discount curve
yield <- 0.02
FixedRateBond(bond,
              coupon.rate,
              schedule,
              calc,
              yield=yield)


                                        #same example with clean price
price <- 103.31
FixedRateBond(bond,
              coupon.rate,
              schedule,
              calc,
              price = price)

## bond.cpp FloatingRateBond, following test-suite/bonds.cpp

bond <- list(faceAmount=100, issueDate=as.Date("2004-11-30"),
             maturityDate=as.Date("2008-11-30"), redemption=100,
             effectiveDate=as.Date("2004-11-30"))
dateparams <- list(settlementDays=1, calendar="UnitedStates/GovernmentBond",
                   dayCounter = 'ActualActual', period=2,
                   businessDayConvention = 1, terminationDateConvention=1,
                   dateGeneration=0, endOfMonth=0, fixingDays = 1)

gearings <- spreads <- caps <- floors <- vector()

params <- list(tradeDate=as.Date('2002-2-15'),
               settleDate=as.Date('2002-2-19'),
               dt=.25,
               interpWhat="discount",
               interpHow="loglinear")

tsQuotes <- list(d1w  =0.0382,
                 d1m  =0.0372,
                 fut1=96.2875,
                 fut2=96.7875,
                 fut3=96.9875,
                 fut4=96.6875,
                 fut5=96.4875,
                 fut6=96.3875,
                 fut7=96.2875,
                 fut8=96.0875,
                 s3y  =0.0398,
                 s5y  =0.0443,
                 s10y =0.05165,
                 s15y =0.055175)


## when both discount and libor curves are flat.

discountCurve.flat <- DiscountCurve(params, list(flat=0.05))
termstructure <- DiscountCurve(params, list(flat=0.03))
iborIndex.params <- list(type="USDLibor", length=6,
                         inTermOf="Month", term=termstructure)
FloatingRateBond(bond, gearings, spreads, caps, floors,
                 iborIndex.params, discountCurve.flat, dateparams)

## borrowed from Rmetrics' unit testing framework
## removed test for interactive()
##
## Dirk Eddelbuettel, 29 Dec 2007

pkg <- "RQuantLib"

if (require("RUnit", quietly = TRUE)) {
    library(package=pkg, character.only = TRUE)
    if(!(exists("path") && file.exists(path)))
        path <- system.file("unitTests", package = pkg)

    ## Define tests
    testSuite <- defineTestSuite(name = paste(pkg, "unit testing"), dirs = path)

    ## Run tests
    tests <- runTestSuite(testSuite)

    if(file.access(path, 02) != 0) {
        ## cannot write to path -> use writable one
        tdir <- tempfile(paste(pkg, "unitTests", sep="_"))
        dir.create(tdir)
        pathReport <- file.path(tdir, "report")
        cat("RUnit reports are written into ", tdir, "/report.(txt|html)", sep = "")
    } else {
        pathReport <- file.path(path, "report")
    }

    ## Print Results:
    printTextProtocol(tests)
    printTextProtocol(tests, fileName = paste(pathReport, ".txt", sep = ""))

    ## Print HTML Version to a File:
    printHTMLProtocol(tests, fileName = paste(pathReport, ".html", sep = ""))

    ## stop() if there are any failures i.e. FALSE to unit test.
    ## This will cause R CMD check to return error and stop
    if(getErrors(tests)$nFail > 0) {
        stop("one of the unit tests failed")
    }
} else {
    cat("R package 'RUnit' cannot be loaded -- no unit tests run\n", "for package", pkg,"\n")
}

##  RQuantLib -- R interface to the QuantLib libraries
##
##  Copyright (C) 2014  Dirk Eddelbuettel <edd@debian.org>
##
##  This file is part of RQuantLib.
##
##  RQuantLib is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 2 of the License, or
##  (at your option) any later version.
##
##  RQuantLib is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with RQuantLib.  If not, see <http://www.gnu.org/licenses/>.

unitTestSetup <- function(file = NULL, packages = NULL) {
    function(){
        if (!is.null(packages)) {
            for (p in packages) {
                suppressMessages(require(p, character.only = TRUE))
            }
        }
        if (!is.null(file)) {
            if (exists("pathRQuantLibTests")) {
                sourceCpp(file.path(get("pathRQuantLibTests"), "cpp", file))
            } else if (file.exists(file.path("cpp", file))) {
                sourceCpp(file.path("cpp", file))
            } else {
                sourceCpp(system.file("unitTests", "cpp", file, package="RQuantLib"))
            }
        }
    }
}

#!/usr/bin/r -t
#
# Copyright (C) 2014  Dirk Eddelbuettel
#
# This file is part of RQuantLib.
#
# RQuantLib is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# RQuantLib is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with RQuantLib.  If not, see <http://www.gnu.org/licenses/>.

.onWindows <- .Platform$OS.type == "windows"

if (!.onWindows) {
    .setUp <- RQuantLib:::unitTestSetup("dates.cpp")
}

test.date.conversion <- function() { 
    if (!.onWindows) {
        given <- as.Date("2000-01-01")
        inc <- 2
        expected <- given + inc

        checkEquals(advanceDateRR(given, inc), expected, msg="date conversion from R to R")
        checkEquals(advanceDateQR(given, inc), expected, msg="date conversion from QuantLib to R")
        checkEquals(advanceDateQQ(given, inc), expected, msg="date conversion from QuantLib to QuantLib")
    }
}

test.datevector.conversion <- function() { 
    if (!.onWindows) {
        given <- as.Date("2000-01-01") + 0:3
        inc <- 2
        expected <- given + inc

        checkEquals(advanceDatesRR(given, inc), expected, msg="date conversion from R to R")
        checkEquals(advanceDatesQR(given, inc), expected, msg="date conversion from QuantLib to QuantLib")
        checkEquals(advanceDatesQQ(given, inc), expected, msg="date conversion from QuantLib to R")
    }
}

## RQuantLib -- R interface to the QuantLib libraries
##
##  Copyright (C) 2002 - 2014  Dirk Eddelbuettel <edd@debian.org>
##  Copyright (C) 2009         Khanh Nguyen <knguyen@cs.umb.edu>
##  Copyright (C) 2010 - 2011  Dirk Eddelbuettel and Khanh Nguyen
##
##  This file is part of RQuantLib.
##
##  RQuantLib is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 2 of the License, or
##  (at your option) any later version.
##
##  RQuantLib is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with RQuantLib.  If not, see <http://www.gnu.org/licenses/>.

ZeroCouponBond <- function(bond, discountCurve, dateparams ) {
    UseMethod("ZeroCouponBond")
}

## TODO: redo interface here
ZeroCouponBond.default <- function(bond,
                                   discountCurve,
                                   dateparams=list(refDate=bond$issueDate,
                                   settlementDays=1,
                                   calendar='UnitedStates/GovernmentBond',
                                   businessDayConvention='Following')) {
    val <- 0

    if (is.null(bond$faceAmount)) bond$faceAmount <- 100
    if (is.null(bond$redemption)) bond$redemption <- 100

    if (is.null(dateparams$settlementDays)) dateparams$settlementDays <- 1
    if (is.null(dateparams$calendar)) dateparams$calendar <- 'UnitedStates/GovernmentBond'
    if (is.null(dateparams$businessDayConvention)) dateparams$businessDayConvention <- 'Following'
    if (is.null(dateparams$refDate)) dateparams$refDate <- bond$issueDate
    dateparams <- matchParams(dateparams)


    val <- ZeroBondWithRebuiltCurve(bond, c(discountCurve$table$date),
                                    discountCurve$table$zeroRates, dateparams)
    class(val) <- c("ZeroCouponBond", "Bond")
    val
}



ZeroPriceByYield <- function(yield, faceAmount,
                             issueDate, maturityDate,
                             dayCounter, frequency,
                             compound, businessDayConvention){
    UseMethod("ZeroPriceByYield")
}

ZeroPriceByYield.default <- function(yield, faceAmount=100,
                                     issueDate, maturityDate,
                                     dayCounter=2, frequency=2,
                                     compound=0, businessDayConvention=4) {

    val <- zeroPriceByYieldEngine(yield, faceAmount, dayCounter,
                                  frequency, businessDayConvention,
                                  compound, maturityDate, issueDate)
    class(val) <- c("ZeroPriceByYield")
    val
}

ZeroYield <- function(price, faceAmount, issueDate, maturityDate,
                      dayCounter, frequency, compound,	businessDayConvention) {
    UseMethod("ZeroYield")
}

ZeroYield.default <- function(price, faceAmount=100,
                              issueDate, maturityDate,
                              dayCounter=2, frequency=2,
                              compound=0, businessDayConvention=4) {

    val <- zeroYieldByPriceEngine(price, faceAmount, dayCounter, frequency,
                                  businessDayConvention, compound,
                                  maturityDate, issueDate)
    class(val) <- c("ZeroYield")
    val
}



FixedRateBond <- function(bond, rates, schedule, calc, discountCurve, yield, price){
    UseMethod("FixedRateBond")
}
FixedRateBond.default <- function(bond = list(),
                                  rates,
                                  schedule,
                                  calc=list(
                                    dayCounter='ActualActual.ISMA',
                                    compounding='Compounded',
                                    freq='Annual',
                                    durationType='Modified'),
                                  discountCurve = NULL,
                                  yield = NA,
                                  price = NA){
    val <- 0

    # check bond params
    if (is.null(bond$settlementDays)) bond$settlementDays <- 1
    if (is.null(bond$faceAmount)) bond$faceAmount <- 100
    if (is.null(bond$dayCounter)) bond$dayCounter <- 'Thirty360'
    # additional parameters have default values on cpp side (see QuantLib::FixedRateBond first ctor)
    # paymentConvention
    # redemption
    # issueDate
    # paymentCalendar
    # exCouponPeriod
    # exCouponCalendar
    # exCouponConvention
    # exCouponEndOfMonth
    bond <- matchParams(bond)
    
    # check schedule params
    if (is.null(schedule$effectiveDate)){
      stop("schedule effective date undefined.")
    }
    if (is.null(schedule$maturityDate)){
      stop("schedule maturity date undefined.")
    }
    if (is.null(schedule$period)) schedule$period <- 'Semiannual'
    if (is.null(schedule$calendar)) schedule$calendar <- 'TARGET'
    if (is.null(schedule$businessDayConvention)) schedule$businessDayConvention <- 'Following'
    if (is.null(schedule$terminationDateConvention)) schedule$terminationDateConvention <- 'Following'
    if (is.null(schedule$dateGeneration)) schedule$dateGeneration <- 'Backward'
    if (is.null(schedule$endOfMonth)) schedule$endOfMonth <- 0
    schedule <- matchParams(schedule)
    
    # check calc params
    if (is.null(calc$dayCounter)) calc$dayCounter <- 'ActualActual.ISMA'
    if (is.null(calc$compounding)) calc$compounding <- 'Compounded'
    if (is.null(calc$freq)) calc$freq <- 'Annual'
    if (is.null(calc$durationType)) calc$durationType <- 'Simple'
    if (is.null(calc$accuracy)) calc$accuracy <- 1.0e-8
    if (is.null(calc$maxEvaluations)) calc$maxEvaluations <- 100
    calc <- matchParams(calc)

    which.calc <- !c(is.null(discountCurve), is.na(yield), is.na(price))
    if (sum(which.calc) != 1)
        stop("one and only one of discountCurve, yield or price must be defined.")
    
    if (!is.null(discountCurve)) {
        val <- FixedRateWithRebuiltCurve(
            bond, rates, schedule, calc, c(discountCurve$table$date), discountCurve$table$zeroRates)
      
    } else if (!is.na(yield)) {
      val <- FixedRateWithYield(bond, rates, schedule, calc, yield)
      
    } else if (!is.na(price)) {
      val <- FixedRateWithPrice(bond, rates, schedule, calc, price)
    }

    class(val) <- c("FixedRateBond", "Bond")
    val
}


FixedRateBondYield <- function(settlementDays, price, faceAmount,
                               effectiveDate, maturityDate,
                               period, calendar, rates,
                               dayCounter, businessDayConvention,
                               compound, redemption, issueDate) {
     UseMethod("FixedRateBondYield")
}
FixedRateBondYield.default <- function(settlementDays = 1, price, faceAmount=100,
                                       effectiveDate, maturityDate,
                                       period, calendar = "UnitedStates/GovernmentBond", rates,
                                       dayCounter=2, businessDayConvention=0,
                                       compound = 0, redemption = 100, issueDate) {

    val <- fixedRateBondYieldByPriceEngine(settlementDays, price, calendar, faceAmount,
                                           businessDayConvention,
                                           compound, redemption, dayCounter,
                                           period, 	## aka frequency
                                           maturityDate, issueDate, effectiveDate,
                                           rates)
    class(val) <- c("FixedRateBondYield")
    val
}


FixedRateBondPriceByYield <- function(settlementDays, yield, faceAmount,
                                      effectiveDate, maturityDate,
                                      period, calendar, rates,
                                      dayCounter, businessDayConvention,
                                      compound, redemption, issueDate) {
    UseMethod("FixedRateBondPriceByYield")
}

FixedRateBondPriceByYield.default <- function(settlementDays = 1, yield, faceAmount=100,
                                              effectiveDate=issueDate, maturityDate,
                                              period, calendar = "UnitedStates/GovernmentBond", rates,
                                              dayCounter=2, businessDayConvention=0,
                                              compound = 0, redemption = 100, issueDate) {
    val <- fixedRateBondPriceByYieldEngine(settlementDays, yield, calendar, faceAmount,
                                           businessDayConvention,
                                           compound, redemption, dayCounter, period,
                                           maturityDate, issueDate, effectiveDate,
                                           rates)
    class(val) <- c("FixedRateBondPriceByYield")
    val
}


FloatingRateBond <- function(bond, gearings, spreads, caps, floors,
                             index, curve, dateparams){
    UseMethod("FloatingRateBond")
}

FloatingRateBond.default <- function(bond,
                                     gearings=vector(),
                                     spreads=vector(),
                                     caps=vector(),
                                     floors=vector(),
                                     index,
                                     curve,
                                     dateparams=list(refDate=bond$issueDate-2,
                                                     settlementDays=1,
                                                     calendar='UnitedStates/GovernmentBond',
                                                     businessDayConvention='Following',
                                                     terminationDateConvention='Following',
                                                     dayCounter='Thirty360',
                                                     period='Semiannual',
                                                     dateGeneration='Backward',
                                                     endOfMonth=0,
                                                     fixingDays=2)
                                     ) {
    val <- 0

    if (is.null(bond$faceAmount)) bond$faceAmount <- 100
    if (is.null(bond$redemption)) bond$redemption <- 100
    if (is.null(bond$effectiveDate)) bond$effectiveDate <- bond$issueDate


    if (is.null(dateparams$settlementDays)) dateparams$settlementDays <- 1
    if (is.null(dateparams$calendar)) dateparams$calendar <- 'UnitedStates/GovernmentBond'
    if (is.null(dateparams$businessDayConvention)) dateparams$businessDayConvention <- 'Following'
    if (is.null(dateparams$terminationDateConvention)) dateparams$terminationDateConvention <- 'Following'
    if (is.null(dateparams$dayCounter)) dateparams$dayCounter <- 'Thirty360'
    if (is.null(dateparams$period)) dateparams$period <- 'Semiannual'
    if (is.null(dateparams$dateGeneration)) dateparams$dateGeneration <- 'Backward'
    if (is.null(dateparams$endOfMonth)) dateparams$endOfMonth <- 0
    if (is.null(dateparams$fixingDays)) dateparams$fixingDays <- 2
    if (is.null(dateparams$refDate))  dateparams$refDate <- bond$issueDate-2

    dateparams <- matchParams(dateparams)

    indexparams <- list(type=index$type, length=index$length, inTermOf=index$inTermOf)
    ibor <- index$term
    val <- floatingWithRebuiltCurveEngine(bond, gearings, spreads, caps, floors, indexparams,
                                          c(ibor$table$date), ibor$table$zeroRates,
                                          c(curve$table$date), curve$table$zeroRates,
                                          dateparams)
    class(val) <- c("FloatingRateBond", "Bond")
    val

}


ConvertibleZeroCouponBond <- function(bondparams, process, dateparams){
    UseMethod("ConvertibleZeroCouponBond")
}

ConvertibleZeroCouponBond.default <- function(bondparams,
                                              process,
                                              dateparams=list(
                                              settlementDays=1,
                                              calendar='UnitedStates/GovernmentBond',
                                              dayCounter='Thirty360',
                                                period='Semiannual',
                                                businessDayConvention='Following'
                                              )
                                              ) {
    val <- 0

    if (is.null(bondparams$exercise)) bondparams$exercise <- 'am'
    if (is.null(bondparams$faceAmount)) bondparams$faceAmount <- 100
    if (is.null(bondparams$redemption)) bondparams$redemption <- 100
    if (is.null(bondparams$divSch)) {
        bondparams$divSch <- data.frame(Type=character(0), Amount=numeric(0),
        Rate <- numeric(0), Date = as.Date(character(0)))
    }
    if (is.null(bondparams$callSch)){
        bondparams$callSch <- data.frame(Price=numeric(0), Type=character(0),
        Date <- as.Date(character(0)))
    }


    if (is.null(dateparams$settlementDays)) dateparams$settlementDays <- 1
    if (is.null(dateparams$calendar)) dateparams$calendar <- 'UnitedStates/GovernmentBond'
    if (is.null(dateparams$businessDayConvention)) dateparams$businessDayConvention <- 'Following'
    if (is.null(dateparams$dayCounter)) dateparams$dayCounter <- 'Thirty360'
    if (is.null(dateparams$period)) dateparams$period <- 'Semiannual'

    dateparams <- matchParams(dateparams)
    callabilitySchedule <- bondparams$callSch
    dividendSchedule <- bondparams$divSch
    dividendYield <- process$divYield
    riskFreeRate <- process$rff
    val <- convertibleZeroBondEngine(bondparams, process,
                                     c(dividendYield$table$date),
                                     dividendYield$table$zeroRates,
                                     c(riskFreeRate$table$date),
                                     riskFreeRate$table$zeroRates,
                                     dividendSchedule, callabilitySchedule, dateparams)
    class(val) <- c("ConvertibleZeroCouponBond", "Bond")
    val
}


ConvertibleFixedCouponBond <- function(bondparams, coupon, process, dateparams){
    UseMethod("ConvertibleFixedCouponBond")
}

ConvertibleFixedCouponBond.default <- function(bondparams,
                                               coupon,
                                               process,
                                               dateparams=list(
                                                 settlementDays=1,
                                                 calendar='UnitedStates/GovernmentBond',
                                                 dayCounter='Thirty360',
                                                 period='Semiannual',
                                                 businessDayConvention='Following'
                                                 )
                                               ){

    val <- 0

    if (is.null(bondparams$exercise)) bondparams$exercise <- 'am'
    if (is.null(bondparams$faceAmount)) bondparams$faceAmount <- 100
    if (is.null(bondparams$redemption)) bondparams$redemption <- 100
    if (is.null(bondparams$divSch)) {
      bondparams$divSch <- data.frame(Type=character(0), Amount=numeric(0),
                                      Rate = numeric(0), Date = as.Date(character(0)))
    }
    if (is.null(bondparams$callSch)) {
      bondparams$callSch <- data.frame(Price=numeric(0), Type=character(0),
                                       Date=as.Date(character(0)))
    }

    if (is.null(dateparams$settlementDays)) dateparams$settlementDays <- 1
    if (is.null(dateparams$calendar)) dateparams$calendar <- 'UnitedStates/GovernmentBond'
    if (is.null(dateparams$businessDayConvention)) dateparams$businessDayConvention <- 'Following'
    if (is.null(dateparams$dayCounter)) dateparams$dayCounter <- 'Thirty360'
    if (is.null(dateparams$period)) dateparams$period <- 'Semiannual'

    dateparams <- matchParams(dateparams)
    callabilitySchedule <- bondparams$callSch
    dividendSchedule <- bondparams$divSch
    dividendYield <- process$divYield
    riskFreeRate <- process$rff
    val <- convertibleFixedBondEngine(bondparams, coupon, process,
                                      c(dividendYield$table$date),
                                      dividendYield$table$zeroRates,
                                      c(riskFreeRate$table$date),
                                      riskFreeRate$table$zeroRates,
                                      dividendSchedule, callabilitySchedule, dateparams)
    class(val) <- c("ConvertibleFixedCouponBond", "Bond")
    val
}

ConvertibleFloatingCouponBond <- function(bondparams, iborindex,spread, process, dateparams){
    UseMethod("ConvertibleFloatingCouponBond")
}

ConvertibleFloatingCouponBond.default <- function(bondparams,
                                                  iborindex,
                                                  spread,
                                                  process,
                                                  dateparams=list(
                                                    settlementDays=1,
                                                    calendar='UnitedStates/GovernmentBond',
                                                    dayCounter='Thirty360',
                                                    period='Semiannual',
                                                    businessDayConvention='Following'
                                                    )){
    val <- 0

    if (is.null(bondparams$exercise)) bondparams$exercise <- 'am'
    if (is.null(bondparams$faceAmount)) bondparams$faceAmount <- 100
    if (is.null(bondparams$redemption)) bondparams$redemption <- 100
    if (is.null(bondparams$divSch)) {
      bondparams$divSch <- data.frame(Type=character(0), Amount=numeric(0),
                                      Rate = numeric(0), Date = as.Date(character(0)))
    }
    if (is.null(bondparams$callSch)) {
        bondparams$callSch <- data.frame(Price=numeric(0), Type=character(0),
                                         Date=as.Date(character(0)))
    }


    if (is.null(dateparams$settlementDays)) dateparams$settlementDays <- 1
    if (is.null(dateparams$calendar)) dateparams$calendar <- 'UnitedStates/GovernmentBond'
    if (is.null(dateparams$businessDayConvention)) dateparams$businessDayConvention <- 'Following'
    if (is.null(dateparams$dayCounter)) dateparams$dayCounter <- 'Thirty360'
    if (is.null(dateparams$period)) dateparams$period <- 'Semiannual'


    dateparams <- matchParams(dateparams)
    callabilitySchedule <- bondparams$callSch
    dividendSchedule <- bondparams$divSch
    dividendYield <- process$divYield
    riskFreeRate <- process$rff

    indexparams <- list(type=iborindex$type, length=iborindex$length,
                        inTermOf=iborindex$inTermOf)
    ibor <- iborindex$term

    val <- convertibleFloatingBondEngine(bondparams,  process,
                                         c(dividendYield$table$date),
                                         dividendYield$table$zeroRates,
                                         c(riskFreeRate$table$date),
                                         riskFreeRate$table$zeroRates,
                                         c(ibor$table$date),
                                         ibor$table$zeroRates,
                                         indexparams,spread,
                                         dividendSchedule, callabilitySchedule,
                                         dateparams)
    class(val) <- c("ConvertibleFloatingCouponBond", "Bond")
    val
}

CallableBond <- function(bondparams, hullWhite, coupon, dateparams){
    UseMethod("CallableBond")
}

CallableBond.default <- function(bondparams, hullWhite,
                                 coupon,
                                 dateparams=list(
                                   settlementDays=1,
                                   calendar='UnitedStates/GovernmentBond',
                                   dayCounter='Thirty360',
                                   period='Semiannual',
                                   businessDayConvention='Following',
                                   terminationDateConvention='Following'
                                 )){
    val <- 0

    if (is.null(bondparams$faceAmount)) bondparams$faceAmount <- 100
    if (is.null(bondparams$redemption)) bondparams$redemption <- 100
    if (is.null(bondparams$callSch)) {
        bondparams$callSch <- data.frame(Price=numeric(0), Type=character(0),
                                         Date=as.Date(character(0)))
    }

    if (is.null(dateparams$settlementDays)) dateparams$settlementDays <- 1
    if (is.null(dateparams$calendar)) dateparams$calendar <- 'UnitedStates/GovernmentBond'
    if (is.null(dateparams$businessDayConvention)) dateparams$businessDayConvention <- 'Following'
    if (is.null(dateparams$terminationDateConvention)) dateparams$terminationDateConvention <- 'Following'
    if (is.null(dateparams$dayCounter)) dateparams$dayCounter <- 'Thirty360'
    if (is.null(dateparams$period)) dateparams$period <- 'Semiannual'

    dateparams <- matchParams(dateparams)
    callSch <- bondparams$callSch
#    hw.termStructure <- hullWhite$term

    val <- callableBondEngine(bondparams, hullWhite,coupon,
                              ## c(hw.termStructure$table$date),
                              ## hw.termStructure$table$zeroRates,
                              callSch, dateparams)
    class(val) <- c("CallableBond", "Bond")
    val
}

FittedBondCurve <- function(curveparams,
                            lengths,
                            coupons,
                            marketQuotes,
                            dateparams){
    UseMethod("FittedBondCurve")
}

FittedBondCurve.default <- function(curveparams,
                                    lengths,
                                    coupons,
                                    marketQuotes,
                                    dateparams){
    val <- 0
    dateparams <- matchParams(dateparams)
    val <- fittedBondCurveEngine(curveparams, lengths, coupons, marketQuotes, dateparams)
    class(val) <- c("DiscountCurve")
    val
}


#CMSBond <- function(bondparams, swapIndex, cap, floor, gearings, spreads
#                    pricer, iborIndex){
#   UseMethod("CMSBond")
#}

#CMSBond.default <- function(bondparams, iborIndex, swapIndex, cap, floor, gearings,
#                spreads, pricer){
#   val <- 0
#   swaptionVol <- pricer$swaptionVol
#   atmOptionTenors <- swaptionVol$atmOptionTenors
#   atmSwapTenors <- swaptionVol$atmSwapTenors
#   volMatrix <- swaptionVol$volatilityMatrix
#   swapIndex <- matchParams(swapIndex)
#   ibor <- iborIndex$term
#   val <- .Call("CMSBond", bondparams, iborIndex, swapIndex, cap, floor, gearings, spreads,
#                swaptionVol, atmOptionTenors, atmSwapTenors, volMatrix, pricer
#                ibor$table$dates, ibor$table$zeroRates)
#}

# matching functions

matchDayCounter <- function(daycounter = c("Actual360", "ActualFixed", "ActualActual",
                 "Business252", "OneDayCounter", "SimpleDayCounter", "Thirty360", "Actual365NoLeap",
                 "ActualActual.ISMA", "ActualActual.Bond", "ActualActual.ISDA", "ActualActual.Historical", "ActualActual.AFB", "ActualActual.Euro"))
{
  
     if (!is.numeric(daycounter)) {
         daycounter <- match.arg(daycounter)
         daycounter <- switch(daycounter,
                              Actual360 = 0,
                              ActualFixed = 1,
                              ActualActual = 2,
                              Business252 = 3,
                              OneDayCounter = 4,
                              SimpleDayCounter = 5,
                              Thirty360 = 6,
                              Actual365NoLeap = 7,
                              ActualActual.ISMA = 8,
                              ActualActual.Bond = 9,
                              ActualActual.ISDA = 10,
                              ActualActual.Historical = 11,
                              ActualActual.AFB = 12,
                              ActualActual.Euro = 13)
     }
     daycounter
}

matchBDC <- function(bdc = c("Following", "ModifiedFollowing",
                             "Preceding", "ModifiedPreceding",
                             "Unadjusted", "HalfMonthModifiedFollowing", "Nearest")) {
     if (!is.numeric(bdc)){
         bdc <- match.arg(bdc)
         bdc <- switch(bdc,
                       Following = 0,
                       ModifiedFollowing = 1,
                       Preceding = 2,
                       ModifiedPreceding = 3,
                       Unadjusted = 4,
                       HalfMonthModifiedFollowing = 5,
                       Nearest = 6)
     }
     bdc
}

matchCompounding <- function(cp = c("Simple", "Compounded",
                                    "Continuous", "SimpleThenCompounded")) {
     if (!is.numeric(cp)){
        cp <- match.arg(cp)
        cp <- switch(cp,
                     Simple = 0,
                     Compounded = 1,
                     Continuous = 2,
                     SimpleThenCompounded = 3)
     }
     cp
}
matchFrequency <- function(freq = c("NoFrequency","Once", "Annual",
                                    "Semiannual", "EveryFourthMonth",
                                    "Quarterly", "Bimonthly", "Monthly",
                                    "EveryFourthWeek", "Biweekly",
                                    "Weekly", "Daily")) {
    if (!is.numeric(freq)){
        freq <- match.arg(freq)
        freq <- switch(freq,
                       NoFrequency = -1, Once = 0, Annual = 1,
                       Semiannual = 2, EveryFourthMonth = 3,
                       Quarterly = 4, Bimonthly = 6,
                       Monthly = 12, EveryFourthWeek = 13,
                       Biweekly = 26, Weekly = 52, Daily = 365)
    }
    freq
}



matchFloatFrequency <- function(freq = c( "Annual",
                                    "Semiannual", "EveryFourthMonth",
                                    "Quarterly", "Bimonthly", "Monthly")) {
    if (!is.numeric(freq)){
       freq <- match.arg(freq)
       freq <- switch(freq,
                      Annual = 12,
                      Semiannual = 6, EveryFourthMonth = 4,
                      Quarterly = 3, Bimonthly = 2,
                      Monthly = 1)
    }
    freq
}

matchDateGen <- function(dg = c("Backward", "Forward", "Zero",
                                "ThirdWednesday", "Twentieth",
                                "TwentiethIMM", "OldCDS", "CDS")){
   if (!is.numeric(dg)){
      dg <- match.arg(dg)
      dg <- switch(dg,
                   Backward = 0, Forward = 1,
                   Zero = 2, ThirdWednesday = 3,
                   Twentieth = 4, TwentiethIMM = 5, OldCDS=6, CDS=7)
   }
   dg
}


matchDurationType <- function(dt = c("Simple", "Macaulay", "Modified")) {
  if (!is.numeric(dt)){
    dt <- match.arg(dt)
    dt <- switch(dt,
                 Simple = 0,
                 Macaulay = 1,
                 Modified = 2)
  }
  dt
}


matchParams <- function(params) {

    if (!is.null(params$dayCounter)) {
        params$dayCounter <- matchDayCounter(params$dayCounter)
    }
    if (!is.null(params$compounding)) {
      params$compounding <- matchCompounding(params$compounding)
    }
    if (!is.null(params$period)) {
        params$period <- matchFrequency(params$period)
    }
    if (!is.null(params$freq)) {
      params$freq <- matchFrequency(params$freq)
    }
    if (!is.null(params$fixFreq)) {
        params$fixFreq <- matchFrequency(params$fixFreq)
    }
    if (!is.null(params$floatFreq)) {
        params$floatFreq <- matchFloatFrequency(params$floatFreq)
    }
    
    if (!is.null(params$businessDayConvention)) {
        params$businessDayConvention <- matchBDC(params$businessDayConvention)
    }
    if (!is.null(params$terminationDateConvention)) {
        params$terminationDateConvention <- matchBDC(params$terminationDateConvention)
    }
    if (!is.null(params$paymentConvention)) {
      params$paymentConvention <- matchBDC(params$paymentConvention)
    }
    if (!is.null(params$exCouponConvention)) {
      params$exCouponConvention <- matchBDC(params$exCouponConvention)
    }
    if (!is.null(params$dateGeneration)) {
        params$dateGeneration <- matchDateGen(params$dateGeneration)
    }
    if (!is.null(params$fixedLegConvention)) {
        params$fixedLegConvention <- matchBDC(params$fixedLegConvention)
    }
    if (!is.null(params$fixedLegDayCounter)) {
        params$fixedLegDayCounter <- matchDayCounter(params$fixedLegDayCounter)
    }
    if (!is.null(params$durationType)) {
      params$durationType <- matchDurationType(params$durationType)
    }
    params

}

# Generic methods
plot.Bond <- function(x, ...) {
    warning("No plotting available for class", class(x)[1],"\n")
    invisible(x)
}

print.Bond <- function(x, digits=5, ...) {
    cat("Concise summary of valuation for", class(x)[1], "\n")
    cat(" Net present value : ", format(x$NPV), "\n")
    cat("       clean price : ", format(x$cleanPrice, digits=digits), "\n")
    cat("       dirty price : ", format(x$dirtyPrice, digits=digits), "\n")
    cat("    accrued coupon : ", format(x$accruedCoupon, digits=digits), "\n")
    cat("             yield : ", format(x$yield, digits=digits), "\n")
    cat("        cash flows : \n")
    print(x$cashFlow, row.names=FALSE, digits=digits)
    #print(round(unlist(x[1:5]), digits))
    invisible(x)
}

print.FixedRateBond <- function(x, digits=5, ...) {
  cat("Concise summary of valuation for", class(x)[1], "\n")
  cat(" Net present value : ", format(x$NPV), "\n")
  cat("       clean price : ", format(x$cleanPrice, digits=digits), "\n")
  cat("       dirty price : ", format(x$dirtyPrice, digits=digits), "\n")
  cat("    accrued coupon : ", format(x$accruedCoupon, digits=digits), "\n")
  cat("             yield : ", format(x$yield, digits=digits), "\n")
  cat("          duration : ", format(x$duration, digits=digits), "\n")
  cat("   settlement date : ", format(x$settlementDate, format="%Y-%m-%d"), "\n")
  cat("        cash flows : \n")
  print(x$cashFlow, row.names=FALSE, digits=digits)
  #print(round(unlist(x[1:5]), digits))
  invisible(x)
}

summary.Bond <- function(object, digits=5, ...) {
    cat("Detailed summary of valuation for", class(object)[1], "\n")
    print(round(unlist(object[1:5]), digits))
    cat("with parameters\n")
    print(unlist(object[["parameters"]]))
    invisible(object)
}

##  RQuantLib -- R interface to the QuantLib libraries
##
##  Copyright (C) 2002 - 2014 Dirk Eddelbuettel <edd@debian.org>
##
##  This file is part of RQuantLib.
##
##  RQuantLib is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 2 of the License, or
##  (at your option) any later version.
##
##  RQuantLib is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with RQuantLib.  If not, see <http://www.gnu.org/licenses/>.

oldEuropeanOptionArrays <- function(type, underlying, strike, dividendYield,
                                    riskFreeRate, maturity, volatility) {
    n.underlying <- length(underlying)
    n.strike <- length(strike)
    n.dividendYield <- length(dividendYield)
    n.riskFreeRate <- length(riskFreeRate)
    n.maturity <- length(maturity)
    n.volatility <- length(volatility)

    res <- numeric(n.underlying * n.strike * n.dividendYield * n.riskFreeRate *
                   n.maturity * n.volatility)
    dim(res) <- c(n.underlying, n.strike, n.dividendYield, n.riskFreeRate,
                  n.maturity, n.volatility)
    dimnames(res) <- list(paste("s",underlying,sep="="),
                          paste("k",strike,sep="="),
                          paste("y",dividendYield,sep="="),
                          paste("r",riskFreeRate,sep="="),
                          paste("t",maturity,sep="="),
                          paste("v",volatility,sep="="))
    value <- delta <- gamma <- vega <- theta <- rho <- divRho <- res
    for (s in 1:n.underlying) {
        for (k in 1:n.strike) {
            for (y in 1:n.dividendYield) {
                for (r in 1:n.riskFreeRate) {
                    for (t in 1:n.maturity) {
                        for (v in 1:n.volatility) {
                            val <- europeanOptionEngine(type, underlying, strike,
                                                        dividendYield, riskFreeRate,
                                                        maturity, volatility)
                            value[s,k,y,r,t,v] <- val$value
                            delta[s,k,y,r,t,v] <- val$delta
                            gamma[s,k,y,r,t,v] <- val$gamma
                            vega[s,k,y,r,t,v] <- val$vega
                            theta[s,k,y,r,t,v] <- val$theta
                            rho[s,k,y,r,t,v] <- val$rho
                            divRho[s,k,y,r,t,v] <- val$divRho
                        }
                    }
                }
            }
        }
    }
    value <- drop(value)
    delta <- drop(delta)
    gamma <- drop(gamma)
    vega <- drop(vega)
    theta <- drop(theta)
    rho <- drop(rho)
    divRho <- drop(divRho)
    invisible(list(value=value, delta=delta, gamma=gamma, vega=vega,
                   theta=theta, rho=rho, divRho=divRho,
                   parameters=list(type=type, underlying=underlying,
                   strike=strike, dividendYield=dividendYield,
                   riskFreeRate=riskFreeRate, maturity=maturity,
                   volatility=volatility)))
}

EuropeanOptionArrays <- function(type, underlying, strike, dividendYield,
                                 riskFreeRate, maturity, volatility) {
    ## check that we have two vectors
    lv <- c(length(underlying) > 1,
            length(strike) > 1,
            length(dividendYield) > 1,
            length(riskFreeRate) > 1, +
            length(maturity) > 1, +
            length(volatility) > 1)
    if (sum(lv) != 2) {
        warning("Need exactly two arguments as vectors")
        return(NULL)
    }
    type <- match.arg(type, c("call", "put"))

    ## expand parameters
    pars <- expand.grid(underlying, strike, dividendYield,
                        riskFreeRate, maturity, volatility)
    nonconst <- which( apply(pars, 2, sd) != 0)
    colnames <- c("spot", "strike", "div", "rfrate", "mat", "vol")

    val <- europeanOptionArraysEngine(type, as.matrix(pars))

    ## turn list of vectors in to list of matrices
    par1 <- unique(pars[, nonconst[1]])
    par2 <- unique(pars[, nonconst[2]])
    len1 <- length(par1)
    len2 <- length(par2)
    ml <- lapply(val, function(x) matrix(x, len1, len2, dimnames=list(par1,par2)))
    return(c(ml, parameters=list(type=type, underlying=underlying,
                 strike=strike, dividendYield=dividendYield,
                 riskFreeRate=riskFreeRate, maturity=maturity,
                 volatility=volatility)))

}

plotOptionSurface <- function(EOres, ylabel="", xlabel="", zlabel="", fov=60) {
    if (requireNamespace("rgl", quietly=TRUE)) {
        if (missing(EOres)) {
            message("No 'EOres' argument supplied. Calling 'EuropeanOptionArrays' to show 'delta'.\n",
                    "See help('EuropeanOptionArrays') for function parameters and available result\n",
                    "matrices in the returned list object.")
            und.seq <- seq(10,180,by=2)
            vol.seq <- seq(0.1,0.9,by=0.1)
            ## evaluate them along with three scalar parameters
            EOarr <- EuropeanOptionArrays("call", underlying=und.seq,
                                          strike=100, dividendYield=0.01,
                                          riskFreeRate=0.03,
                                          maturity=1, volatility=vol.seq)
            EOres <- EOarr$delta
        }
        if (packageVersion("rgl") < "0.111.5")
            surface3d <- rgl::rgl.surface
        else
            surface3d <- rgl::surface3d

        axis.col <- "black"
        text.col <- axis.col
        ylab <- ylabel
        xlab <- xlabel
        zlab <- zlabel
        y <- EOres

        ## clear scene:
        rgl::clear3d()
        rgl::clear3d(type="bbox")
        rgl::clear3d(type="lights")

        ## setup env:
        rgl::bg3d(color="#DDDDDD")
        rgl::light3d()

        rgl::view3d(fov=fov)

        x <- 1:nrow(y)
        z <- 1:ncol(y)
        x <- (x-min(x))/(max(x)-min(x))
        y <- (y-min(y))/(max(y)-min(y))
        z <- (z-min(z))/(max(z)-min(z))

        surface3d(x = x, y = y, z = z, alpha=0.6, lit=TRUE, color="blue")
        rgl::lines3d(c(0,1), c(0,0), c(0,0), col=axis.col)
        rgl::lines3d(c(0,0), c(0,1), c(0,0), col=axis.col)
        rgl::lines3d(c(0,0),c(0,0), c(0,1), col=axis.col)
        rgl::text3d(1,0,0, xlab, adj=1, col=text.col)
        rgl::text3d(0,1,0, ylab, adj=1, col=text.col)
        rgl::text3d(0,0,1, zlab, adj=1, col=text.col)

        ## add grid (credit's to John Fox scatter3d)
        xgridind <- round(seq(1, nrow(y), length=25))
        zgridind <- round(seq(1, ncol(y), length=25))
        surface3d(x = x[xgridind], y = y[xgridind,zgridind], z = z[zgridind],
                         color="darkgray", alpha=0.5, lit=TRUE,
                         front="lines", back="lines")

        ## animate (credit to view3d() example)
        start <- proc.time()[3]
        while ((i <- 36*(proc.time()[3]-start)) < 360) {
            rgl::view3d(i,i/8);
        }
    } else {
        message("Please install the 'rgl' package before using this function.")
    }
}

utils::globalVariables(c("clear3d", "bg3d", "ligh3d", "view3d", "surface3d", "text3d"))

## RQuantLib -- R interface to the QuantLib libraries
##
##  Copyright (C) 2014         Michele Salvadore and Dirk Eddelbuettel
##
##  This file is part of RQuantLib.
##
##  RQuantLib is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 2 of the License, or
##  (at your option) any later version.
##
##  RQuantLib is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with RQuantLib.  If not, see <http://www.gnu.org/licenses/>.

Schedule <- function(params){
    UseMethod("Schedule")
}

Schedule.default <- function(params) {
    val <- 0
    
    # check schedule params
    if (is.null(params$effectiveDate)){
        stop("schedule effective date undefined.")
    }
    if (is.null(params$maturityDate)){
        stop("schedule maturity date undefined.")
    }
    if (is.null(params$period)) params$period <- 'Semiannual'
    if (is.null(params$calendar)) params$calendar <- 'TARGET'
    if (is.null(params$businessDayConvention)) params$businessDayConvention <- 'Following'
    if (is.null(params$terminationDateConvention)) params$terminationDateConvention <- 'Following'
    if (is.null(params$dateGeneration)) params$dateGeneration <- 'Backward'
    if (is.null(params$endOfMonth)) params$endOfMonth <- 0
    params <- matchParams(params)
    
    CreateSchedule(params)
}

library(RQuantLib)

## For QuantLib 1.31 or newer we need to adjust three test expectations
is131 <- gsub("-[a-z]+", "", RQuantLib::getQuantLibVersion()) >= "1.31"
is138 <- gsub("-[a-z]+", "", RQuantLib::getQuantLibVersion()) >= "1.38"

#test.Schedule <- function() {
## these tests are taken from the QuantLib test-suite/schedule.cpp file
startDate <- as.Date("2012-1-17")
params <- list(effectiveDate=startDate,
               maturityDate=startDate+7,
               period = "Daily",
               businessDayConvention="Preceding")
expected <- as.Date(c("2012-1-17", "2012-1-18", "2012-1-19", "2012-1-20", "2012-1-23", "2012-1-24"))
expect_equal(Schedule(params), expected, info="Testing schedule with daily frequency...")

params <- list(effectiveDate= as.Date("2009-09-30"),
               maturityDate = as.Date("2012-06-15"),
               calendar = "Japan",
               period = "Semiannual",
               businessDayConvention="Following",
               terminationDateConvention="ModifiedFollowing",
               dateGeneration="Forward",
               endOfMonth=T)
expected <- as.Date(c("2009-09-30", "2010-03-31", "2010-09-30", "2011-03-31", "2011-09-30",
                      if (is138) "2012-04-02" else "2012-03-30",
                      if (is131) "2012-06-15" else "2012-06-29"))
expect_equal(Schedule(params), expected, info="Testing end date for schedule with end-of-month adjustment...")

params$terminationDateConvention="Unadjusted"
expected[7] <- as.Date("2012-06-15")
expect_equal(Schedule(params), expected)

params <- list(effectiveDate = as.Date("2013-03-28"),
               maturityDate = as.Date("2015-03-30"),
               calendar = "TARGET",
               period="Annual",
               businessDayConvention="Unadjusted",
               terminationDateConvention="Unadjusted",
               dateGeneration="Forward",
               endOfMonth=T)
expected <- as.Date(c(if (is131) "2013-03-28" else "2013-03-31",
                      "2014-03-31", "2015-03-30"))
expect_equal(Schedule(params), expected, info="Testing that no dates are past the end date with EOM adjustment...")

params <- list(effectiveDate = as.Date("1996-08-31"),
               maturityDate = as.Date("1997-09-15"),
               calendar = "UnitedStates/GovernmentBond",
               period="Semiannual",
               businessDayConvention="Unadjusted",
               terminationDateConvention="Unadjusted",
               dateGeneration="Forward",
               endOfMonth=T)
expected <- as.Date(c("1996-08-31", "1997-02-28", "1997-08-31", "1997-09-15"))
expect_equal(Schedule(params), expected, info=paste("Testing that the last date is not adjusted for EOM",
                                                   "when termination date convention is unadjusted..."))

params <- list(effectiveDate = as.Date("1996-08-22"),
               maturityDate = as.Date("1997-08-31"),
               calendar = "UnitedStates/GovernmentBond",
               period="Semiannual",
               businessDayConvention="Unadjusted",
               terminationDateConvention="Unadjusted",
               dateGeneration="Backward",
               endOfMonth=T)

expected <- as.Date(c("1996-08-22", "1996-08-31", "1997-02-28", "1997-08-31"))
expect_equal(Schedule(params), expected, info=paste("Testing that the first date is not adjusted for EOM",
                                                   "going backward when termination date convention is unadjusted..."))

params <- list(effectiveDate = as.Date("1996-08-22"),
               maturityDate = as.Date("1997-08-31"),
               calendar = "UnitedStates/GovernmentBond",
               period="Semiannual",
               businessDayConvention="Following",
               terminationDateConvention="Following",
               dateGeneration="Backward",
               endOfMonth=T)
expected <- if (is138) {
                as.Date(c("1996-08-22", "1996-09-03", "1997-02-28", "1997-09-02"))
            } else if (is131) {
                as.Date(c("1996-08-22", "1996-08-30", "1997-02-28", "1997-09-02"))
            } else {
                as.Date(c("1996-08-30", "1997-02-28", "1997-08-29"))
            }
expect_equal(Schedule(params), expected, info=paste("Testing that the first date is not duplicated due to",
                                                   "EOM convention when going backwards..."))

if( compareVersion(gsub("-[a-z]+", "", getQuantLibVersion()), "1.7.1") >= 0 ) {
    params <- list(effectiveDate = as.Date("2016-01-13"),
                   maturityDate = as.Date("2016-05-04"),
                   calendar = "TARGET",
                   period = "EveryFourthWeek",
                   businessDayConvention="Following",
                   terminationDateConvention="Following",
                   dateGeneration="Forward")

    expected <- as.Date(c("2016-01-13", "2016-02-10", "2016-03-09", "2016-04-06", "2016-05-04"))
    expect_equal(Schedule(params), expected, info="Testing that a four-weeks tenor works...")
}


#test.CDS <- function() {
params <- list(effectiveDate = as.Date("2016-06-21"),
               maturityDate = as.Date("2021-06-20"),
               calendar = "UnitedStates/GovernmentBond",
               period = "Quarterly",
               businessDayConvention="ModifiedFollowing",
               terminationDateConvention="Unadjusted",
               dateGeneration="CDS")
## values come from Bloomberg (last date is wrong, because of a bug in QuantLib)
expected <-  as.Date(c("2016-06-20", "2016-09-20", "2016-12-20", "2017-03-20", "2017-06-20",
                       "2017-09-20", "2017-12-20", "2018-03-20", "2018-06-20", "2018-09-20",
                       "2018-12-20", "2019-03-20", "2019-06-20", "2019-09-20", "2019-12-20",
                       "2020-03-20", "2020-06-22", "2020-09-21", "2020-12-21", "2021-03-22",
                       "2021-06-20"))
expect_equal(Schedule(params), expected, info="Checking schedule of IG26")

library(RQuantLib)

## For QuantLib 1.31 or newer we need to adjust three test expectations
is131 <- gsub("-[a-z]+", "", RQuantLib::getQuantLibVersion()) >= "1.31"

#test.Schedule <- function() {
## these tests are taken from the QuantLib test-suite/schedule.cpp file
startDate <- as.Date("2012-1-17")
params <- list(effectiveDate=startDate,
               maturityDate=startDate+7,
               period = "Daily",
               businessDayConvention="Preceding")
expected <- as.Date(c("2012-1-17", "2012-1-18", "2012-1-19", "2012-1-20", "2012-1-23", "2012-1-24"))
expect_equal(Schedule(params), expected, info="Testing schedule with daily frequency...")

params <- list(effectiveDate= as.Date("2009-09-30"),
               maturityDate = as.Date("2012-06-15"),
               calendar = "Japan",
               period = "Semiannual",
               businessDayConvention="Following",
               terminationDateConvention="ModifiedFollowing",
               dateGeneration="Forward",
               endOfMonth=T)
expected <- as.Date(c("2009-09-30", "2010-03-31", "2010-09-30", "2011-03-31", "2011-09-30",
                      "2012-03-30",
                      if (is131) "2012-06-15" else "2012-06-29"))
expect_equal(Schedule(params), expected, info="Testing end date for schedule with end-of-month adjustment...")

params$terminationDateConvention="Unadjusted"
expected[7] <- as.Date("2012-06-15")
expect_equal(Schedule(params), expected)

params <- list(effectiveDate = as.Date("2013-03-28"),
               maturityDate = as.Date("2015-03-30"),
               calendar = "TARGET",
               period="Annual",
               businessDayConvention="Unadjusted",
               terminationDateConvention="Unadjusted",
               dateGeneration="Forward",
               endOfMonth=T)
expected <- as.Date(c(if (is131) "2013-03-28" else "2013-03-31",
                      "2014-03-31", "2015-03-30"))
expect_equal(Schedule(params), expected, info="Testing that no dates are past the end date with EOM adjustment...")

params <- list(effectiveDate = as.Date("1996-08-31"),
               maturityDate = as.Date("1997-09-15"),
               calendar = "UnitedStates/GovernmentBond",
               period="Semiannual",
               businessDayConvention="Unadjusted",
               terminationDateConvention="Unadjusted",
               dateGeneration="Forward",
               endOfMonth=T)
expected <- as.Date(c("1996-08-31", "1997-02-28", "1997-08-31", "1997-09-15"))
expect_equal(Schedule(params), expected, info=paste("Testing that the last date is not adjusted for EOM",
                                                   "when termination date convention is unadjusted..."))

params <- list(effectiveDate = as.Date("1996-08-22"),
               maturityDate = as.Date("1997-08-31"),
               calendar = "UnitedStates/GovernmentBond",
               period="Semiannual",
               businessDayConvention="Unadjusted",
               terminationDateConvention="Unadjusted",
               dateGeneration="Backward",
               endOfMonth=T)

expected <- as.Date(c("1996-08-22", "1996-08-31", "1997-02-28", "1997-08-31"))
expect_equal(Schedule(params), expected, info=paste("Testing that the first date is not adjusted for EOM",
                                                   "going backward when termination date convention is unadjusted..."))

params <- list(effectiveDate = as.Date("1996-08-22"),
               maturityDate = as.Date("1997-08-31"),
               calendar = "UnitedStates/GovernmentBond",
               period="Semiannual",
               businessDayConvention="Following",
               terminationDateConvention="Following",
               dateGeneration="Backward",
               endOfMonth=T)
expected <- if (is131) { as.Date(c("1996-08-22", "1996-08-30", "1997-02-28", "1997-09-02"))
            } else { as.Date(c("1996-08-30", "1997-02-28", "1997-08-29")) }
expect_equal(Schedule(params), expected, info=paste("Testing that the first date is not duplicated due to",
                                                   "EOM convention when going backwards..."))

if( compareVersion(gsub("-[a-z]+", "", getQuantLibVersion()), "1.7.1") >= 0 ) {
    params <- list(effectiveDate = as.Date("2016-01-13"),
                   maturityDate = as.Date("2016-05-04"),
                   calendar = "TARGET",
                   period = "EveryFourthWeek",
                   businessDayConvention="Following",
                   terminationDateConvention="Following",
                   dateGeneration="Forward")

    expected <- as.Date(c("2016-01-13", "2016-02-10", "2016-03-09", "2016-04-06", "2016-05-04"))
    expect_equal(Schedule(params), expected, info="Testing that a four-weeks tenor works...")
}


#test.CDS <- function() {
params <- list(effectiveDate = as.Date("2016-06-21"),
               maturityDate = as.Date("2021-06-20"),
               calendar = "UnitedStates/GovernmentBond",
               period = "Quarterly",
               businessDayConvention="ModifiedFollowing",
               terminationDateConvention="Unadjusted",
               dateGeneration="CDS")
## values come from Bloomberg (last date is wrong, because of a bug in QuantLib)
expected <-  as.Date(c("2016-06-20", "2016-09-20", "2016-12-20", "2017-03-20", "2017-06-20",
                       "2017-09-20", "2017-12-20", "2018-03-20", "2018-06-20", "2018-09-20",
                       "2018-12-20", "2019-03-20", "2019-06-20", "2019-09-20", "2019-12-20",
                       "2020-03-20", "2020-06-22", "2020-09-21", "2020-12-21", "2021-03-22",
                       "2021-06-20"))
expect_equal(Schedule(params), expected, info="Checking schedule of IG26")

test.Schedule <- function() {
    ## these tests are taken from the QuantLib test-suite/schedule.cpp file
    startDate <- as.Date("2012-1-17")
    params <- list(effectiveDate=startDate,
                   maturityDate=startDate+7,
                   period = "Daily",
                   businessDayConvention="Preceding")
    expected <- as.Date(c("2012-1-17", "2012-1-18", "2012-1-19", "2012-1-20", "2012-1-23", "2012-1-24"))
    checkEquals(Schedule(params), expected,
                "Testing schedule with daily frequency...")

    params <- list(effectiveDate= as.Date("2009-09-30"),
                   maturityDate = as.Date("2012-06-15"),
                   calendar = "Japan",
                   period = "Semiannual",
                   businessDayConvention="Following",
                   terminationDateConvention="ModifiedFollowing",
                   dateGeneration="Forward",
                   endOfMonth=T)
    expected <- as.Date(c("2009-09-30", "2010-03-31", "2010-09-30", "2011-03-31", "2011-09-30",
                          "2012-03-30", "2012-06-29"))
    checkEquals(Schedule(params), expected,
                "Testing end date for schedule with end-of-month adjustment...")

    params$terminationDateConvention="Unadjusted"
    expected[7] <- as.Date("2012-06-15")
    checkEquals(Schedule(params), expected)

    params <- list(effectiveDate = as.Date("2013-03-28"),
                   maturityDate = as.Date("2015-03-30"),
                   calendar = "TARGET",
                   period="Annual",
                   businessDayConvention="Unadjusted",
                   terminationDateConvention="Unadjusted",
                   dateGeneration="Forward",
                   endOfMonth=T)
    expected <- as.Date(c("2013-03-31", "2014-03-31", "2015-03-30"))
    checkEquals(Schedule(params), expected,
                "Testing that no dates are past the end date with EOM adjustment...")

    params <- list(effectiveDate = as.Date("1996-08-31"),
                   maturityDate = as.Date("1997-09-15"),
                   calendar = "UnitedStates/GovernmentBond",
                   period="Semiannual",
                   businessDayConvention="Unadjusted",
                   terminationDateConvention="Unadjusted",
                   dateGeneration="Forward",
                   endOfMonth=T)
    expected <- as.Date(c("1996-08-31", "1997-02-28", "1997-08-31", "1997-09-15"))
    checkEquals(Schedule(params), expected,
                paste("Testing that the last date is not adjusted for EOM",
                      "when termination date convention is unadjusted..."))

    params <- list(effectiveDate = as.Date("1996-08-22"),
                   maturityDate = as.Date("1997-08-31"),
                   calendar = "UnitedStates/GovernmentBond",
                   period="Semiannual",
                   businessDayConvention="Unadjusted",
                   terminationDateConvention="Unadjusted",
                   dateGeneration="Backward",
                   endOfMonth=T)

    expected <- as.Date(c("1996-08-22", "1996-08-31", "1997-02-28", "1997-08-31"))
    checkEquals(Schedule(params), expected,
                paste("Testing that the first date is not adjusted for EOM",
                      "going backward when termination date convention is unadjusted..."))

    params <- list(effectiveDate = as.Date("1996-08-22"),
                   maturityDate = as.Date("1997-08-31"),
                   calendar = "UnitedStates/GovernmentBond",
                   period="Semiannual",
                   businessDayConvention="Following",
                   terminationDateConvention="Following",
                   dateGeneration="Backward",
                   endOfMonth=T)
    expected <- as.Date(c("1996-08-30", "1997-02-28", "1997-08-29"))
    checkEquals(Schedule(params), expected,
                paste("Testing that the first date is not duplicated due to",
                      "EOM convention when going backwards..."))

    if( compareVersion(getQuantLibVersion(), "1.7.1") >= 0 ) {
        params <- list(effectiveDate = as.Date("2016-01-13"),
                       maturityDate = as.Date("2016-05-04"),
                       calendar = "TARGET",
                       period = "EveryFourthWeek",
                       businessDayConvention="Following",
                       terminationDateConvention="Following",
                       dateGeneration="Forward")

        expected <- as.Date(c("2016-01-13", "2016-02-10", "2016-03-09", "2016-04-06", "2016-05-04"))
        checkEquals(Schedule(params), expected,
                    "Testing that a four-weeks tenor works...")
    }
}

test.CDS <- function() {
    params <- list(effectiveDate = as.Date("2016-06-21"),
                   maturityDate = as.Date("2021-06-20"),
                   calendar = "UnitedStates/GovernmentBond",
                   period = "Quarterly",
                   businessDayConvention="ModifiedFollowing",
                   terminationDateConvention="Unadjusted",
                   dateGeneration="CDS")
    ## values come from Bloomberg (last date is wrong, because of a bug in QuantLib)
    expected <-  as.Date(c("2016-06-20", "2016-09-20", "2016-12-20", "2017-03-20", "2017-06-20",
                           "2017-09-20", "2017-12-20", "2018-03-20", "2018-06-20", "2018-09-20",
                           "2018-12-20", "2019-03-20", "2019-06-20", "2019-09-20", "2019-12-20",
                           "2020-03-20", "2020-06-22", "2020-09-21", "2020-12-21", "2021-03-22",
                           "2021-06-20"))
    checkEquals(Schedule(params), expected, "Checking schedule of IG26")
}

#' Vol Cube Example Data
#'
#' Data for valuing swaption examples including rates and a lognormal vol cube
#'
#' @name vcube
#' @docType data
#' @usage data(vcube)
#' @format two data frames: \code{vcube}, a data frame with four columns: \code{Expiry}, \code{Tenor},
#' \code{LogNormalVol},  and \code{Spread} 
#' @source TBA
NULL

#' Vol Cube Example Data  
#'  
#' Short time series examples
#'  
#' @name tsQuotes 
#' @docType data
#' @format A series of tenors and rates approppriate for calling DiscountCurve
#' @source TBA
NULL

##  RQuantLib function AffineSwaption
##
##  Copyright (C) 2005         Dominick Samperi
##  Copyright (C) 2007 - 2014  Dirk Eddelbuettel
##  Copyright (C) 2016 - 2022  Terry Leitch and Dirk Eddelbuettel
##

##  This file is part of RQuantLib.
##
##  RQuantLib is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 2 of the License, or
##  (at your option) any later version.
##
##  RQuantLib is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with RQuantLib.  If not, see <http://www.gnu.org/licenses/>.

AffineSwaption <- function(params,
                           ts, swaptionMaturities,
                           swapTenors, volMatrix,
                           legparams=list(dayCounter="Thirty360",
                                          fixFreq="Annual",
                                          floatFreq="Semiannual")) {
    UseMethod("AffineSwaption")
}

AffineSwaption.default <- function(params,
                                   ts, swaptionMaturities,
                                   swapTenors, volMatrix,
                                   legparams=list(dayCounter="Thirty360",
                                                  fixFreq="Annual",
                                                  floatFreq="Semiannual")) {
    # Check that params list names

    if (!is.list(params) || length(params) == 0) {
        stop("The params parameter must be a non-empty list", call.=FALSE)
    }
    if(is.null(params$startDate)){
        params$startDate=advance("UnitedStates",params$tradeDate, 1, 3)
        warning("swaption start date not set, defaulting to 1 year from trade date using US calendar")
    }
    if(is.null(params$maturity)){
        params$maturity=advance("UnitedStates",params$startDate, 5, 3)
        warning("swaption maturity not set, defaulting to 5 years from startDate using US calendar")
    }
    if(is.null(params$european)){
        params$european=TRUE
        warning("affine swaption european flag not set defaulting to european")
    }
    if(is.null(params$payFix)){
        params$payFix=TRUE
        warning("affine swaption payFix flag not set defaulting to pay fix swap")
    }

    matYears=as.numeric(params$maturity-params$tradeDate)/365
    expYears=as.numeric(params$startDate-params$tradeDate)/365
    increment=min(matYears/6,1.0)
    numObs=floor(matYears/increment)+1
    optStart=as.numeric(params$startDate-params$tradeDate)/365

    # find closest option to our target to ensure it is in calibration
    tenor=expiry=vol=vector(length=numObs,mode="numeric")

    expiryIDX=findInterval(expYears,swaptionMaturities)
    tenorIDX=findInterval(matYears-expYears,swapTenors)
    if(tenorIDX >0 & expiryIDX>0){
        vol[1]=volMatrix[expiryIDX,tenorIDX]
        expiry[1]=swaptionMaturities[expiryIDX]
        tenor[1]=swapTenors[tenorIDX]
    } else {
        vol[1]=expiry[1]=tenor[1]=0
    }

    for(i in 2:numObs){
        expiryIDX=findInterval(i*increment,swaptionMaturities)
        tenorIDX=findInterval(matYears-(i-1)*increment,swapTenors)
        if(tenorIDX >0 & expiryIDX>0){
            vol[i]=volMatrix[expiryIDX,tenorIDX]
            expiry[i]=swaptionMaturities[expiryIDX]
            tenor[i]=swapTenors[tenorIDX]

        } else {
            vol[i]=volMatrix[expiryIDX,tenorIDX+1]
            expiry[i]=swaptionMaturities[expiryIDX]
            tenor[i]=swapTenors[tenorIDX+1]
        }
    }

    # remove if search was out of bounds
    expiry=expiry[expiry>0];tenor=tenor[tenor>0];vol=vol[vol>0]
    if(length(expiry)<5){
        warning("Insufficent vols to fit affine model")
        return(NULL)
    }
    #Take 1st 5 which includes closest to initial date
    expiry=expiry[1:5];tenor=tenor[1:5];vol=vol[1:5]

    # Finally ready to make the call...
    # We could coerce types here and pass as.integer(round(swapTenors)),
    # temp <- as.double(volMatrix), dim(temp) < dim(a) [and pass temp instead
    # of volMatrix]. But this is taken care of in the C/C++ code.
    if (inherits(ts, "DiscountCurve")) {
        matchlegs<-matchParams(legparams)
        val <- affineWithRebuiltCurveEngine(params, matchlegs, c(ts$table$date), ts$table$zeroRates,
                                            expiry,tenor,vol)
    } else{
        stop("DiscountCurve class term structure required", call.=FALSE)

    }
    class(val) <- paste(params$method, "AffineSwaption",sep="")
    summary(val)
    val
}

summary.G2AnalyticAffineSwaption <- function(object,...) {
    cat('\n\tSummary of pricing results for Affine Swaption\n')
    cat('\nPrice (in bp) of Affine swaption is ', object$NPV)
    cat('\nStike is ', format(object$strike,digits=6))
    cat(' (ATM strike is ', format(object$ATMStrike,digits=6), ')')
    cat('\nModel used is: G2/Jamshidian using analytic formulas')
    cat('\nCalibrated model parameters are:')
    cat('\na = ', format(object$a,digits=4))
    cat('\nb = ', format(object$b,digits=4))
    cat('\nsigma = ', format(object$sigma,digits=4))
    cat('\neta = ', format(object$eta,digits=4))
    cat('\nrho = ', format(object$rho,digits=4))
    cat('\n\n')
}

summary.HWAnalyticAffineSwaption <- function(object,...) {
    cat('\n\tSummary of pricing results for Affine Swaption\n')
    cat('\nPrice (in bp) of Affine swaption is ', object$NPV)
    cat('\nStike is ', format(object$params$strike,digits=6))
    cat(' (ATM strike is ', format(object$ATMStrike,digits=6), ')')
    cat('\nModel used is: Hull-White using analytic formulas')
    cat('\nCalibrated model parameters are:')
    cat('\na = ', format(object$a,digits=4))
    cat('\nsigma = ', format(object$sigma,digits=4))
    cat('\n\n')
}

summary.HWTreeAffineSwaption <- function(object,...) {
    cat('\n\tSummary of pricing results for Affine Swaption\n')
    cat('\nPrice (in bp) of Affine swaption is ', object$NPV)
    cat('\nStike is ', format(object$params$strike,digits=6))
    cat(' (ATM strike is ', format(object$ATMStrike,digits=6), ')')
    cat('\nModel used is: Hull-White using a tree')
    cat('\nCalibrated model parameters are:')
    cat('\na = ', format(object$a,digits=4))
    cat('\nsigma = ', format(object$sigma,digits=4))
    cat('\n\n')
}

summary.BKTreeAffineSwaption <- function(object,...) {
    cat('\n\tSummary of pricing results for Affine Swaption\n')
    cat('\nPrice (in bp) of Affine swaption is ', object$NPV)
    cat('\nStike is ', format(object$params$strike,digits=6))
    cat(' (ATM strike is ', format(object$ATMStrike,digits=6), ')')
    cat('\nModel used is: Black-Karasinski using a tree')
    cat('\nCalibrated model parameters are:')
    cat('\na = ', format(object$a,digits=4))
    cat('\nsigma = ', format(object$sigma,digits=4))
    cat('\n\n')
}

##  RQuantLib -- R interface to the QuantLib libraries
##
##  Copyright (C) 2002 - 2014  Dirk Eddelbuettel <edd@debian.org>
##  Copyright (C) 2009         Khanh Nguyen <knguyen@cs.umb.edu>
##
##  This file is part of RQuantLib.
##
##  RQuantLib is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 2 of the License, or
##  (at your option) any later version.
##
##  RQuantLib is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with RQuantLib.  If not, see <http://www.gnu.org/licenses/>.

AsianOption <- function(averageType, type, underlying, strike, dividendYield,
                        riskFreeRate, maturity, volatility,
                        first, length, fixings) {
    UseMethod("AsianOption")
}

AsianOption.default <- function(averageType, type, underlying, strike, dividendYield,
                                riskFreeRate, maturity, volatility,
                                first=0, length=11.0/12.0, fixings=26) {
    averageType <- match.arg(averageType, c("geometric", "arithmetic"))
    type <- match.arg(type, c("call", "put"))
    if (missing(maturity)) {
        if (averageType=="geometric") {
            warning("Geometric Asian Option requires maturity argument")
            return(NULL)
        } else {
            maturity <- 1.0             # actually unused for arithmetic option case
        }
    }
    val <- asianOptionEngine(averageType, type, underlying,
                             strike, dividendYield, riskFreeRate,
                             maturity, volatility,
                             first, length, fixings)
    class(val) <- c("AsianOption","Option")
    val
}

##  RQuantLib -- R interface to the QuantLib libraries
##
##  Copyright (C) 2002-2026  Dirk Eddelbuettel <edd@debian.org>
##
##  This file is part of RQuantLib.
##
##  RQuantLib is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 2 of the License, or
##  (at your option) any later version.
##
##  RQuantLib is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with RQuantLib.  If not, see <http://www.gnu.org/licenses/>.

## also dumps core (0.3.7)
## no longer under 0.3.9 and 0.3.10 with g++ 3.4/4.0
EuropeanOptionImpliedVolatility <- function(type, value, underlying,
                                            strike, dividendYield,
                                            riskFreeRate, maturity,
                                            volatility, dayCounter=0) {
        UseMethod("EuropeanOptionImpliedVolatility")
}

EuropeanOptionImpliedVolatility.default <- function(type, value, underlying,
                                                    strike, dividendYield,
                                                    riskFreeRate, maturity,
                                                    volatility, dayCounter=0)  {

    val <- europeanOptionImpliedVolatilityEngine(type, value, underlying, strike,
                                                 dividendYield, riskFreeRate,
                                                 if (inherits(maturity, "numeric")) maturity else NULL,
                                                 if (inherits(maturity, "Date")) maturity else NULL,
                                                 volatility, dayCounter)
    class(val) <- c("EuropeanOptionImpliedVolatility","ImpliedVolatility")
    val
}

# also dumps core (0.3.7)
## no longer under 0.3.9 and 0.3.10 with g++ 3.4/4.0
AmericanOptionImpliedVolatility <- function(type, value, underlying, strike,
                                            dividendYield, riskFreeRate,
                                            maturity, volatility,
                                            timeSteps=150, gridPoints=151,
                                            dayCounter=0) {
    UseMethod("AmericanOptionImpliedVolatility")
}

AmericanOptionImpliedVolatility.default <- function(type, value, underlying, strike,
                                                    dividendYield, riskFreeRate, maturity,
                                                    volatility, timeSteps=150, gridPoints=151,
                                                    dayCounter=0) {
    val <- americanOptionImpliedVolatilityEngine(type, value, underlying, strike,
                                                 dividendYield, riskFreeRate,
                                                 if (inherits(maturity, "numeric")) maturity else NULL,
                                                 if (inherits(maturity, "Date")) maturity else NULL,
                                                 volatility, timeSteps, gridPoints,
                                                 dayCounter)
    val
}

BinaryOptionImpliedVolatility <- function(type, value, underlying, strike, dividendYield, riskFreeRate,
                                          maturity, volatility, cashPayoff=1, dayCounter=0) {
    UseMethod("BinaryOptionImpliedVolatility")
}

BinaryOptionImpliedVolatility.default <- function(type, value, underlying, strike, dividendYield, riskFreeRate,
                                                  maturity, volatility, cashPayoff=1, dayCounter=0) {
    val <- binaryOptionImpliedVolatilityEngine(type, value, underlying, strike, dividendYield, riskFreeRate,
                                               if (inherits(maturity, "numeric")) maturity else NULL,
                                               if (inherits(maturity, "Date")) maturity else NULL,
                                               volatility, cashPayoff, dayCounter)
    class(val) <- c("BinaryOptionImpliedVolatility","ImpliedVolatility")
    val
}

print.ImpliedVolatility <- function(x, digits=3, ...) {
    impvol <- x[[1]]
    cat("Implied Volatility for", class(x)[1], "is", round(impvol, digits), "\n")
    invisible(x)
}

summary.ImpliedVolatility <- function(object, digits=3, ...) {
    impvol <- object[[1]]
    cat("Implied Volatility for", class(object)[1], "is", round(impvol, digits), "\n")
    #cat("with parameters\n")
    #print(unlist(object[[2]]))
    invisible(object)
}

##  RQuantLib function DiscountCurve
##
##  Copyright (C) 2005         Dominick Samperi
##  Copyright (C) 2007 - 2016  Dirk Eddelbuettel
##  Copyright (C) 2009 - 2010  Dirk Eddelbuettel and Khanh Nguyen
##
##  This file is part of RQuantLib.
##
##  RQuantLib is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 2 of the License, or
##  (at your option) any later version.
##
##  RQuantLib is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with RQuantLib.  If not, see <http://www.gnu.org/licenses/>.

DiscountCurve <- function(params, tsQuotes, times=seq(0,10,.1),
                          legparams=list(dayCounter="Thirty360",
                                         fixFreq="Annual",
                                         floatFreq="Semiannual")) {
    UseMethod("DiscountCurve")
}

DiscountCurve.default <- function(params, tsQuotes, times=seq(0,10,.1),
                                  legparams=list(dayCounter="Thirty360",
                                                 fixFreq="Annual",
                                                 floatFreq="Semiannual")) {
    ## Check that params is properly formatted.
    if (!is.list(params) || length(params) == 0) {
        stop("The params parameter must be a non-empty list", call.=FALSE)
    }
  
    ## Check that the term structure quotes are properly formatted.
    if (!is.list(tsQuotes) || length(tsQuotes) == 0) {
        stop("Term structure quotes must be a non-empty list", call.=FALSE)
    }
    if (length(tsQuotes) != length(names(tsQuotes))) {
        stop("Term structure quotes must include labels", call.=FALSE)
    }
    if (!is.numeric(unlist(tsQuotes))) {
        stop("Term structure quotes must have numeric values", call.=FALSE)
    }
  
    ## Check the times vector
    if (!is.numeric(times) || length(times) == 0) {
        stop("The times parameter must be a non-emptry numeric vector", call.=FALSE)
    }
  
    ## Finally ready to make the call...
    ##val <- .Call("DiscountCurve", params, tsQuotes, times, PACKAGE="RQuantLib")
    matchlegs<-matchParams(legparams)
    ##val <- discountCurveEngine(params, tsQuotes, times,matchCpnmonthFreq=as.integer(monthFreq))
    val <- discountCurveEngine(params, tsQuotes, times,matchlegs)
  
    val[["table"]] <- as.data.frame(val[["table"]])  ## Windows all of a sudden needs this
    class(val) <- c("DiscountCurve")
    val
}

plot.DiscountCurve <- function(x, setpar=TRUE, dolegend=TRUE,...) {
    if (setpar) {
        savepar <- par(mfrow=c(3,1))
    }
    if (x$flatQuotes) {
        ## Don't want to plot noise when we look at a flat yield curve
        plot(c(x$times[1],x$times[length(x$times)]), c(0,.5),type='n',
             main='forwards', xlab='time',ylab='forward rate')
        lines(x$times, x$forwards, type='l')
        if (dolegend) {
            legend('center','center','flat',bty='n',text.col='red')
        }
        plot(c(x$times[1],x$times[length(x$times)]), c(0,.5),type='n',
             main='zero rates', xlab='time',ylab='zero rate')
        lines(x$times, x$zerorates, type='l')
        if (dolegend) {
            legend('center','center','flat',bty='n',text.col='red')
        }
    } else {
        plot(x$times, x$forwards, type='l', main='forwards',xlab='time',ylab='fwd rate')
        if (dolegend) {
            legend('center','center',paste(x$params$interpHow, 'discount'), bty='n', text.col='red')
        }
        plot(x$times, x$zerorates, type='l', main='zero rates',xlab='time',ylab='zero rate')
        if (dolegend) {
            legend('center','center',paste(x$params$interpHow, 'discount'),bty='n', text.col='red')
        }
    }
    plot(x$times, x$discounts, type='l', main='discounts',xlab='time',ylab='discount')
    if (dolegend) {
        if (x$flatQuotes) {
            legend('center','center','flat',bty='n',text.col='red')
        } else {
            legend('center','center',paste(x$params$interpHow, 'discount'),bty='n', text.col='red')
        }
    }
    if (setpar) {
        par(savepar)
    }
}

## RQuantLib -- R interface to the QuantLib libraries
##
## Copyright (C) 2010 - 2011  Dirk Eddelbuettel and Khanh Nguyen
## Copyright (C) 2012 - 2014  Dirk Eddelbuettel
##
## This file is part of the RQuantLib library for GNU R.
## It is made available under the terms of the GNU General Public
## License, version 2, or at your option, any later version,
## incorporated herein by reference.
##
## This program is distributed in the hope that it will be
## useful, but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with this program; if not, write to the Free
## Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
## MA 02111-1307, USA

#dayCount <- function(startDates, endDates, dayCounters) {
#    val <- .Call('dayCount', startDates, endDates, dayCounters, PACKAGE="RQuantLib")
#    invisible(val)
#}

#yearFraction <- function(startDates, endDates, dayCounters) {
#    val <- .Call('yearFraction', startDates, endDates, dayCounters, PACKAGE="RQuantLib")
#    invisible(val)
#}

#setEvaluationDate <- function(evalDate) {
#    val <- .Call("setEvaluationDate", evalDate, PACKAGE="RQuantLib")
#    invisible(val)
#}

##  RQuantLib function BermudanSwaption
##
##  Copyright (C) 2005         Dominick Samperi
##  Copyright (C) 2007 - 2014  Dirk Eddelbuettel
##  Copyright (C) 2016         Terry Leitch
##
##  This file is part of RQuantLib.
##
##  RQuantLib is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 2 of the License, or
##  (at your option) any later version.
##
##  RQuantLib is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with RQuantLib.  If not, see <http://www.gnu.org/licenses/>.

BermudanSwaption <- function(params, ts, swaptionMaturities,
                             swapTenors, volMatrix) {
    UseMethod("BermudanSwaption")
}

BermudanSwaption.default <- function(params, ts, swaptionMaturities,
                                     swapTenors, volMatrix) {
    # Check that params list names

        if (!is.list(params) || length(params) == 0) {
            stop("The params parameter must be a non-empty list", call.=FALSE)
        }
        if(is.null(params$startDate)){
            params$startDate=advance("UnitedStates",params$tradeDate, 1, 3)
            warning("swaption start date not set, defaulting to 1 year from trade date using US calendar")
        }
        if(is.null(params$maturity)){
            params$maturity=advance("UnitedStates",params$startDate, 5, 3)
            warning("swaption maturity not set, defaulting to 5 years from startDate using US calendar")
        }
    matYears=as.numeric(params$maturity-params$tradeDate)/365
    expYears=as.numeric(params$startDate-params$tradeDate)/365
    increment=min(matYears/6,1.0)
    numObs=floor(matYears/increment)+1
    optStart=as.numeric(params$startDate-params$tradeDate)/365

    # find closest option to our target to ensure it is in calibration
    tenor=expiry=vol=vector(length=numObs,mode="numeric")

    expiryIDX=findInterval(expYears,swaptionMaturities)
    tenorIDX=findInterval(matYears-expYears,swapTenors)
    if(tenorIDX >0 & expiryIDX>0){
        vol[1]=volMatrix[expiryIDX,tenorIDX]
        expiry[1]=swaptionMaturities[expiryIDX]
        tenor[1]=swapTenors[tenorIDX]
    } else {
        vol[1]=expiry[1]=tenor[1]=0
    }

    for(i in 2:numObs){
        expiryIDX=findInterval(i*increment,swaptionMaturities)
        tenorIDX=findInterval(matYears-(i-1)*increment,swapTenors)
        if(tenorIDX >0 & expiryIDX>0){
            vol[i]=volMatrix[expiryIDX,tenorIDX]
            expiry[i]=swaptionMaturities[expiryIDX]
            tenor[i]=swapTenors[tenorIDX]

        } else {
            vol[i]=volMatrix[expiryIDX,tenorIDX+1]
            expiry[i]=swaptionMaturities[expiryIDX]
            tenor[i]=swapTenors[tenorIDX+1]
        }
    }

    # remove if search was out of bounds
    expiry=expiry[expiry>0];tenor=tenor[tenor>0];vol=vol[vol>0]
    if(length(expiry)<5){
        warning("Insufficent vols to fit affine model")
        return(NULL)
    }
    #Take 1st 5 which includes closest to initial date
    expiry=expiry[1:5];tenor=tenor[1:5];vol=vol[1:5]

#
# Check that the term structure quotes are properly formatted.
#         if(is)
#             if (!is.list(ts) || length(ts) == 0) {
#                 stop("Term structure quotes must be a non-empty list", call.=FALSE)
#             }
#         if (length(ts) != length(names(ts))) {
#             stop("Term structure quotes must include labels", call.=FALSE)
#         }
#         if (!is.numeric(unlist(ts))) {
#             stop("Term structure quotes must have numeric values", call.=FALSE)
#         }



    # Check for correct matrix/vector types
    if (!is.matrix(volMatrix)
        || !is.vector(swaptionMaturities)
        || !is.vector(swapTenors)) {
        stop("Swaption vol must be a matrix, maturities/tenors must be vectors",
             call.=FALSE)
    }

    # Check that matrix/vectors have compatible dimensions
    if (prod(dim(volMatrix)) != length(swaptionMaturities)*length(swapTenors)) {
        stop("Dimensions of swaption vol matrix not compatible with maturity/tenor vectors",
             call.=FALSE)
    }

    # Finally ready to make the call...
    # We could coerce types here and pass as.integer(round(swapTenors)),
    # temp <- as.double(volMatrix), dim(temp) < dim(a) [and pass temp instead
    # of volMatrix]. But this is taken care of in the C/C++ code.
    if (inherits(ts, "DiscountCurve")) {
        val <- bermudanWithRebuiltCurveEngine(params, c(ts$table$date), ts$table$zeroRates,
                                      swaptionMaturities,
                                      swapTenors, volMatrix)
    } else{
        if (!is.numeric(ts) | length(ts) !=1) {
            stop("Flat Term structure yield must have single numeric value", call.=FALSE)
        }
        val <- bermudanFromYieldEngine(params, ts,
                                  swaptionMaturities,
                                  swapTenors, volMatrix)
    }
    class(val) <- c(params$method, "BermudanSwaption")
    val
}

summary.G2Analytic <- function(object,...) {
    cat('\n\tSummary of pricing results for Bermudan Swaption\n')
    cat('\nPrice (in bp) of Bermudan swaption is ', object$price)
    cat('\nStike is ', format(object$params$strike,digits=6))
    cat(' (ATM strike is ', format(object$ATMStrike,digits=6), ')')
    cat('\nModel used is: G2/Jamshidian using analytic formulas')
    cat('\nCalibrated model parameters are:')
    cat('\na = ', format(object$a,digits=4))
    cat('\nb = ', format(object$b,digits=4))
    cat('\nsigma = ', format(object$sigma,digits=4))
    cat('\neta = ', format(object$eta,digits=4))
    cat('\nrho = ', format(object$rho,digits=4))
    cat('\n\n')
}

summary.HWAnalytic <- function(object,...) {
    cat('\n\tSummary of pricing results for Bermudan Swaption\n')
    cat('\nPrice (in bp) of Bermudan swaption is ', object$price)
    cat('\nStike is ', format(object$params$strike,digits=6))
    cat(' (ATM strike is ', format(object$ATMStrike,digits=6), ')')
    cat('\nModel used is: Hull-White using analytic formulas')
    cat('\nCalibrated model parameters are:')
    cat('\na = ', format(object$a,digits=4))
    cat('\nsigma = ', format(object$sigma,digits=4))
    cat('\n\n')
}

summary.HWTree <- function(object,...) {
    cat('\n\tSummary of pricing results for Bermudan Swaption\n')
    cat('\nPrice (in bp) of Bermudan swaption is ', object$price)
    cat('\nStike is ', format(object$params$strike,digits=6))
    cat(' (ATM strike is ', format(object$ATMStrike,digits=6), ')')
    cat('\nModel used is: Hull-White using a tree')
    cat('\nCalibrated model parameters are:')
    cat('\na = ', format(object$a,digits=4))
    cat('\nsigma = ', format(object$sigma,digits=4))
    cat('\n\n')
}

summary.BKTree <- function(object,...) {
    cat('\n\tSummary of pricing results for Bermudan Swaption\n')
    cat('\nPrice (in bp) of Bermudan swaption is ', object$price)
    cat('\nStike is ', format(object$params$strike,digits=6))
    cat(' (ATM strike is ', format(object$ATMStrike,digits=6), ')')
    cat('\nModel used is: Black-Karasinski using a tree')
    cat('\nCalibrated model parameters are:')
    cat('\na = ', format(object$a,digits=4))
    cat('\nsigma = ', format(object$sigma,digits=4))
    cat('\n\n')
}

## RQuantLib -- R interface to the QuantLib libraries
##
## Copyright (C) 2002 - 2016  Dirk Eddelbuettel <edd@debian.org>
##
## This file is part of the RQuantLib library for GNU R.
## It is made available under the terms of the GNU General Public
## License, version 2, or at your option, any later version,
## incorporated herein by reference.
##
## This program is distributed in the hope that it will be
## useful, but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public License
## along with Rcpp.  If not, see <http://www.gnu.org/licenses/>.

.pkgglobalenv <- new.env(parent=emptyenv())

.onLoad <- function(libname, pkgname) {

    ## default to NULL
    qlcflags <- qllibs <- NULL
    
    if (.Platform$OS.type=="windows") {
        if (Sys.getenv("QUANTLIB_ROOT") != "") {
            ## by convention with CRAN, on Windows we expect these two
            ## variables to be set, and to be pointing to the respective
            ## helper installations
            qlroot   <- Sys.getenv("QUANTLIB_ROOT")
            boostlib <- Sys.getenv("BOOSTLIB")
            rarch    <- Sys.getenv("R_ARCH")
            qlcflags <- sprintf("-I%s -I. -I\"%s\"", qlroot, boostlib)
            qllibs   <- sprintf("-L%s/lib%s -lQuantLib", qlroot, rarch)
        }
    } else {
        ## on Linux and OS X, see if we have quantlib-config which may well be
        ## false in the case of prebuild binary packages as eg r-cran-rquantlib
        ## on Debian / Ubuntu as well as the OS X package from CRAN
        ## first we check whether we have quantlib-config in the path
        if (isTRUE(unname(Sys.which("quantlib-config")) != "")) {
            qc <- system("bash -c 'type -p quantlib-config'", ignore.stderr=TRUE, intern=TRUE)
            if (is.character(qc) && nchar(qc) > 1) {
                qlcflags <- system(paste(qc, "--cflags"), intern = TRUE)
                qllibs   <- system(paste(qc, "--libs"),   intern = TRUE)
            }
        }
    }

    assign("ql_cflags", qlcflags, envir=.pkgglobalenv)
    assign("ql_libs", qllibs, envir=.pkgglobalenv)
}

LdFlags <- function(print = TRUE) {
    if (is.null(.pkgglobalenv$ql_libs)) stop("Cannot supply LdFlags as none set.")
    if (print) cat(.pkgglobalenv$ql_libs) else .pkgglobalenv$ql_libs
}

CFlags <- function(print = TRUE) {
    if (is.null(.pkgglobalenv$ql_cflags)) stop("Cannot supply CFlags as none set.")
    if (print) cat(.pkgglobalenv$ql_cflags) else .pkgglobalenv$ql_cflags
}

inlineCxxPlugin <- function(...) {
    plugin <- Rcpp.plugin.maker(include.before = "#include <RQuantLib.h>",
                                libs = sprintf("%s $(LAPACK_LIBS) $(BLAS_LIBS) $(FLIBS)", LdFlags(FALSE)),
                                package = "RQuantLib",
                                Makevars = NULL,
                                Makevars.win = NULL)
    settings <- plugin()
    settings$env$PKG_CPPFLAGS <- paste("-DRQuantLib_Plugin", CFlags(FALSE))
    settings
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

affineWithRebuiltCurveEngine <- function(rparam, legparams, dateVec, zeroVec, swaptionMat, swapLengths, swaptionVols) {
    .Call(`_RQuantLib_affineWithRebuiltCurveEngine`, rparam, legparams, dateVec, zeroVec, swaptionMat, swapLengths, swaptionVols)
}

asianOptionEngine <- function(averageType, type, underlying, strike, dividendYield, riskFreeRate, maturity, volatility, first, length, fixings) {
    .Call(`_RQuantLib_asianOptionEngine`, averageType, type, underlying, strike, dividendYield, riskFreeRate, maturity, volatility, first, length, fixings)
}

binaryOptionEngine <- function(binType, type, excType, underlying, strike, dividendYield, riskFreeRate, maturity, exDate, volatility, cashPayoff, dayCounter) {
    .Call(`_RQuantLib_binaryOptionEngine`, binType, type, excType, underlying, strike, dividendYield, riskFreeRate, maturity, exDate, volatility, cashPayoff, dayCounter)
}

binaryOptionImpliedVolatilityEngine <- function(type, value, underlying, strike, dividendYield, riskFreeRate, maturity, exDate, volatility, cashPayoff, dayCounter) {
    .Call(`_RQuantLib_binaryOptionImpliedVolatilityEngine`, type, value, underlying, strike, dividendYield, riskFreeRate, maturity, exDate, volatility, cashPayoff, dayCounter)
}

barrierOptionEngine <- function(barrType, type, underlying, strike, dividendYield, riskFreeRate, maturity, exDate, volatility, barrier, rebate, dayCounter) {
    .Call(`_RQuantLib_barrierOptionEngine`, barrType, type, underlying, strike, dividendYield, riskFreeRate, maturity, exDate, volatility, barrier, rebate, dayCounter)
}

bermudanFromYieldEngine <- function(rparam, yield, swaptionMat, swapLengths, swaptionVols) {
    .Call(`_RQuantLib_bermudanFromYieldEngine`, rparam, yield, swaptionMat, swapLengths, swaptionVols)
}

bermudanWithRebuiltCurveEngine <- function(rparam, dateVec, zeroVec, swaptionMat, swapLengths, swaptionVols) {
    .Call(`_RQuantLib_bermudanWithRebuiltCurveEngine`, rparam, dateVec, zeroVec, swaptionMat, swapLengths, swaptionVols)
}

zeroPriceByYieldEngine <- function(yield, faceAmount, dayCounter, frequency, businessDayConvention, compound, maturityDate, issueDate) {
    .Call(`_RQuantLib_zeroPriceByYieldEngine`, yield, faceAmount, dayCounter, frequency, businessDayConvention, compound, maturityDate, issueDate)
}

zeroYieldByPriceEngine <- function(price, faceAmount, dayCounter, frequency, businessDayConvention, compound, maturityDate, issueDate) {
    .Call(`_RQuantLib_zeroYieldByPriceEngine`, price, faceAmount, dayCounter, frequency, businessDayConvention, compound, maturityDate, issueDate)
}

fixedRateBondYieldByPriceEngine <- function(settlementDays, price, cal, faceAmount, businessDayConvention, compound, redemption, dayCounter, frequency, maturityDate, issueDate, effectiveDate, rates) {
    .Call(`_RQuantLib_fixedRateBondYieldByPriceEngine`, settlementDays, price, cal, faceAmount, businessDayConvention, compound, redemption, dayCounter, frequency, maturityDate, issueDate, effectiveDate, rates)
}

fixedRateBondPriceByYieldEngine <- function(settlementDays, yield, cal, faceAmount, businessDayConvention, compound, redemption, dayCounter, frequency, maturityDate, issueDate, effectiveDate, rates) {
    .Call(`_RQuantLib_fixedRateBondPriceByYieldEngine`, settlementDays, yield, cal, faceAmount, businessDayConvention, compound, redemption, dayCounter, frequency, maturityDate, issueDate, effectiveDate, rates)
}

FloatBond1 <- function(bond, gearings, caps, spreads, floors, indexparams, index, discountCurve, dateparams) {
    .Call(`_RQuantLib_FloatBond1`, bond, gearings, caps, spreads, floors, indexparams, index, discountCurve, dateparams)
}

FloatBond2 <- function(bond, gearings, caps, spreads, floors, indexparams, index_params, index_tsQuotes, index_times, discountCurve, dateparams) {
    .Call(`_RQuantLib_FloatBond2`, bond, gearings, caps, spreads, floors, indexparams, index_params, index_tsQuotes, index_times, discountCurve, dateparams)
}

FloatBond3 <- function(bond, gearings, caps, spreads, floors, indexparams, index, disc_params, disc_tsQuotes, disc_times, dateparams) {
    .Call(`_RQuantLib_FloatBond3`, bond, gearings, caps, spreads, floors, indexparams, index, disc_params, disc_tsQuotes, disc_times, dateparams)
}

FloatBond4 <- function(bond, gearings, caps, spreads, floors, indexparams, index_params, index_tsQuotes, index_times, disc_params, disc_tsQuotes, disc_times, dateparams) {
    .Call(`_RQuantLib_FloatBond4`, bond, gearings, caps, spreads, floors, indexparams, index_params, index_tsQuotes, index_times, disc_params, disc_tsQuotes, disc_times, dateparams)
}

floatingWithRebuiltCurveEngine <- function(bondparams, gearings, spreads, caps, floors, indexparams, iborDateVec, iborzeroVec, dateVec, zeroVec, dateparams) {
    .Call(`_RQuantLib_floatingWithRebuiltCurveEngine`, bondparams, gearings, spreads, caps, floors, indexparams, iborDateVec, iborzeroVec, dateVec, zeroVec, dateparams)
}

FixedRateWithYield <- function(bondparam, ratesVec, scheduleparam, calcparam, yield) {
    .Call(`_RQuantLib_FixedRateWithYield`, bondparam, ratesVec, scheduleparam, calcparam, yield)
}

FixedRateWithPrice <- function(bondparam, ratesVec, scheduleparam, calcparam, price) {
    .Call(`_RQuantLib_FixedRateWithPrice`, bondparam, ratesVec, scheduleparam, calcparam, price)
}

FixedRateWithRebuiltCurve <- function(bondparam, ratesVec, scheduleparam, calcparam, dateVec, zeroVec) {
    .Call(`_RQuantLib_FixedRateWithRebuiltCurve`, bondparam, ratesVec, scheduleparam, calcparam, dateVec, zeroVec)
}

ZeroBondWithRebuiltCurve <- function(bond, dateVec, zeroVec, dateparams) {
    .Call(`_RQuantLib_ZeroBondWithRebuiltCurve`, bond, dateVec, zeroVec, dateparams)
}

convertibleZeroBondEngine <- function(rparam, processParam, dividendYieldDateVec, dividendYieldZeroVec, rffDateVec, rffZeroVec, dividendScheduleFrame, callabilityScheduleFrame, datemisc) {
    .Call(`_RQuantLib_convertibleZeroBondEngine`, rparam, processParam, dividendYieldDateVec, dividendYieldZeroVec, rffDateVec, rffZeroVec, dividendScheduleFrame, callabilityScheduleFrame, datemisc)
}

convertibleFixedBondEngine <- function(rparam, rates, processParam, dividendYieldDateVec, dividendYieldZeroVec, rffDateVec, rffZeroVec, dividendScheduleFrame, callabilityScheduleFrame, datemisc) {
    .Call(`_RQuantLib_convertibleFixedBondEngine`, rparam, rates, processParam, dividendYieldDateVec, dividendYieldZeroVec, rffDateVec, rffZeroVec, dividendScheduleFrame, callabilityScheduleFrame, datemisc)
}

convertibleFloatingBondEngine <- function(rparam, processParam, dividendYieldDateVec, dividendYieldZeroVec, rffDateVec, rffZeroVec, iborIndexDateVec, iborIndexZeroVec, iborparams, spreads, dividendScheduleFrame, callabilityScheduleFrame, datemisc) {
    .Call(`_RQuantLib_convertibleFloatingBondEngine`, rparam, processParam, dividendYieldDateVec, dividendYieldZeroVec, rffDateVec, rffZeroVec, iborIndexDateVec, iborIndexZeroVec, iborparams, spreads, dividendScheduleFrame, callabilityScheduleFrame, datemisc)
}

callableBondEngine <- function(rparam, hwparam, coupon, callabilityScheduleFrame, datemisc) {
    .Call(`_RQuantLib_callableBondEngine`, rparam, hwparam, coupon, callabilityScheduleFrame, datemisc)
}

fittedBondCurveEngine <- function(curveparam, length, coupons, marketQuotes, datemisc) {
    .Call(`_RQuantLib_fittedBondCurveEngine`, curveparam, length, coupons, marketQuotes, datemisc)
}

setCalendarContext <- function(calendar, fixingDays, settleDate) {
    .Call(`_RQuantLib_setCalendarContext`, calendar, fixingDays, settleDate)
}

isBusinessDay <- function(calendar, dates) {
    .Call(`_RQuantLib_isBusinessDay`, calendar, dates)
}

isHoliday <- function(calendar, dates) {
    .Call(`_RQuantLib_isHoliday`, calendar, dates)
}

isWeekend <- function(calendar, dates) {
    .Call(`_RQuantLib_isWeekend`, calendar, dates)
}

isEndOfMonth <- function(calendar, dates) {
    .Call(`_RQuantLib_isEndOfMonth`, calendar, dates)
}

getEndOfMonth <- function(calendar, dates) {
    .Call(`_RQuantLib_getEndOfMonth`, calendar, dates)
}

adjust <- function(calendar, dates, bdc = 0L) {
    .Call(`_RQuantLib_adjust`, calendar, dates, bdc)
}

advance1 <- function(calendar, amount, unit, bdcVal, emr, dates) {
    .Call(`_RQuantLib_advance1`, calendar, amount, unit, bdcVal, emr, dates)
}

advance2 <- function(calendar, period, bdcVal, emr, dates) {
    .Call(`_RQuantLib_advance2`, calendar, period, bdcVal, emr, dates)
}

businessDaysBetween <- function(calendar, from, to, includeFirst = TRUE, includeLast = FALSE) {
    .Call(`_RQuantLib_businessDaysBetween`, calendar, from, to, includeFirst, includeLast)
}

getHolidayList <- function(calendar, from, to, includeWeekends = FALSE) {
    .Call(`_RQuantLib_getHolidayList`, calendar, from, to, includeWeekends)
}

getBusinessDayList <- function(calendar, from, to) {
    .Call(`_RQuantLib_getBusinessDayList`, calendar, from, to)
}

addHolidays <- function(calendar, dates) {
    invisible(.Call(`_RQuantLib_addHolidays`, calendar, dates))
}

removeHolidays <- function(calendar, dates) {
    invisible(.Call(`_RQuantLib_removeHolidays`, calendar, dates))
}

advanceDate <- function(issueDate, days) {
    .Call(`_RQuantLib_advanceDate`, issueDate, days)
}

dayCount <- function(startDates, endDates, dayCounters) {
    .Call(`_RQuantLib_dayCount`, startDates, endDates, dayCounters)
}

yearFraction <- function(startDates, endDates, dayCounters) {
    .Call(`_RQuantLib_yearFraction`, startDates, endDates, dayCounters)
}

setEvaluationDate <- function(evalDate) {
    .Call(`_RQuantLib_setEvaluationDate`, evalDate)
}

discountCurveEngine <- function(rparams, tslist, times, legParams) {
    .Call(`_RQuantLib_discountCurveEngine`, rparams, tslist, times, legParams)
}

calibrateHullWhiteUsingCapsEngine <- function(termStrcDateVec, termStrcZeroVec, capDF, iborDateVec, iborZeroVec, iborType, evalDate) {
    .Call(`_RQuantLib_calibrateHullWhiteUsingCapsEngine`, termStrcDateVec, termStrcZeroVec, capDF, iborDateVec, iborZeroVec, iborType, evalDate)
}

calibrateHullWhiteUsingSwapsEngine <- function(termStrcDateVec, termStrcZeroVec, swapDF, iborDateVec, iborZeroVec, iborType, evalDate) {
    .Call(`_RQuantLib_calibrateHullWhiteUsingSwapsEngine`, termStrcDateVec, termStrcZeroVec, swapDF, iborDateVec, iborZeroVec, iborType, evalDate)
}

europeanOptionImpliedVolatilityEngine <- function(type, value, underlying, strike, dividendYield, riskFreeRate, maturity, exDate, volatility, dayCounter) {
    .Call(`_RQuantLib_europeanOptionImpliedVolatilityEngine`, type, value, underlying, strike, dividendYield, riskFreeRate, maturity, exDate, volatility, dayCounter)
}

americanOptionImpliedVolatilityEngine <- function(type, value, underlying, strike, dividendYield, riskFreeRate, maturity, exDate, volguess, timesteps, gridpoints, dayCounter) {
    .Call(`_RQuantLib_americanOptionImpliedVolatilityEngine`, type, value, underlying, strike, dividendYield, riskFreeRate, maturity, exDate, volguess, timesteps, gridpoints, dayCounter)
}

sabrengine <- function(rparam, legParams, dateVec, zeroVec, swaptionMat, swapLengths, atmVols, strikes, smirkVols) {
    .Call(`_RQuantLib_sabrengine`, rparam, legParams, dateVec, zeroVec, swaptionMat, swapLengths, atmVols, strikes, smirkVols)
}

CreateSchedule <- function(params) {
    .Call(`_RQuantLib_CreateSchedule`, params)
}

#' This function returns the QuantLib version string as encoded in the header
#' file \code{config.hpp} and determined at compilation time of the QuantLib library.
#'
#' @title Return the QuantLib version number
#' @return A character variable
#' @references \url{https://www.quantlib.org} for details on \code{QuantLib}.
#' @author Dirk Eddelbuettel
#' @examples
#'   getQuantLibVersion()
getQuantLibVersion <- function() {
    .Call(`_RQuantLib_getQuantLibVersion`)
}

#' This function returns a named vector of boolean variables describing several
#' configuration options determined at compilation time of the QuantLib library.
#'
#' Not all of these features are used (yet) by RQuantLib.
#' @title Return configuration options of the QuantLib library
#' @return A named vector of logical variables
#' @references \url{https://www.quantlib.org} for details on \code{QuantLib}.
#' @author Dirk Eddelbuettel
#' @examples
#'   getQuantLibCapabilities()
getQuantLibCapabilities <- function() {
    .Call(`_RQuantLib_getQuantLibCapabilities`)
}

europeanOptionEngine <- function(type, underlying, strike, dividendYield, riskFreeRate, maturity, exDate, volatility, discreteDividends, discreteDividendsTimeUntil, dayCounter) {
    .Call(`_RQuantLib_europeanOptionEngine`, type, underlying, strike, dividendYield, riskFreeRate, maturity, exDate, volatility, discreteDividends, discreteDividendsTimeUntil, dayCounter)
}

americanOptionEngine <- function(type, underlying, strike, dividendYield, riskFreeRate, maturity, exDate, volatility, timeSteps, gridPoints, engine, discreteDividends, discreteDividendsTimeUntil, dayCounter) {
    .Call(`_RQuantLib_americanOptionEngine`, type, underlying, strike, dividendYield, riskFreeRate, maturity, exDate, volatility, timeSteps, gridPoints, engine, discreteDividends, discreteDividendsTimeUntil, dayCounter)
}

europeanOptionArraysEngine <- function(type, par) {
    .Call(`_RQuantLib_europeanOptionArraysEngine`, type, par)
}

zeroprice <- function(yield, maturity, settle, period, basis) {
    .Call(`_RQuantLib_zeroprice`, yield, maturity, settle, period, basis)
}

zeroyield <- function(price, maturity, settle, period, basis) {
    .Call(`_RQuantLib_zeroyield`, price, maturity, settle, period, basis)
}

# Register entry points for exported C++ functions
methods::setLoadAction(function(ns) {
    .Call(`_RQuantLib_RcppExport_registerCCallable`)
})

## test for calendaring functions
##
## cf test-suite/calendars.cpp in the QL sources

library(RQuantLib)

#test.isHoliday <- function() {
expect_true(isHoliday("UnitedStates", as.Date("2004-05-01")), info="isHoliday.US")

#test.isBusinessDay <- function() {
expect_true(isBusinessDay("UnitedStates", as.Date("2004-04-26")), info="isBusinessDay.US")

## test for calendaring functions
##
## cf test-suite/calendars.cpp in the QL sources

test.isHoliday <- function() {
    checkEquals(isHoliday("UnitedStates", as.Date("2004-05-01")), TRUE, msg="isHoliday.US")
}

test.isBusinessDay <- function() {
    checkEquals(isBusinessDay("UnitedStates", as.Date("2004-04-26")), TRUE, msg="isBusinessDay.US")
}

if(!file.exists("../windows/quantlib/include/ql/quantlib.hpp")){
  unlink("../windows", recursive = TRUE)
  url <- if(grepl("aarch", R.version$platform)){
    "https://github.com/r-windows/bundles/releases/download/quantlib-1.32/quantlib-1.32-clang-aarch64.tar.xz"
  } else if(grepl("clang", Sys.getenv('R_COMPILED_BY'))){
    "https://github.com/r-windows/bundles/releases/download/quantlib-1.32/quantlib-1.32-clang-x86_64.tar.xz"
  } else if(getRversion() >= "4.3") {
    "https://github.com/r-windows/bundles/releases/download/quantlib-1.32/quantlib-1.32-ucrt-x86_64.tar.xz"
  } else {
    "https://github.com/rwinlib/quantlib/archive/v1.29.tar.gz"
  }
  download.file(url, basename(url), quiet = TRUE)
  dir.create("../windows", showWarnings = FALSE)
  untar(basename(url), exdir = "../windows", tar = 'internal')
  unlink(basename(url))
  setwd("../windows")
  file.rename(list.files(), 'quantlib')
  if(file.exists("quantlib/lib.zip")) unzip("quantlib/lib.zip", exdir = "quantlib")
}

##  RQuantLib function sabrSwaption
##
##  Copyright (C) 2005         Dominick Samperi
##  Copyright (C) 2007 - 2015  Dirk Eddelbuettel
##  Copyright (C) 2016         Terry Leitch and Dirk Eddelbuettel
##
##  This file is part of RQuantLib.
##
##  RQuantLib is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 2 of the License, or
##  (at your option) any later version.
##
##  RQuantLib is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with RQuantLib.  If not, see <http://www.gnu.org/licenses/>.

SabrSwaption <- function(params,
                         ts,volCubeDF,
                         legparams=list(dayCounter="Thirty360",
                                        fixFreq="Annual",
                                        floatFreq="Semiannual"),
                         tsUp01=NA,tsDn01=NA,vega=FALSE) {

    UseMethod("SabrSwaption")
}

SabrSwaption.default <- function(params,
                                 ts,  volCubeDF,
                                 legparams=list(dayCounter="Thirty360",
                                                fixFreq="Annual",
                                                floatFreq="Semiannual"),
                                 tsUp01=NA,tsDn01=NA,vega=FALSE) {
    # Check that params list names

    if(is.null(params$startDate)){
        params$startDate=advance("UnitedStates",params$tradeDate, 1, 3)
        warning("swaption start date not set, defaulting to 1 year from trade date using US calendar")
    }
    if(is.null(params$expiryDate)){
        params$expiryDate=params$startDate
        warning("swaption expiry date not set, defaulting to 1 year from trade date using US calendar")
    }
    if(is.null(params$maturity)){
        params$maturity=advance("UnitedStates",params$startDate, 5, 3)
        warning("swaption maturity not set, defaulting to 5 years from startDate using US calendar")
    }
    volCube=volDF2CubeK(params,volCubeDF)
    if(vega){
        volCubeDF$LogNormalVol=volCubeDF$LogNormalVol+.01
        volCubeUp=volDF2CubeK(params,volCubeDF)
    }
    swapTenors=volCube$tenors
    if (!is.list(params) || length(params) == 0) {
        stop("The params parameter must be a non-empty list", call.=FALSE)
    }
    swaptionMaturities=volCube$expiries
    ##reshape dataframe to fit QL call
    vc=volDF2CubeK(params,volCubeDF)
    matYears=as.numeric(params$maturity-params$tradeDate)/365
    expYears=as.numeric(params$expiryDate-params$tradeDate)/365

    if (inherits(ts, "DiscountCurve")) {
        matchlegs<-matchParams(legparams)
        val <- sabrengine(params, matchlegs, c(ts$table$date), ts$table$zeroRates,
                          volCube$expiries,volCube$tenors,volCube$atmVol,volCube$strikes,volCube$smirk)
        if(vega){
            valUp <- sabrengine(params, matchlegs, c(ts$table$date), ts$table$zeroRates,
                                volCubeUp$expiries,volCubeUp$tenors,volCubeUp$atmVol,volCubeUp$strikes,volCubeUp$smirk)

            val$payVega=valUp$pay-val$pay
            val$rcvVega=valUp$rcv-val$rcv
            if(anyNA(tsUp01)){
            }else{
                valTsUp <- sabrengine(params, matchlegs, c(tsUp01$table$date), tsUp01$table$zeroRates,
                                      volCube$expiries,volCube$tenors,volCube$atmVol,volCube$strikes,volCube$smirk)

                val$payDV01=valTsUp$pay-val$pay
                val$rcvDV01=valTsUp$rcv-val$rcv
                if(anyNA(tsDn01)){
                } else{
                    valTsDn <- sabrengine(params, matchlegs, c(tsDn01$table$date), tsDn01$table$zeroRates,
                                          volCube$expiries,volCube$tenors,volCube$atmVol,volCube$strikes,volCube$smirk)
                    val$payCnvx=(valTsUp$pay+valTsDn$pay-2*val$pay)/2
                    val$rcvCnvx=(valTsUp$rcv+valTsDn$rcv-2*val$rcv)/2
                }
            }
        }
    } else{
        stop("DiscountCurve class term structure required", call.=FALSE)

    }
    val$params=params
    val$atmRate=as.numeric(val$atmRate)
    class(val) <- "SabrSwaption"
    summary(val)
    val
}


volDF2CubeK <- function(params, tbl, source = "CME") {
    strikes <- levels(tbl$Spread)<-c(-200,-150,-100,-75,-50,-25,0,25,50,75,100,150,200)

    matYears <- as.numeric(params$maturity-params$tradeDate)/365
    expYears <- as.numeric(params$expiryDate-params$tradeDate)/365
    expLvl <- c( "1M","3M","6M","1Y","2Y","3Y","4Y",  "5Y",  "6Y",  "7Y",  "8Y",  "9Y","10Y")
    tbl$Expiry <- factor(tbl$Expiry, levels  <-  expLvl)
    expiries <- c(1/12,.25,.5,1,2,3,4,5,6,7,8,9,10)
    tenorLvl <- c( "1Y", "2Y",  "5Y",  "10Y","15Y","20Y","30Y")
    tbl$Tenor <- factor(tbl$Tenor, levels = tenorLvl)
    tenors <- c(1,2,5,10,15,20,30)
    tenorIDX <- max(findInterval(matYears-expYears,tenors),1)
    tenorIDX <- min(tenorIDX,length(tenors)-3)
    expiryIDX <- findInterval(expYears,expiries)
    expiryIDX <- min(expiryIDX,length(expiries)-3)

    strikeIDX <- 1
    expire <- expLvl[expiryIDX]
    tenor <- tenorLvl[tenorIDX]

    for(strike in levels(tbl$Spread)){
        if(!is.na(tbl[tbl$Expiry==expire & tbl$Spread==strike &tbl$Tenor==tenor,]$LogNormalVol))break;
        strikeIDX <- strikeIDX+1
    }
    strikes <- strikes[strikeIDX:length(strikes)]
    expLvl <- expLvl[expiryIDX:length(expLvl)]
    expiries <- expiries[expiryIDX:length(expiries)]
    tenorLvl <- tenorLvl[tenorIDX:length(tenorLvl)]
    tenors <- tenors[tenorIDX:length(tenors)]
    tbl <- tbl[tbl$Expiry%in%expLvl,]
    tbl <- tbl[tbl$Tenor%in%tenorLvl,]
    tbl <- tbl[tbl$Spread%in%strikes,]
    tbl <- tbl[with(tbl,order(Expiry,Tenor,Spread)),]
    tbl3 <- tbl[tbl$Spread==0,]
    # atm vol matrix
    #atmMat=acast(tbl3,Expiry~Tenor,value.var = "LogNormalVol")
    atmMat=matrix(data=NA,nrow=length(expLvl),ncol=length(tenorLvl),dimnames=list(expLvl,tenorLvl))
    for(i in 1:length(expLvl)){
      for(j in 1:length(tenorLvl)){
        atmMat[i,j]=tbl[tbl$Expiry==expLvl[i]& tbl$Tenor==tenorLvl[j] & tbl$Spread==0,]$LogNormalVol
      }
    }
    smirk=matrix(ncol=length(strikes),nrow=length(expLvl)*length(tenorLvl))

    #tmp3=acast(tbl,Expiry~Tenor~Spread,value.var="LogNormalVol")
    k=0
    for(i in 1:length(expLvl)){
      for(j in 1:length(tenorLvl)){
        k=k+1
        for(n in 1:length(strikes)){
          #smirk[k,]=tmp3[i,j,]-tmp3[i,j,"0"]
          smirk[k,n]=tbl[tbl$Expiry==expLvl[i]& tbl$Tenor==tenorLvl[j] & tbl$Spread==strikes[n],]$LogNormalVol -
            tbl[tbl$Expiry==expLvl[i]& tbl$Tenor==tenorLvl[j] & tbl$Spread==0,]$LogNormalVol
        }
      }
    }


    smirk <- na.spline(smirk,method="natural")

    tmp <- list(atmVol=atmMat,tenors=tenors,expiries=expiries,smirk=smirk,strikes=strikes/10000)
    class(tmp) <- "volcube"

    return(tmp)
}

if (requireNamespace("tinytest", quietly=TRUE) &&
    utils::packageVersion("tinytest") >= "1.0.0") {

    ## Set a seed to make the test deterministic
    set.seed(42)

    ## R makes us to this
    Sys.setenv("R_TESTS"="")

    ## there are several more granular ways to test files in a tinytest directory,
    ## see its package vignette; tests can also run once the package is installed
    ## using the same command `test_package(pkgName)`, or by director or file
    tinytest::test_package("RQuantLib", ncpu=getOption("Ncpus", 1))
}

##  RQuantLib -- R interface to the QuantLib libraries
##
##  Copyright (C) 2010        Dirk Eddelbuettel and Khanh Nguyen
##
##  This file is part of RQuantLib.
##
##  RQuantLib is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 2 of the License, or
##  (at your option) any later version.
##
##  RQuantLib is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with RQuantLib.  If not, see <http://www.gnu.org/licenses/>.

hullWhiteCalibrateUsingCap <- function(termStrc, capHelpers,
                                       index, evaluationDate) {
    capData <- capHelpers$data
    ibor <- index$term
    val <- calibrateHullWhiteUsingCapsEngine(termStrc$table$date,
                                             termStrc$table$zeroRates,
                                             capData,
                                             ibor$table$date,
                                             ibor$table$zeroRates,
                                             index$type,
                                             evaluationDate)
}

hullWhiteCalibrateUsingSwap <- function(termStrc, swapHelpers,
                                        index, evaluationDate) {
    swapData <- swapHelpers$data
    ibor <- index$term
    val <- calibrateHullWhiteUsingSwapsEngine(termStrc$table$date,
                                              termStrc$table$zeroRates,
                                              swapData,
                                              ibor$table$date,
                                              ibor$table$zeroRates,
                                              index$type,
                                              evaluationDate)
}

.onWinOrMac <- grepl("darwin|mingw", R.Version()$platform)

if (.onWinOrMac) exit_file("Skipping dates test on Windows.")

Rcpp::sourceCpp("cpp/dates.cpp")

#test.date.conversion <- function() {
given <- as.Date("2000-01-01")
inc <- 2
expected <- given + inc

expect_equal(advanceDateRR(given, inc), expected, info="date conversion from R to R")
expect_equal(advanceDateQR(given, inc), expected, info="date conversion from QuantLib to R")
expect_equal(advanceDateQQ(given, inc), expected, info="date conversion from QuantLib to QuantLib")

#test.datevector.conversion <- function() {
given <- as.Date("2000-01-01") + 0:3
inc <- 2
expected <- given + inc

expect_equal(advanceDatesRR(given, inc), expected, info="date conversion from R to R")
expect_equal(advanceDatesQR(given, inc), expected, info="date conversion from QuantLib to QuantLib")
expect_equal(advanceDatesQQ(given, inc), expected, info="date conversion from QuantLib to R")

#!/usr/bin/r -t
#
# Copyright (C) 2014 - 2019  Dirk Eddelbuettel
#
# This file is part of RQuantLib.
#
# RQuantLib is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# RQuantLib is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with RQuantLib.  If not, see <http://www.gnu.org/licenses/>.

.onWindows <- .Platform$OS.type == "windows"

if (.onWindows) exit_file("Skipping dates test on Windows.")

Rcpp::sourceCpp("cpp/dates.cpp")

#test.date.conversion <- function() {
given <- as.Date("2000-01-01")
inc <- 2
expected <- given + inc

expect_equal(advanceDateRR(given, inc), expected, info="date conversion from R to R")
expect_equal(advanceDateQR(given, inc), expected, info="date conversion from QuantLib to R")
expect_equal(advanceDateQQ(given, inc), expected, info="date conversion from QuantLib to QuantLib")

#test.datevector.conversion <- function() {
given <- as.Date("2000-01-01") + 0:3
inc <- 2
expected <- given + inc

expect_equal(advanceDatesRR(given, inc), expected, info="date conversion from R to R")
expect_equal(advanceDatesQR(given, inc), expected, info="date conversion from QuantLib to QuantLib")
expect_equal(advanceDatesQQ(given, inc), expected, info="date conversion from QuantLib to R")

#!/bin/sh
#set -x
set -e

###############################################################################
# NOTES:
#
# All directories must be specified as Windows paths (e.g. C:/Windows)
#
# This file has only been tested using MinGW's msys_1.0. Attempts to use msys2
# were not successful. Use msys.bat to call build_RQuantLib.sh
#
# Download QuantLib and boost sources and unzip them into $BUILD_ROOT/src.
# Update the $BOOST_VERSION and $QL_VERSION environment variables to reflect
# the versions used.
#
# The current RQuantLib Makefile deletes all object files and downloads the
# pre-built QuantLib library from https://github.com/rwinlib/quantlib. Update
# the Makefile to point to your local libQuantLib.a and remove/comment the
# 'make all' commands to prevent the deletion and download.

###############################################################################
# build environment
BOOST_VERSION=1_68_0
QL_VERSION=1.14
RQL_VERSION=0.4.5.2
BUILD_ROOT="E:/build"
R_TOOLS_DIR="E:/R/Rtools35"

BINPREF=$R_TOOLS_DIR/mingw_32/bin/
BINPREF64=$R_TOOLS_DIR/mingw_64/bin/

QUANTLIB_SRC="$BUILD_ROOT/src/QuantLib-$QL_VERSION"
BOOSTLIB="$BUILD_ROOT/src/boost_$BOOST_VERSION"
QUANTLIB_ROOT="$BUILD_ROOT/QuantLib"

###############################################################################
# Check Rtools locations
if [ ! -d "$R_TOOLS_DIR/mingw_32" ]; then
    echo "$R_TOOLS_DIR/mingw_32 not found"
    exit 1
fi
if [ ! -d "$R_TOOLS_DIR/mingw_64" ]; then
    echo "$R_TOOLS_DIR/mingw_64 not found"
    exit 1
fi
# Check for boost headers
if [ ! -d "$BOOSTLIB/boost" ]; then
    echo "Did not find boost headers in $BOOSTLIB/boost"
    exit 1
fi
# Check for quantlib source
if [ ! -d "$QUANTLIB_SRC" ]; then
    echo "Did not find QuantLib source in $QUANTLIB_SRC"
    exit 1
fi
# Ensure there is an /etc/fstab
if [ -n "/etc/fstab" ]; then
    touch /etc/fstab
fi

# Install architecture-specific files into these directories via the
# --exec-prefix argument to QuantLib's configure script
mkdir -p $QUANTLIB_ROOT/lib/i386
mkdir -p $QUANTLIB_ROOT/lib/x64

# Ensure a mount point exists and add bin/ to PATH
mkdir -p /mingw
export PATH="$(cygpath -u $R_TOOLS_DIR)/bin":/mingw/bin:$PATH

# Use ccache if available
if [ -x "$(command -v ccache)" ]; then
	export PATH=/usr/lib/ccache/bin:$PATH
fi

################################################################################
## 32-bit QuantLib
mount --replace $R_TOOLS_DIR/mingw_32 /mingw

    ## Create and change to build directory
    #mkdir -p $BUILD_ROOT/ql32
    #cd $BUILD_ROOT/ql32

# QuantLib 'buglet' prevents building outside source directory
cd $QUANTLIB_SRC

# Clean prior build before (re-)running configure
if [ -f "Makefile" ]; then
  make -j6 distclean
fi

CXXFLAGS="-g0 -O2" $QUANTLIB_SRC/configure --prefix=$QUANTLIB_ROOT \
    --exec-prefix=$QUANTLIB_ROOT/lib/i386 --with-boost-include=$BOOSTLIB \
    --enable-intraday --enable-shared --enable-static --with-gnu-ld
make -j6
make install

# Shrink quantlib library
cp $QUANTLIB_ROOT/lib/i386/libQuantLib.a \
   $QUANTLIB_ROOT/lib/i386/libQuantLib.a.bak
strip --strip-unneeded $QUANTLIB_ROOT/lib/i386/libQuantLib.a

################################################################################
## 64-bit QuantLib
mount --replace $R_TOOLS_DIR/mingw_64 /mingw

  ## Create and change to build directory
  #mkdir -p $BUILD_ROOT/ql64
  #cd $BUILD_ROOT/ql64

# QuantLib 'bug' prevents building outside source directory
cd $QUANTLIB_SRC

# Clean prior build before (re-)running configure
if [ -f "Makefile" ]; then
  make -j6 distclean
fi

CXXFLAGS="-m64 -g0 -O2" $QUANTLIB_SRC/configure --prefix=$QUANTLIB_ROOT \
    --exec-prefix=$QUANTLIB_ROOT/lib/x64 --with-boost-include=$BOOSTLIB \
    --enable-intraday --enable-shared --enable-static --with-gnu-ld
make -j6
make install

## Shrink quantlib library
cp $QUANTLIB_ROOT/lib/x64/libQuantLib.a \
   $QUANTLIB_ROOT/lib/x64/libQuantLib.a.bak
strip --strip-unneeded $QUANTLIB_ROOT/lib/x64/libQuantLib.a

###############################################################################
# Install RQuantLib
#export QUANTLIB_ROOT
#export BOOSTLIB
# RQuantLib expects headers in $QUANTLIB_ROOT/ql, not $QUANTLIB_ROOT/include/ql
#ln -s $QUANTLIB_ROOT/include/ql $QUANTLIB_ROOT/.

#mkdir -p $BUILD_ROOT/library
#R_CMD="Rscript -e 'install.packages(c(\"Rcpp\",\"zoo\"), lib = \"$BUILD_ROOT/library\")'"
#Rscript -e $R_CMD
#
## Need to remove the OS_type: unix from DESCRIPTION
#cd $BUILD_ROOT
#R CMD INSTALL RQuantLib_${RQL_VERSION}.tar.gz --library=$BUILD_ROOT/library --no-clean-on-error --preclean --build
#
## run tests on each system
#Rscript --arch x64 -e 'source(system.file(package=\"RQuantLib\", \"unitTests/runTests.R\"))'
#Rscript --arch i386 -e 'source(system.file(package=\"RQuantLib\", \"unitTests/runTests.R\"))'

\name{Enum}
\alias{Enum}
\title{Documentation for parameters}
\description{
Reference for parameters when constructing a bond
}
\usage{
  # These parameters are being used an arguments to other functions.
}
\arguments{
\item{DayCounter}{an int value
       \tabular{ll}{
         \code{0} \tab Actual360 \cr
         \code{1} \tab Actual365Fixed \cr
         \code{2} \tab ActualActual (NB: soft deprecated, defaults to ActualActual.ISDA)\cr
         \code{3} \tab ActualBusiness252 \cr
         \code{4} \tab OneDayCounter \cr
         \code{5} \tab SimpleDayCounter \cr
         \code{6} \tab Thirty360 (NB: soft deprecated, defaults to Thirty360.BondBasis) \cr
         \code{7} \tab Actual365Fixed.NoLeap (NB: Actual365NoLeap if QuantLib version < 1.16) \cr
         \code{8} \tab ActualActual.ISMA \cr
         \code{9} \tab ActualActual.Bond \cr
         \code{10} \tab ActualActual.ISDA \cr
         \code{11} \tab ActualActual.Historical \cr
         \code{12} \tab ActualActual.AFB \cr
         \code{13} \tab ActualActual.Euro \cr
         \code{14} \tab Thirty360.USA \cr
         \code{15} \tab Thirty360.BondBasis \cr
         \code{16} \tab Thirty360.European \cr
         \code{17} \tab Thirty360.EurobondBasis \cr
         \code{18} \tab Thirty360.Italian \cr
         \code{19} \tab Thirty360.German
       }
}

\item{businessDayConvention}{an int value
      \tabular{ll}{
        \code{0} \tab Following \cr
        \code{1} \tab ModifiedFollowing \cr
        \code{2} \tab Preceding \cr
        \code{3} \tab ModifiedPreceding \cr
        \code{4} \tab Unadjusted \cr
        \code{5} \tab HalfMonthModifiedFollowing \cr
        \code{6} \tab Nearest \cr
        \code{anything else} \tab Unadjusted
      }
}

\item{compounding}{an int value
     \tabular{ll}{
       \code{0} \tab Simple \cr
       \code{1} \tab Compounded \cr
       \code{2} \tab Continuous \cr
       \code{3} \tab SimpleThenCompounded
     }
}

\item{period or frequency}{an int value
     \tabular{ll}{
       \code{-1} \tab NoFrequency \cr
       \code{0} \tab Once \cr
       \code{1} \tab Annual \cr
       \code{2} \tab Semiannual \cr
       \code{3} \tab EveryFourthMonth \cr
       \code{4} \tab Quarterly \cr
       \code{6} \tab BiMonthtly \cr
       \code{12} \tab Monthly \cr
       \code{13} \tab EveryFourthWeek \cr
       \code{26} \tab BiWeekly \cr
       \code{52} \tab Weekly \cr
       \code{365} \tab Daily \cr
       \code{anything else} \tab OtherFrequency
     }
}

\item{date generation}{an int value to specify date generation rule
     \tabular{ll}{
        \code{0} \tab Backward \cr
        \code{1} \tab Forward \cr
        \code{2} \tab Zero \cr
        \code{3} \tab ThirdWednesday \cr
        \code{4} \tab Twentieth \cr
        \code{5} \tab TwentiethIMM \cr
        \code{6} \tab OldCDS \cr
        \code{7} \tab CDS \cr
        \code{anything else} \tab TwentiethIMM
     }
}

\item{durationType}{an int value to specify duration type
     \tabular{ll}{
        \code{0} \tab Simple \cr
        \code{1} \tab Macaulay \cr
        \code{2} \tab Modified
     }
}

}
\value{
None
}
\details{
 Please see any decent Finance textbook for background reading, and the
  \code{QuantLib} documentation for details on the \code{QuantLib}
  implementation, particularly the datetime classes.
}
\references{\url{https://www.quantlib.org/} for details on \code{QuantLib}.}
\author{Khanh Nguyen \email{knguyen@cs.umb.edu}}

\keyword{misc}

\name{Option}
\alias{Option}
\alias{plot.Option}
\alias{print.Option}
\alias{summary.Option}
\title{Base class for option price evalution}
\description{
  This class forms the basis from which the more specific classes are
  derived. }
\usage{
\method{print}{Option}(x, digits=4, ...)
\method{plot}{Option}(x, ...)
\method{summary}{Option}(object, digits=4, ...)
}
\arguments{
  \item{x}{Any option object derived from this base class} 
  \item{object}{Any option object derived from this base class} 
  \item{digits}{Number of digits of precision shown} 
  \item{...}{Further arguments} 
}
\value{
  None, but side effects of displaying content.
}
\details{
  Please see any decent Finance textbook for background reading, and the
  \code{QuantLib} documentation for details on the \code{QuantLib}
  implementation.  
}
\references{\url{https://www.quantlib.org/} for details on \code{QuantLib}.}
\author{Dirk Eddelbuettel \email{edd@debian.org} for the \R interface;
  the QuantLib Group for \code{QuantLib}}
\note{The interface might change in future release as \code{QuantLib}
  stabilises its own API.}
\seealso{\code{\link{AmericanOption}},\code{\link{EuropeanOption}},
\code{\link{BinaryOption}}}

\examples{
EO<-EuropeanOption("call", strike=100, volatility=0.4, 100, 0.01, 0.03, 0.5)
print(EO)
summary(EO)
}
\keyword{misc}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{vcube}
\alias{vcube}
\title{Vol Cube Example Data}
\format{two data frames: \code{vcube}, a data frame with four columns: \code{Expiry}, \code{Tenor},
\code{LogNormalVol},  and \code{Spread}}
\source{
TBA
}
\usage{
data(vcube)
}
\description{
Data for valuing swaption examples including rates and a lognormal vol cube
}

\name{AffineSwaption}
\alias{AffineSwaption}
\alias{AffineSwaption.default}
\alias{summary.G2AnalyticAffineSwaption}
\alias{summary.HWAnalyticAffineSwaption}
\alias{summary.HWTreeAffineSwaption}
\alias{summary.BKTreeAffineSwaption}
\title{Affine swaption valuation using several short-rate models}
\description{
  \code{AffineSwaption} prices a swaption with specified
  strike and maturity (in years), after calibrating the selected
  affine short-rate model to an input swaption volatility matrix. Swaption
  maturities are in years down the rows, and swap tenors are in years
  along the columns, in the usual fashion. It is assumed that the
  swaption is
  exercisable  at the start of the swap if \code{params$european} flag is set to \code{TRUE} or
  on each reset date (Bermudan) of the underlying swap if \code{params$european} flag is
  set to \code{FALSE}.
}
\usage{
AffineSwaption(params, ts, swaptionMaturities, swapTenors,
volMatrix,legparams)
}
\arguments{
  \item{params}{A list specifying the \code{tradeDate} (month/day/year),
    \code{settlementDate}, logical flags \code{payFixed} & \code{european}
    (european=FALSE generates Bermudan vlaue), \code{strike}, pricing \code{method}, and curve construction options
    (see \emph{Examples} section below). Curve construction options are
    \code{interpWhat} (possible values are \code{discount},
    \code{forward}, and \code{zero}) and
    \code{interpHow} (possible values are \code{linear},
    \code{loglinear}
    , and \code{spline}). Both \code{interpWhat} and \code{interpHow}
    are ignored when a flat yield curve is requested, but they must be
    present nevertheless.
    The pricing method can be one of the following (all short-rate models):
    \tabular{ll}{
      \code{G2Analytic} \tab G2 2-factor Gaussian model using analytic formulas.\cr
      \code{HWAnalytic} \tab Hull-White model using analytic formulas.\cr
      \code{HWTree} \tab Hull-White model using a tree.\cr
      \code{BKTree} \tab Black-Karasinski model using a tree.
      }
    }
  \item{ts}{A term structure built with DiscountCurve is required. See the
    help page for \code{\link{DiscountCurve}} and example below for details.}
  \item{swaptionMaturities}{A vector containing the swaption maturities
    associated with the rows of the swaption volatility matrix.}
  \item{swapTenors}{A vector containing the underlying swap tenors
    associated with the columns of the swaption volatility matrix.}
  \item{volMatrix}{The swaption volatility matrix. Must be a 2D matrix
  stored by rows. See the example below.}
    \item{legparams}{A list specifying the \code{dayCounter} the day count convention for the
   fixed leg (default is Thirty360), and  \code{fixFreq}, fixed coupon frequecny (defualt is Annual), \code{floatFreq},
   floating leg reset frequency (default is Semiannual).
  }
 }
\value{
  \code{AffineSwaption} returns a list containing calibrated model
  paramters (what parameters are returned depends on the model
  selected) along with:
  \item{NPV}{NPV of swaption in basis points (actual price
  equals \code{price} times notional divided by 10,000)}
  \item{ATMStrike}{At-the-money strike}
  \item{params}{Input parameter list}
}
\details{
  This function is based on \code{QuantLib} Version 0.3.10. It
  introduces support for fixed-income instruments in \code{RQuantLib}.

  At present only a small number of the many parameters that can be set
  in \code{QuantLib} are exposed by this function. Some of the
  hard-coded parameters that apply to the current version include:
  day-count conventions, fixing days (2), index (Euribor),
  fixed leg frequency (annual), and floating leg frequency
  (semi-annual). Also, it is assumed that the swaption
  volatility matrix corresponds to expiration dates and tenors that are
  measured in years (a 6-month expiration date is not currently
  supported, for example).

  Given the number of parameters that must be
  specified and the care with which they must be specified (with no
  defaults), it is not practical to use this function in the usual
  interactive fashion.

  The simplest approach is simply to save the
  example below to a file, edit as desired, and \code{source} the result.
  Alternatively, the input commands can be kept in a script file
  (under Windows) or an Emacs/ESS session (under Linux), and selected
  parts of the script can be executed in the usual way.

  Fortunately, the C++ exception mechanism seems to work well with the R
  interface, and \code{QuantLib} exceptions are propagated back to the
  R user, usually with a message that indicates what went wrong. (The
  first part of the message contains technical information about the
  precise location of the problem in the \code{QuantLib} code. Scroll to
  the end to find information that is meaningful to the R user.)
}
\references{
  Brigo, D. and Mercurio, F. (2001) \emph{Interest Rate Models: Theory and
    Practice}, Springer-Verlag, New York.

  For information about \code{QuantLib} see \url{https://www.quantlib.org/}.

  For information about \code{RQuantLib} see
\url{http://dirk.eddelbuettel.com/code/rquantlib.html}.

}
\author{Terry Leitch}

\seealso{\code{\link{DiscountCurve}}}

\examples{
if (.Platform$OS.type != "windows" && .Platform$r_arch != "i386") {
\dontrun{

# This data was generated to match the original quantlib example for Bermudan Swaption
params <- list(tradeDate=as.Date('2016-2-15'),
               settleDate=as.Date('2016-2-17'),
               startDate=as.Date('2017-2-17'),
               maturity=as.Date('2022-2-17'),
               payFixed=TRUE,
               european=FALSE,
               dt=.25,
               strike=.06,
               method="G2Analytic",
               interpWhat="discount",
               interpHow="loglinear")

# Market data used to construct the term structure of interest rates
tsQuotes <- list(d1w  =0.0382,
                 d1m  =0.0372,
                 fut1=96.2875,
                 fut2=96.7875,
                 fut3=96.9875,
                 fut4=96.6875,
                 fut5=96.4875,
                 fut6=96.3875,
                 fut7=96.2875,
                 fut8=96.0875,
                 s3y  =0.0398,
                 s5y  =0.0443,
                 s10y =0.05165,
                 s15y =0.055175)


# Swaption volatility matrix with corresponding maturities and tenors
swaptionMaturities <- c(1,2,3,4,5)

swapTenors <- c(1,2,3,4,5)

volMatrix <- matrix(
    c(0.1490, 0.1340, 0.1228, 0.1189, 0.1148,
      0.1290, 0.1201, 0.1146, 0.1108, 0.1040,
      0.1149, 0.1112, 0.1070, 0.1010, 0.0957,
      0.1047, 0.1021, 0.0980, 0.0951, 0.1270,
      0.1000, 0.0950, 0.0900, 0.1230, 0.1160),
    ncol=5, byrow=TRUE)

legparams=list(dayCounter="Thirty360",
               fixFreq="Annual",
               floatFreq="Semiannual")

setEvaluationDate(as.Date("2016-2-16"))
times<-times <- seq(0,14.75,.25)
dcurve <- DiscountCurve(params, tsQuotes, times=times,legparams)

# Price the Bermudan swaption
pricing <- AffineSwaption(params, dcurve,swaptionMaturities, swapTenors, volMatrix,legparams)
summary(pricing)

}
}
}
\keyword{models}

\name{Calendars}
\alias{isBusinessDay}
\alias{businessDay}
\alias{isHoliday}
\alias{isWeekend}
\alias{isEndOfMonth}
\alias{getEndOfMonth}
\alias{endOfMonth}
\alias{getHolidayList}
\alias{holidayList}
\alias{getBusinessDayList}
\alias{businessDayList}
\alias{setCalendarContext}
\alias{adjust}
\alias{advance}
\alias{businessDaysBetween}
\alias{dayCount}
\alias{yearFraction}
\alias{setEvaluationDate}
\alias{advanceDate}
\alias{addHolidays}
\alias{removeHolidays}
\alias{calendars}
\title{Calendar functions from QuantLib}
\description{
The \code{isBusinessDay} function evaluates the given dates in the context
of the given calendar, and returns a vector of booleans indicating
business day status. \code{BusinessDay} is also
recognised (but may be deprecated one day).

The \code{isHoliday} function evaluates the given dates in the context
of the given calendar, and returns a vector of booleans indicating
holiday day status.

The \code{isWeekend} function evaluates the given dates in the context
of the given calendar, and returns a vector of booleans indicating
weekend status.

The \code{isEndOfMonth} function evaluates the given dates in the context
of the given calendar, and returns a vector of booleans indicating
end of month status.

The \code{getEndOfMonth} function evaluates the given dates in the context
of the given calendar, and returns a vector that corresponds to the end
of month. \code{endOfMonth} is a deprecated form for this function.

The \code{getHolidayList} function returns the holidays between the
given dates, with an option to exclude weekends.  \code{holidayList} is
a deprecated form for this function. Similarly,
\code{getBusinessDayList} and, for symmetry, \code{businessDayList}
return the list of business days; this always excludes weekends.

The \code{adjust} function evaluates the given dates in the context
of the given calendar, and returns a vector that adjusts each input
dates to the appropriate near business day with respect to the given convention.

The \code{advance} function evaluates the given dates in the context
of the given calendar, and returns a vector that advances the given
dates of the given number of business days and returns the result.
This functions gets called either with both argument \code{n} and
\code{timeUnit}, or with argument \code{period}.

The \code{businessDaysBetween} function evaluates two given dates in the context
of the given calendar, and returns a vector that gives the number of
business day between.

The \code{dayCount} function returns the number of day between two dates
given a day counter, see \link{Enum}.

The \code{yearFraction} function returns year fraction between two dates
given a day counter, see \link{Enum}.

The \code{setCalendarContext} function sets three values to a singleton
instance at the C++ layer.

The \code{setEvaluationDate} function sets the evaluation date used by
the QuantLib pricing engines.

The \code{advanceDate} function advances the given date by the given
number of days in the current calendar instance.

The \code{addHolidays} and \code{removeHolidays} add (and remove)
holidays to (from) the given calendar. Note that this change is
transitory and does not persist the session as all actual calendar
information comes from the QuantLib library that this package is linked
against.

The \code{calendars} vector contains all calendar identifiers.

}
\usage{
isBusinessDay(calendar, dates)
businessDay(calendar="TARGET", dates=Sys.Date())  # deprecated form
isHoliday(calendar, dates)
isWeekend(calendar, dates)
isEndOfMonth(calendar, dates)
getEndOfMonth(calendar, dates)
endOfMonth(calendar="TARGET", dates=Sys.Date())
getHolidayList(calendar, from, to, includeWeekends=FALSE)
holidayList(calendar="TARGET", from=Sys.Date(), to = Sys.Date() + 5,
            includeWeekends=FALSE)
getBusinessDayList(calendar, from, to)
businessDayList(calendar="TARGET", from=Sys.Date(), to = Sys.Date() + 5)
adjust(calendar, dates, bdc = 0L)
advance(calendar="TARGET", dates=Sys.Date(), n, timeUnit, period, bdc = 0, emr =0)
%advance(calendar="TARGET", dates=Sys.Date(), period, bdc = 0, emr =0)
businessDaysBetween(calendar, from, to, includeFirst = TRUE, includeLast = FALSE)
dayCount(startDates, endDates, dayCounters)
yearFraction(startDates, endDates, dayCounters)
setCalendarContext(calendar, fixingDays, settleDate)
setEvaluationDate(evalDate)
addHolidays(calendar, dates)
removeHolidays(calendar, dates)
}
\arguments{
  \item{calendar}{A string identifying one of the supported QuantLib
    calendars, see Details for more}
  \item{dates}{A vector (or scalar) of \code{Date} types.}
  \item{from}{A vector (or scalar) of \code{Date} types.}
  \item{to}{A vector (or scalar) of \code{Date} types.}
  \item{includeWeekends}{boolean that indicates whether the calculation
    should include the weekends. Default = false}
  \item{fixingDays}{An integer for the fixing day period, defaults to 2.}
  \item{settleDate}{A date on which trades settles, defaults to two days
    after the current day.}
  \item{n}{an integer number}
  \item{timeUnit}{A value of 0,1,2,3 that corresponds to Days, Weeks,
    Months, and Year; for more detail, see the QuantLib documentation at
    \url{https://www.quantlib.org//reference/group__datetime.html}}
  \item{period}{See \link{Enum}}
  \item{bdc}{Business day convention. By default, this value is 0 and correspond to Following convention}
  \item{emr}{End Of Month rule, default is false}
  \item{includeFirst}{boolean that indicates whether the calculation
    should include the first day. Default = true}
  \item{includeLast}{Default = false}
  \item{startDates}{A vector of \code{Date} type.}
  \item{endDates}{A vector of \code{Date} type.}
  \item{dayCounters}{A vector of numeric type. See \link{Enum}}
  \item{evalDate}{A single date used for the pricing valuations.}
}
\value{
  A named vector of booleans each of which is true if the corresponding
  date is a business day (or holiday or weekend) in the given calendar.
  The element names are
  the dates (formatted as text in yyyy-mm-dd format).

  For \code{setCalendarContext}, a boolean or NULL in case of error.
}
\details{
  The calendars are coming from QuantLib, and the QuantLib documentation
  should be consulted for details.

  Currently, the following strings are recognised: TARGET (a default
  calendar), Argentina, Australia, Brazil, Canada and Canada/Settlement,
  Canada/TSX, China, CzechRepublic, Denmark, Finland, Germany and
  Germany/FrankfurtStockExchange, Germany/Settlement, Germany/Xetra,
  Germany/Eurex, HongKong, Hungary, Iceland, India, Indonesia, Italy and
  Italy/Settlement, Italy/Exchange, Japan, Mexico, NewZealand, Norway,
  Poland, Russia, SaudiArabia, Singapore, Slovakia, SouthAfrica,
  SouthKorea, SouthKorea/KRX, Sweden, Switzerland, Taiwan, Turkey,
  Ukraine, UnitedKingdom and UnitedKingdom/Settlement,
  UnitedKingdom/Exchange, UnitedKingdom/Metals, UnitedStates and
  UnitedStates/Settlement, UnitedStates/NYSE,
  UnitedStates/GovernmentBond, UnitedStates/NERC and WeekendsOnly.

  (In case of multiples entries per country, the country default is listed
  right after the country itself. Using the shorter form is equivalent.)

}
\references{\url{https://www.quantlib.org/} for details on \code{QuantLib}.}
\author{Dirk Eddelbuettel \email{edd@debian.org} for the \R interface;
  the QuantLib Group for \code{QuantLib}}
\note{The interface might change in future release as \code{QuantLib}
  stabilises its own API.}
\examples{
  dates <- seq(from=as.Date("2009-04-07"), to=as.Date("2009-04-14"), by=1)
  isBusinessDay("UnitedStates", dates)
  isBusinessDay("UnitedStates/Settlement", dates)      ## same as previous
  isBusinessDay("UnitedStates/NYSE", dates)            ## stocks
  isBusinessDay("UnitedStates/GovernmentBond", dates)  ## bonds
  isBusinessDay("UnitedStates/NERC", dates)            ## energy

  isHoliday("UnitedStates", dates)
  isHoliday("UnitedStates/Settlement", dates)      ## same as previous
  isHoliday("UnitedStates/NYSE", dates)            ## stocks
  isHoliday("UnitedStates/GovernmentBond", dates)  ## bonds
  isHoliday("UnitedStates/NERC", dates)            ## energy

  isWeekend("UnitedStates", dates)
  isWeekend("UnitedStates/Settlement", dates)      ## same as previous
  isWeekend("UnitedStates/NYSE", dates)            ## stocks
  isWeekend("UnitedStates/GovernmentBond", dates)  ## bonds
  isWeekend("UnitedStates/NERC", dates)            ## energy

  isEndOfMonth("UnitedStates", dates)
  isEndOfMonth("UnitedStates/Settlement", dates)      ## same as previous
  isEndOfMonth("UnitedStates/NYSE", dates)            ## stocks
  isEndOfMonth("UnitedStates/GovernmentBond", dates)  ## bonds
  isEndOfMonth("UnitedStates/NERC", dates)            ## energy

  getEndOfMonth("UnitedStates", dates)
  getEndOfMonth("UnitedStates/Settlement", dates)      ## same as previous
  getEndOfMonth("UnitedStates/NYSE", dates)            ## stocks
  getEndOfMonth("UnitedStates/GovernmentBond", dates)  ## bonds
  getEndOfMonth("UnitedStates/NERC", dates)            ## energy

  from <- as.Date("2009-04-07")
  to<-as.Date("2009-04-14")
  getHolidayList("UnitedStates", from, to)
  to <- as.Date("2009-10-7")
  getHolidayList("UnitedStates", from, to)

  dates <- seq(from=as.Date("2009-04-07"), to=as.Date("2009-04-14"), by=1)

  adjust("UnitedStates", dates)
  adjust("UnitedStates/Settlement", dates)      ## same as previous
  adjust("UnitedStates/NYSE", dates)            ## stocks
  adjust("UnitedStates/GovernmentBond", dates)  ## bonds
  adjust("UnitedStates/NERC", dates)            ## energy

  advance("UnitedStates", dates, 10, 0)
  advance("UnitedStates/Settlement", dates, 10, 1)      ## same as previous
  advance("UnitedStates/NYSE", dates, 10, 2)            ## stocks
  advance("UnitedStates/GovernmentBond", dates,  10, 3) ## bonds
  advance("UnitedStates/NERC", dates, period = 3)       ## energy

  from <- as.Date("2009-04-07")
  to<-as.Date("2009-04-14")
  businessDaysBetween("UnitedStates", from, to)

  startDates <- seq(from=as.Date("2009-04-07"), to=as.Date("2009-04-14"),by=1)
  endDates <- seq(from=as.Date("2009-11-07"), to=as.Date("2009-11-14"), by=1)
  dayCounters <- c(0,1,2,3,4,5,6,1)
  dayCount(startDates, endDates, dayCounters)
  yearFraction(startDates, endDates, dayCounters)

  head(calendars, 10)
}
\keyword{misc}

\name{ConvertibleBond}
\alias{ConvertibleFixedCouponBond}
\alias{ConvertibleFixedCouponBond.default}
\alias{ConvertibleFloatingCouponBond}
\alias{ConvertibleFloatingCouponBond.default}
\alias{ConvertibleZeroCouponBond}
\alias{ConvertibleZeroCouponBond.default}
\title{Convertible Bond evaluation for Fixed, Floating and Zero Coupon}
\description{
The \code{ConvertibleFixedCouponBond} function setups and evaluates a
ConvertibleFixedCouponBond using QuantLib's BinomialConvertibleEngine
%\url{https://www.quantlib.org/reference/class_quant_lib_1_1_binomial_convertible_engine.html}
and BlackScholesMertonProcess
%\url{https://www.quantlib.org/reference/class_quant_lib_1_1_black_scholes_merton_process.html}.
The NPV, clean price, dirty price, accrued interest, yield and cash flows of
the bond is returned. For detail, see test-suite/convertiblebond.cpp

The \code{ConvertibleFloatingCouponBond} function setups and evaluates a
ConvertibleFixedCouponBond using QuantLib's BinomialConvertibleEngine
% \url{https://www.quantlib.org/reference/class_quant_lib_1_1_binomial_convertible_engine.html}
and BlackScholesMertonProcess
% \url{https://www.quantlib.org/reference/class_quant_lib_1_1_black_scholes_merton_process.html}.
The NPV, clean price, dirty price, accrued interest, yield and cash flows of
the bond is returned. For detail, see test-suite/convertiblebond.cpp

The \code{ConvertibleZeroCouponBond} function setups and evaluates a
ConvertibleFixedCouponBond using QuantLib's BinomialConvertibleEngine
% \url{https://www.quantlib.org/reference/class_quant_lib_1_1_binomial_convertible_engine.html}
and BlackScholesMertonProcess
% \url{https://www.quantlib.org/reference/class_quant_lib_1_1_black_scholes_merton_process.html}.
The NPV, clean price, dirty price, accrued interest, yield and cash flows of
the bond is returned. For detail, see \code{test-suite/convertiblebond.cpp}.
}
\usage{
\method{ConvertibleFloatingCouponBond}{default}(bondparams, iborindex, spread, process, dateparams)
\method{ConvertibleFixedCouponBond}{default}(bondparams, coupon, process, dateparams)
\method{ConvertibleZeroCouponBond}{default}(bondparams, process, dateparams)
}
\arguments{

\item{bondparams}{bond parameters, a named list whose elements are:
	\tabular{ll}{
       \code{issueDate}     \tab a Date, the bond's issue date\cr
       \code{maturityDate}  \tab a Date, the bond's maturity date\cr
       \code{creditSpread}  \tab a double, credit spread parameter \cr
       \code{}        \tab  in the constructor of the bond. \cr
       \code{conversitionRatio}  \tab a double, conversition ratio \cr
       \code{}        \tab parameter in the constructor of the bond. \cr
       \code{exercise} 	\tab (Optional) a string, either "eu" for European \cr
       \code{}        \tab 	option, or "am" for American option. \cr
       \code{}        \tab  Default value is 'am'.\cr
       \code{faceAmount}    \tab (Optional) a double, face amount of the bond.\cr
       \code{}        \tab  Default value is 100. \cr
       \code{redemption}    \tab (Optional) a double, percentage of the initial \cr
       \code{}        \tab 	face amount that will be returned at maturity \cr
       \code{}        \tab    date. Default value is 100.\cr
       \code{divSch} \tab (Optional) a data frame whose columns are \cr
       \code{}        \tab "Type", "Amount", "Rate", and "Date" \cr
       \code{}        \tab corresponding to QuantLib's DividendSchedule. \cr
       \code{}        \tab  Default value is an empty frame, or no dividend. \cr
       \code{callSch} \tab (Optional) a data frame whose columns are "Price",\cr
       \code{}        \tab "Type" and "Date" corresponding to QuantLib's \cr
       \code{}        \tab CallabilitySchedule. Defaule is an empty frame, \cr
       \code{}         \tab or no callability.\cr
     }
   }

\item{iborindex}{a DiscountCurve object, represents an IborIndex}
\item{spread}{ a double vector, represents paramter 'spreads' in ConvertibleFloatingBond's constructor. }
\item{coupon}{a double vector of coupon rate}
\item{process}{arguments to construct a BlackScholes process and set up the binomial pricing engine for this bond.
  \tabular{ll}{
    \code{underlying} \tab a double, flat underlying term structure \cr
    \code{volatility} \tab a double, flat volatility term structure \cr
    \code{dividendYield} \tab a DiscountCurve object \cr
    \code{riskFreeRate} \tab a DiscountCurve object \cr
  }
}

\item{dateparams}{(Optional) a named list, QuantLib's date parameters of the bond.
  \tabular{ll}{
    \code{settlementDays} \tab (Optional) a double, settlement days. \cr
    \code{}        \tab Default value is 1.\cr
    \code{calendar} \tab (Optional) a string, either 'us' or 'uk' \cr
    \code{}        \tab corresponding to US Goverment Bond \cr
    \code{}		   \tab calendar and UK Exchange calendar.\cr
    \code{}        \tab  Default value is 'us'.\cr
    \code{dayCounter} \tab (Optional) a number or string, \cr
    \code{}        \tab day counter convention.\cr
    \code{}        \tab  See \link{Enum}. Default value is 'Thirty360' \cr
    \code{period}  \tab (Optional) a number or string, \cr
    \code{}       \tab  interest compounding interval. See \link{Enum}. \cr
    \code{}       \tab Default value is 'Semiannual'.\cr
    \code{businessDayConvention} \tab (Optional) a number or string, \cr
    \code{}                 \tab business day convention. \cr
    \tab See \link{Enum}. Default value is 'Following'. \cr
  }
  See the examples below.
}
}
\value{
  The \code{ConvertibleFloatingCouponBond} function returns an object of class
  \code{ConvertibleFloatingCouponBond} (which inherits from class
  \code{Bond}). It contains a list with the following
  components:
  \item{NPV}{net present value of the bond}
  \item{cleanPrice}{price price of the bond}
  \item{dirtyPrice}{dirty price of the bond}
  \item{accruedAmount}{accrued amount of the bond}
  \item{yield}{yield of the bond}
  \item{cashFlows}{cash flows of the bond}

  The \code{ConvertibleFixedCouponBond} function returns an object of class
  \code{ConvertibleFixedCouponBond} (which inherits from class
  \code{Bond}). It contains a list with the following
  components:
  \item{NPV}{net present value of the bond}
  \item{cleanPrice}{price price of the bond}
  \item{dirtyPrice}{dirty price of the bond}
  \item{accruedAmount}{accrued amount of the bond}
  \item{yield}{yield of the bond}
  \item{cashFlows}{cash flows of the bond}

  The \code{ConvertibleZeroCouponBond} function returns an object of class
  \code{ConvertibleZeroCouponBond} (which inherits from class
  \code{Bond}). It contains a list with the following
  components:
  \item{NPV}{net present value of the bond}
  \item{cleanPrice}{price price of the bond}
  \item{dirtyPrice}{dirty price of the bond}
  \item{accruedAmount}{accrued amount of the bond}
  \item{yield}{yield of the bond}
  \item{cashFlows}{cash flows of the bond}

}
\details{
 Please see any decent Finance textbook for background reading, and the
  \code{QuantLib} documentation for details on the \code{QuantLib}
  implementation.
}
\references{
  \url{https://www.quantlib.org/} for details on \code{QuantLib}.
}
\author{Khanh Nguyen \email{knguyen@cs.umb.edu} for the inplementation; Dirk Eddelbuettel \email{edd@debian.org} for the \R interface;
  the QuantLib Group for \code{QuantLib}
}
\examples{
# commented-out as it runs longer than CRAN likes
\dontrun{
#this follow an example in test-suite/convertiblebond.cpp
params <- list(tradeDate=Sys.Date()-2,
               settleDate=Sys.Date(),
               dt=.25,
               interpWhat="discount",
               interpHow="loglinear")


dividendYield <- DiscountCurve(params, list(flat=0.02))
riskFreeRate <- DiscountCurve(params, list(flat=0.05))

dividendSchedule <- data.frame(Type=character(0), Amount=numeric(0),
                            Rate = numeric(0), Date = as.Date(character(0)))
callabilitySchedule <- data.frame(Price = numeric(0), Type=character(0),
                          Date = as.Date(character(0)))

process <- list(underlying=50, divYield = dividendYield,
                rff = riskFreeRate, volatility=0.15)

today <- Sys.Date()
bondparams <- list(exercise="am", faceAmount=100,
                   divSch = dividendSchedule,
                   callSch = callabilitySchedule,
                   redemption=100,
                   creditSpread=0.005,
                   conversionRatio = 0.0000000001,
                   issueDate=as.Date(today+2),
                   maturityDate=as.Date(today+3650))
dateparams <- list(settlementDays=3,
                   dayCounter="ActualActual",
                   period = "Semiannual", calendar = "UnitedStates/GovernmentBond",
                   businessDayConvention="Following")

lengths <- c(2,4,6,8,10,12,14,16,18,20,22,24,26,28,30)
coupons <- c( 0.0200, 0.0225, 0.0250, 0.0275, 0.0300,
              0.0325, 0.0350, 0.0375, 0.0400, 0.0425,
              0.0450, 0.0475, 0.0500, 0.0525, 0.0550 )
marketQuotes <- rep(100, length(lengths))
curvedateparams <- list(settlementDays=0, period="Annual",
                   dayCounter="ActualActual",
                  businessDayConvention ="Unadjusted")
curveparams <- list(method="ExponentialSplinesFitting",
                    origDate = Sys.Date())
curve <- FittedBondCurve(curveparams, lengths, coupons, marketQuotes, curvedateparams)
iborindex <- list(type="USDLibor", length=6,
                  inTermOf="Month", term=curve)
spreads <- c()
#ConvertibleFloatingCouponBond(bondparams, iborindex, spreads, process, dateparams)


#example using default values
#ConvertibleFloatingCouponBond(bondparams, iborindex,spreads, process)

dateparams <- list(settlementDays=3,
                   period = "Semiannual",
                   businessDayConvention="Unadjusted")

bondparams <- list(
                   creditSpread=0.005, conversionRatio = 0.0000000001,
                   issueDate=as.Date(today+2),
                   maturityDate=as.Date(today+3650))
#ConvertibleFloatingCouponBond(bondparams, iborindex,
#spreads, process, dateparams)



#this follow an example in test-suite/convertiblebond.cpp
#for ConvertibleFixedCouponBond

#set up arguments to build a pricing engine.
params <- list(tradeDate=Sys.Date()-2,
               settleDate=Sys.Date(),
               dt=.25,
               interpWhat="discount",
               interpHow="loglinear")
times <- seq(0,10,.1)

dividendYield <- DiscountCurve(params, list(flat=0.02), times)
riskFreeRate <- DiscountCurve(params, list(flat=0.05), times)

dividendSchedule <- data.frame(Type=character(0), Amount=numeric(0),
                            Rate = numeric(0), Date = as.Date(character(0)))
callabilitySchedule <- data.frame(Price = numeric(0), Type=character(0),
                          Date = as.Date(character(0)))

process <- list(underlying=50, divYield = dividendYield,
                rff = riskFreeRate, volatility=0.15)

today <- Sys.Date()
bondparams <- list(exercise="am", faceAmount=100, divSch = dividendSchedule,
                   callSch = callabilitySchedule, redemption=100,
                   creditSpread=0.005, conversionRatio = 0.0000000001,
                   issueDate=as.Date(today+2),
                   maturityDate=as.Date(today+3650))
dateparams <- list(settlementDays=3,
                   dayCounter="Actual360",
                   period = "Once", calendar = "UnitedStates/GovernmentBond",
                   businessDayConvention="Following"
                   )
coupon <- c(0.05)
ConvertibleFixedCouponBond(bondparams, coupon, process, dateparams)

#example with default value
ConvertibleFixedCouponBond(bondparams, coupon, process)

dateparams <- list(settlementDays=3,
                   dayCounter="Actual360")
ConvertibleFixedCouponBond(bondparams, coupon, process, dateparams)

bondparams <- list(creditSpread=0.005, conversionRatio = 0.0000000001,
                   issueDate=as.Date(today+2),
                   maturityDate=as.Date(today+3650))
ConvertibleFixedCouponBond(bondparams, coupon, process, dateparams)



#this follow an example in test-suite/convertiblebond.cpp
params <- list(tradeDate=Sys.Date()-2,
               settleDate=Sys.Date(),
               dt=.25,
               interpWhat="discount",
               interpHow="loglinear")
times <- seq(0,10,.1)


dividendYield <- DiscountCurve(params, list(flat=0.02), times)
riskFreeRate <- DiscountCurve(params, list(flat=0.05), times)

dividendSchedule <- data.frame(Type=character(0), Amount=numeric(0),
                            Rate = numeric(0), Date = as.Date(character(0)))
callabilitySchedule <- data.frame(Price = numeric(0), Type=character(0),
                          Date = as.Date(character(0)))

process <- list(underlying=50, divYield = dividendYield,
                rff = riskFreeRate, volatility=0.15)

today <- Sys.Date()
bondparams <- list(exercise="am", faceAmount=100, divSch = dividendSchedule,
                   callSch = callabilitySchedule, redemption=100,
                   creditSpread=0.005, conversionRatio = 0.0000000001,
                   issueDate=as.Date(today+2),
                   maturityDate=as.Date(today+3650))
dateparams <- list(settlementDays=3,
                   dayCounter="Actual360",
                   period = "Once", calendar = "UnitedStates/GovernmentBond",
                   businessDayConvention="Following"
                   )

ConvertibleZeroCouponBond(bondparams, process, dateparams)

#example with default values
ConvertibleZeroCouponBond(bondparams, process)


bondparams <- list(creditSpread=0.005,
                   conversionRatio=0.0000000001,
                   issueDate=as.Date(today+2),
                   maturityDate=as.Date(today+3650))

dateparams <- list(settlementDays=3, dayCounter='Actual360')
ConvertibleZeroCouponBond(bondparams, process, dateparams)
ConvertibleZeroCouponBond(bondparams, process)
}
}

\name{SabrSwaption}
\alias{SabrSwaption}
\alias{SabrSwaption.default}
\title{SABR swaption using vol cube data with bermudan alternative using markovfunctional}
\description{
  \code{SabrSwaption} prices a swaption with specified
  expiration or time range if Bermudan, strike, and maturity, using quantlibs SABR model for europeans
  and quantlib's markovfunctional for Bermudans.  Currently the input is a zero offset log-normal vol surface.
  An example of a dataset can be found in the dataset \code{rqlib} inlcuded with Rquantlib. It is assumed that the
  swaption is
  exercisable  at the start of a forward start swap if \code{params$european} flag is set to \code{TRUE} or starting
  immediately on each reset date (Bermudan) of an existing underlying swap or spot start
  swap if \code{params$european} flag is set to \code{FALSE}.
}
\usage{
SabrSwaption(params, ts, volCubeDF,
legparams = list(dayCounter = "Thirty360", fixFreq = "Annual", floatFreq = "Semiannual"),
tsUp01 = NA, tsDn01 = NA, vega = FALSE)}
\arguments{
  \item{params}{A list specifying the \code{tradeDate} (month/day/year),
    \code{settlementDate}, logical flags \code{payFixed} & \code{european}
    (european=FALSE generates Bermudan vlaue), \code{strike}, pricing \code{method}, and curve
    construction options (see \emph{Examples} section below). Curve construction options are
    \code{interpWhat} (possible values are \code{discount},
    \code{forward}, and \code{zero}) and
    \code{interpHow} (possible values are \code{linear},
    \code{loglinear}
    , and \code{spline}). Both \code{interpWhat} and \code{interpHow}
    are ignored when a flat yield curve is requested, but they must be
    present nevertheless.
    }
  \item{ts}{A term structure built with DiscountCurve is required. See the
    help page for \code{\link{DiscountCurve}} and example below for details.}
  \item{volCubeDF}{The swaption volatility cube in dataframe format with columns Expiry, Tenor, Spread,
  and LogNormalVol stored by rows. See the example below.}
    \item{legparams}{A list specifying the \code{dayCounter} the day count convention for the
   fixed leg (default is Thirty360), and  \code{fixFreq}, fixed coupon frequecny (defualt is Annual),
   \code{floatFreq}, floating leg reset frequency (default is Semiannual).
  }
    \item{tsUp01}{Discount for a user specied up move in rates.}
    \item{tsDn01}{Discount for a user specied down move in rates.}
    \item{vega}{Discount for a user specied up move.}
 }
\value{
  \code{SabrSwaption} returns a list containing the value of the payer and receiver swaptions at the
  strike specified in \code{params}.
  \item{NPV}{NPV of swaption in basis points (actual price
  equals \code{price} times notional divided by 10,000)}
  \item{strike}{swaption strike}
  \item{params}{Input parameter list}
  \item{atmRate}{fair rate for swap at swap start date for european or fair swap rate
  for swap at expiration for bermudan}
  \item{vol}{vol for swaption at swap start date and rate strike for european or vol for swaption
  for given expiration and strike for bermudan}
  \item{rcvDv01}{reveiver value for a change in rates defined by dv01Up}
  \item{payDv01}{payer value for a change in rates defined by dv01Up}
  \item{rcvCnvx}{reveiver second order value change for a change in rates defined by dv01Up & dv01Dn}
  \item{payCnvx}{payer second order value for a change in rates defined by dv01Up & dv01Dn}
  \item{strike}{swaption strike}

}
\details{
  This function is based on \code{QuantLib} Version 1.64. It
  introduces support for fixed-income instruments in \code{RQuantLib}.


}
\references{
  Brigo, D. and Mercurio, F. (2006) \emph{Interest Rate Models: Theory and
    Practice, 2nd Edition}, Springer-Verlag, New York.

  For information about \code{QuantLib} see \url{https://www.quantlib.org/}.

  For information about \code{RQuantLib} see
\url{http://dirk.eddelbuettel.com/code/rquantlib.html}.

}
\author{Terry Leitch}

\seealso{\code{\link{AffineSwaption}}}

\examples{

params <- list(tradeDate=as.Date('2016-2-15'),
               settleDate=as.Date('2016-2-17'),
               startDate=as.Date('2017-2-17'),
               maturity=as.Date('2022-2-17'),
               european=TRUE,
               dt=.25,
               expiryDate=as.Date('2017-2-17'),
               strike=.02,
               interpWhat="discount",
               interpHow="loglinear")

# Set leg paramters for generating discount curve
dclegparams=list(dayCounter="Thirty360",
                 fixFreq="Annual",
                 floatFreq="Semiannual")

setEvaluationDate(as.Date("2016-2-15"))
times<-times <- seq(0,14.75,.25)

data(tsQuotes)
dcurve <- DiscountCurve(params, tsQuotes, times=times,dclegparams)

# Price the Bermudan swaption
swaplegparams=list(fixFreq="Semiannual",floatFreq="Quarterly")

data(vcube)
pricing <- SabrSwaption(params, dcurve,vcube,swaplegparams)
pricing

}
\keyword{models}

\name{FixedRateBond}
\alias{FixedRateBond}
\alias{FixedRateBond.default}
\alias{FixedRateBondPriceByYield}
\alias{FixedRateBondPriceByYield.default}
\alias{FixedRateBondYield}
\alias{FixedRateBondYield.default}
\title{Fixed-Rate bond pricing}
\description{
  The \code{FixedRateBond} function evaluates a fixed rate bond using discount curve, the yield or the clean price. 
  More specificly, when a discount curve is provided the calculation is done by DiscountingBondEngine from QuantLib.
  The NPV, clean price, dirty price, accrued interest, yield, duration, actual settlement date and cash flows of the bond is returned.
  When a yield is provided instead, no engine is provided to the bond class and prices are computed from yield. In the latter case, NPV is set to NA. Same situation when the clean price is given instead of discount curve or yield.
  For more detail, see the source codes in QuantLib's file \code{test-suite/bond.cpp}.

  The \code{FixedRateBondPriceByYield} function calculates the theoretical price of a fixed rate bond from its yield.

  The \code{FixedRateBondYield} function calculates the theoretical yield of a fixed rate bond from its price.
}
\usage{

\method{FixedRateBond}{default}(bond, rates, schedule,
                                calc=list(dayCounter='ActualActual.ISMA',
                                          compounding='Compounded',
                                          freq='Annual',
                                          durationType='Modified'),
                                discountCurve = NULL, yield = NA, price = NA)

\method{FixedRateBondPriceByYield}{default}( settlementDays=1, yield, faceAmount=100, 
                                 effectiveDate, maturityDate,
                                 period, calendar="UnitedStates/GovernmentBond", 
                                 rates, dayCounter=2,
                                 businessDayConvention=0, compound = 0, redemption=100, 
                                 issueDate)

\method{FixedRateBondYield}{default}( settlementDays=1, price, faceAmount=100, 
                                 effectiveDate, maturityDate,
                                 period, calendar="UnitedStates/GovernmentBond", 
                                 rates, dayCounter=2,
                                 businessDayConvention=0, 
                                 compound = 0, redemption=100, 
                                 issueDate)
}
\arguments{
\item{bond}{(Optional) bond parameters, a named list whose elements are: 
  \tabular{ll}{
    \code{settlementDays}      \tab (Optional) a double, settlement days. \cr 
    \code{}                    \tab Default value is 1.\cr
    \code{faceAmount}          \tab (Optional) a double, face amount of the bond.\cr
    \code{}                    \tab  Default value is 100. \cr
    \code{dayCounter}          \tab (Optional) a number or string, \cr 
    \code{}                    \tab day counter convention. Defaults to 'Thirty360' \cr
    \code{issueDate}           \tab (Optional) a Date, the bond's issue date\cr
    \code{}                    \tab Defaults to QuantLib default. \cr
    \code{paymentConvention}   \tab (Optional) a number or string, the bond \cr
    \code{}                    \tab payment convention. \cr
    \code{}                    \tab Defaults to QuantLib default. \cr
    \code{redemption}          \tab (Optional) a double, the redemption amount. \cr
    \code{}                    \tab Defaults to QuantLib default (100). \cr
    \code{paymentCalendar}     \tab (Optional) a string, the name of the calendar. \cr
    \code{}                    \tab Defaults to QuantLib default. \cr
    \code{exCouponPeriod}      \tab (Optional) a number, the number of days when \cr
    \code{}                    \tab the coupon goes ex relative to the coupon date. \cr
    \code{}                    \tab Defaults to QuantLib default. \cr
    \code{exCouponCalendar}    \tab (Optional) a string, the name of the \cr
    \code{}                    \tab ex-coupon calendar. \cr
    \code{}                    \tab Defaults to QuantLib default. \cr
    \code{exCouponConvention}  \tab (Optional) a number or string, the coupon \cr
    \code{}                    \tab payment convention. \cr
    \code{}                    \tab Defaults to QuantLib default. \cr
    \code{exCouponEndOfMonth}  \tab (Optional) 1 or 0, use End of Month rule for \cr
    \code{}                    \tab ex-coupon dates. Defaults to 0 (false).
  }
}

\item{rates}{a numeric vector, bond's coupon rates}

\item{schedule}{(Optional) a named list, QuantLib's parameters of the bond's schedule. 
  \tabular{ll}{
    \code{effectiveDate}       \tab a Date, when the schedule becomes effective. \cr 
    \code{maturityDate}        \tab a Date, when the schedule matures. \cr 
    \code{period}              \tab (Optional) a number or string, the frequency of \cr
    \code{}                    \tab the schedule. Default value is 'Semiannual'. \cr
    \code{calendar}            \tab (Optional) a string, the calendar name. \cr 
    \code{}                    \tab Defaults to 'TARGET' \cr
    \code{businessDayConvention} \tab (Optional) a number or string, the \cr 
    \code{}                    \tab day convention to use. \cr
    \code{}                    \tab Defaults to 'Following'. \cr
    \code{terminationDateConvention} \tab (Optional) a number or string, the \cr 
    \code{}                    \tab day convention to use for the terminal date. \cr
    \code{}                    \tab Defaults to 'Following'. \cr
    \code{dateGeneration}      \tab (Optional) a number or string, the \cr 
    \code{}                    \tab date generation rule. \cr
    \code{}                    \tab Defaults to 'Backward'. \cr
    \code{endOfMonth}          \tab (Optional) 1 or 0, use End of Month rule for \cr
    \code{}                    \tab schedule dates. Defaults to 0 (false).
  }
  See example below.
}

\item{calc}{(Optional) a named list, QuantLib's parameters for calculations. 
  \tabular{ll}{
    \code{dayCounter}          \tab (Optional) a number or string, day counter \cr 
    \code{}                    \tab convention. Defaults to 'ActualActual.ISMA' \cr
    \code{compounding}         \tab a string, what kind of compounding to use. \cr 
    \code{}                    \tab Defaults to 'Compounded' \cr
    \code{freq}                \tab (Optional) a number or string, the frequency \cr
    \code{}                    \tab to use. Default value is 'Annual'. \cr
    \code{durationType}        \tab (Optional) a number or string, the type of \cr 
    \code{}                    \tab duration to calculate. Defaults to 'Simple' \cr
    \code{accuracy}            \tab (Optional) a number, the accuracy required. \cr 
    \code{}                    \tab Defaults to 1.0e-8. \cr
    \code{maxEvaluations}      \tab (Optional) a number, max number of iterations. \cr 
    \code{}                    \tab Defaults to 100.
  }
}

\item{discountCurve}{Can be one of the following:
  \tabular{ll}{
    \code{a DiscountCurve} \tab  a object of DiscountCurve class \cr
    \code{}                \tab For more detail, see example or \cr 
    \code{}                \tab the discountCurve function \cr
    \code{A 2 items list} \tab specifies a flat curve in two \cr 
    \code{}               \tab values "todayDate" and "rate"   \cr
    \code{A 3 items list} \tab specifies three values to construct a \cr
    \code{}                \tab DiscountCurve object, "params" , \cr 
    \code{} \tab "tsQuotes", "times". \cr
    \code{}                \tab For more detail, see example or \cr 
    \code{}                \tab the discountCurve function
  }
}

\item{yield}{yield of the bond}
\item{price}{clean price of the bond}
\item{settlementDays}{an integer, 1 for T+1, 2 for T+2, etc...}
\item{effectiveDate}{bond's effective date}
\item{maturityDate}{bond's maturity date}
\item{period}{frequency of events,0=NoFrequency, 1=Once, 2=Annual, 3=Semiannual, 4=EveryFourthMonth, 5=Quarterly, 6=Bimonthly ,7=Monthly ,8=EveryFourthWeek,9=Biweekly, 10=Weekly, 11=Daily. For more information, see QuantLib's Frequency class }
\item{calendar}{Business Calendar. Either \code{us} or \code{uk}}
\item{faceAmount}{face amount of the bond}
\item{businessDayConvention}{convention used to adjust a date in case it is not a valid business day. See quantlib for more detail. 0 = Following, 1 = ModifiedFollowing, 2 = Preceding, 3 = ModifiedPreceding, other = Unadjusted}
\item{dayCounter}{day count convention. 0 = Actual360(), 1 = Actual365Fixed(), 2 = ActualActual(), 3 = Business252(), 4 = OneDayCounter(), 5 = SimpleDayCounter(), all other = Thirty360(). For more information, see QuantLib's DayCounter class}
\item{compound}{compounding type. 0=Simple, 1=Compounded, 2=Continuous, all other=SimpleThenCompounded. See QuantLib's Compound class}
\item{redemption}{redemption when the bond expires}
\item{issueDate}{date the bond is issued}

}
\value{
  The \code{FixedRateBond} function returns an object of class
  \code{FixedRateBond} (which inherits from class 
  \code{Bond}). It contains a list with the following
  components:
  \item{NPV}{net present value of the bond}
  \item{cleanPrice}{clean price of the bond}
  \item{dirtyPrice}{dirty price of the bond}
  \item{accruedAmount}{accrued amount of the bond}
  \item{yield}{yield of the bond}
  \item{duration}{the duration of the bond}
  \item{settlementDate}{the actual settlement date used for the bond}
  \item{cashFlows}{cash flows of the bond}

  The \code{FixedRateBondPriceByYield} function returns an object of class
  \code{FixedRateBondPriceByYield} (which inherits from class 
  \code{Bond}). It contains a list with the following
  components:
  \item{price}{price of the bond}

  The \code{FixedRateBondYield} function returns an object of class
  \code{FixedRateBondYield} (which inherits from class 
  \code{Bond}). It contains a list with the following
  components:
  \item{yield}{yield of the bond}
}
\details{
  A discount curve is built to calculate the bond value. 

  Please see any decent Finance textbook for background reading, and the
  \code{QuantLib} documentation for details on the \code{QuantLib}
  implementation.  
}
\references{\url{https://www.quantlib.org/} for details on \code{QuantLib}.}

\author{Khanh Nguyen \email{knguyen@cs.umb.edu} for the inplementation; Dirk Eddelbuettel \email{edd@debian.org} for the \R interface;
  the QuantLib Group for \code{QuantLib}}
\note{The interface might change in future release as \code{QuantLib}
  stabilises its own API.}


\examples{

#Simple call with a flat curve
bond <- list(settlementDays=1,
             issueDate=as.Date("2004-11-30"),
             faceAmount=100,
             dayCounter='Thirty360',
             paymentConvention='Unadjusted')
schedule <- list(effectiveDate=as.Date("2004-11-30"),
                 maturityDate=as.Date("2008-11-30"),
                 period='Semiannual',
                 calendar='UnitedStates/GovernmentBond',
                 businessDayConvention='Unadjusted',
                 terminationDateConvention='Unadjusted',
                 dateGeneration='Forward',
                 endOfMonth=1)
calc=list(dayCounter='Actual360',
          compounding='Compounded',
          freq='Annual',
          durationType='Modified')
coupon.rate <- c(0.02875)
                       
params <- list(tradeDate=as.Date('2002-2-15'),
               settleDate=as.Date('2002-2-19'),
               dt=.25,
               interpWhat="discount",
               interpHow="loglinear")
setEvaluationDate(as.Date("2004-11-22"))

discountCurve.flat <- DiscountCurve(params, list(flat=0.05))
FixedRateBond(bond,
              coupon.rate,
              schedule,
              calc,
              discountCurve=discountCurve.flat)


#Same bond with a discount curve constructed from market quotes
tsQuotes <- list(d1w  =0.0382,
                 d1m  =0.0372,
                 fut1=96.2875,
                 fut2=96.7875,
                 fut3=96.9875,
                 fut4=96.6875,
                 fut5=96.4875,
                 fut6=96.3875,
                 fut7=96.2875,
                 fut8=96.0875,
                 s3y  =0.0398,
                 s5y  =0.0443,
                 s10y =0.05165,
                 s15y =0.055175)
tsQuotes <- list("flat" = 0.02)		## While discount curve code is buggy

discountCurve <- DiscountCurve(params, tsQuotes)
FixedRateBond(bond,
              coupon.rate,
              schedule,
              calc,
              discountCurve=discountCurve)

#Same bond calculated from yield rather than from the discount curve
yield <- 0.02
FixedRateBond(bond,
              coupon.rate,
              schedule,
              calc,
              yield=yield)


#same example with clean price
price <- 103.31
FixedRateBond(bond,
              coupon.rate,
              schedule,
              calc,
              price = price)

#example with default calc parameter
FixedRateBond(bond,
              coupon.rate,
              schedule,
              discountCurve=discountCurve)
              
#example with default calc and schedule parameters
schedule <- list(effectiveDate=as.Date("2004-11-30"),
                 maturityDate=as.Date("2008-11-30"))
FixedRateBond(bond,
              coupon.rate,
              schedule,
              discountCurve=discountCurve)

#example with default calc, schedule and bond parameters
FixedRateBond(,
              coupon.rate,
              schedule,
              discountCurve=discountCurve)

FixedRateBondPriceByYield(,0.0307, 100000, as.Date("2004-11-30"),
                          as.Date("2008-11-30"), 3, , c(0.02875),
                          , , , ,as.Date("2004-11-30"))

FixedRateBondYield(,90, 100000, as.Date("2004-11-30"), as.Date("2008-11-30"),
                   3, , c(0.02875), , , , ,as.Date("2004-11-30"))

}
\keyword{misc}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{tsQuotes}
\alias{tsQuotes}
\title{Vol Cube Example Data  
 
Short time series examples}
\format{A series of tenors and rates approppriate for calling DiscountCurve}
\source{
TBA
}
\description{
Vol Cube Example Data  
 
Short time series examples
}

\name{ZeroCouponBond}
\alias{ZeroCouponBond}
\alias{ZeroCouponBond.default}
\alias{ZeroPriceByYield}
\alias{ZeroPriceByYield.default}
\alias{ZeroYield}
\alias{ZeroYield.default}
\title{Zero-Coupon bond pricing}
\description{
  The \code{ZeroCouponBond} function evaluates a zero-coupon plainly using discount curve. 
  More specificly, the calculation is done by DiscountingBondEngine from QuantLib.
  The NPV, clean price, dirty price, accrued interest, yield and cash flows of the bond is returned. 
  For more detail, see the source code in the QuantLib file \code{test-suite/bond.cpp}.

  The \code{ZeroPriceYield} function evaluates a zero-coupon clean price based on its yield. 

  The \code{ZeroYield} function evaluations a zero-coupon yield based.
  See also http://www.mathworks.com/access/helpdesk/help/toolbox/finfixed/zeroyield.html
}
\usage{
\method{ZeroCouponBond}{default}(bond, discountCurve, dateparams)

\method{ZeroPriceByYield}{default}(yield, faceAmount, 
                            issueDate, maturityDate, 
                            dayCounter=2, frequency=2, 
                            compound=0, businessDayConvention=4)

\method{ZeroYield}{default}(price, faceAmount, 
                            issueDate, maturityDate, 
                            dayCounter=2, frequency=2, 
                            compound=0, businessDayConvention=4)
}
\arguments{

\item{bond}{bond parameters, a named list whose elements are: 
  \tabular{ll}{
    \code{issueDate}     \tab a Date, the bond's issue date\cr
    \code{maturityDate}  \tab a Date, the bond's maturity date\cr
    \code{faceAmount}    \tab (Optional) a double, face amount of the bond.\cr
    \code{}        \tab  Default value is 100. \cr
    \code{redemption}    \tab (Optional) a double, percentage of the initial \cr
    \code{}        \tab 	face amount that will be returned at maturity \cr
    \code{}        \tab    date. Default value is 100.\cr
  }
}
\item{discountCurve}{Can be one of the following:
  \tabular{ll}{
    \code{a DiscountCurve} \tab  a object of DiscountCurve class \cr
    \code{}                \tab For more detail, see example or \cr 
    \code{}                \tab the discountCurve function \cr
    \code{A 2 items list} \tab specifies a flat curve in two \cr 
    \code{}               \tab values "todayDate" and "rate"   \cr
    \code{A 3 items list} \tab specifies three values to construct a \cr
    \code{}                \tab DiscountCurve object, "params" , \cr 
    \code{} \tab "tsQuotes", "times". \cr
    \code{}                \tab For more detail, see example or \cr 
    \code{}                \tab the discountCurve function \cr
  }
}

\item{dateparams}{(Optional) a named list, QuantLib's date parameters of the bond. 
  \tabular{ll}{
    \code{settlementDays} \tab (Optional) a double, settlement days. \cr 
    \code{}        \tab Default value is 1.\cr
    \code{calendar} \tab (Optional) a string, either 'us' or 'uk' \cr 
    \code{}        \tab corresponding to US Goverment Bond \cr
    \code{}		   \tab calendar and UK Exchange calendar.\cr
    \code{}        \tab  Default value is 'us'.\cr
    \code{businessDayConvention} \tab (Optional) a number or string, \cr 
    \code{}                 \tab business day convention. \cr 
    \tab See \link{Enum}. Default value is 'Following'. \cr
  }
  See example below.
}
\item{yield}{yield of the bond}
\item{price}{price of the bond}
\item{faceAmount}{face amount of the bond}
\item{issueDate}{date the bond is issued}
\item{maturityDate}{maturity date, an R's date type}
\item{dayCounter}{day count convention. 0 = Actual360(), 1 = Actual365Fixed(), 2 = ActualActual(), 3 = Business252(), 4 = OneDayCounter(), 5 = SimpleDayCounter(), all other = Thirty360(). For more information, see QuantLib's DayCounter class}
\item{frequency}{frequency of events,0=NoFrequency, 1=Once, 2=Annual, 3=Semiannual, 4=EveryFourthMonth, 5=Quarterly, 6=Bimonthly ,7=Monthly ,8=EveryFourthWeely,9=Biweekly, 10=Weekly, 11=Daily. For more information, see QuantLib's Frequency class }
\item{compound}{compounding type. 0=Simple, 1=Compounded, 2=Continuous, all other=SimpleThenCompounded. See QuantLib's Compound class}
\item{businessDayConvention}{convention used to adjust a date in case it is not a valid business day. See quantlib for more detail. 0 = Following, 1 = ModifiedFollowing, 2 = Preceding, 3 = ModifiedPreceding, other = Unadjusted}

}
\value{
  The \code{ZeroCouponBond} function returns an object of class
  \code{ZeroCouponBond} (which inherits from class 
  \code{Bond}). It contains a list with the following
  components:
  \item{NPV}{net present value of the bond}
  \item{cleanPrice}{clean price of the bond}
  \item{dirtyPrice}{dirty price of the bond}
  \item{accruedAmount}{accrued amount of the bond}
  \item{yield}{yield of the bond}
  \item{cashFlows}{cash flows of the bond}

  The \code{ZeroPriceByYield} function returns an object of class
  \code{ZeroPriceByYield} (which inherits from class 
  \code{Bond}). It contains a list with the following
  components:
  \item{price}{price of the bond}

  The \code{ZeroYield} function returns an object of class
  \code{ZeroYield} (which inherits from class 
  \code{Bond}). It contains a list with the following
  components:
  \item{yield}{yield of the bond}
}
\details{
  A discount curve is built to calculate the bond value. 

  Please see any decent Finance textbook for background reading, and the
  \code{QuantLib} documentation for details on the \code{QuantLib}
  implementation.  
}
\references{\url{https://www.quantlib.org/} for details on \code{QuantLib}.}

\author{Khanh Nguyen \email{knguyen@cs.umb.edu} for the inplementation; Dirk Eddelbuettel \email{edd@debian.org} for the \R interface;
  the QuantLib Group for \code{QuantLib}}
\note{The interface might change in future release as \code{QuantLib}
  stabilises its own API.}


\examples{

# Simple call with all parameter and a flat curve
bond <- list(faceAmount=100,issueDate=as.Date("2004-11-30"),
             maturityDate=as.Date("2008-11-30"), redemption=100 )

dateparams <-list(settlementDays=1, calendar="UnitedStates/GovernmentBond",
                  businessDayConvention='Unadjusted')

discountCurve.param <- list(tradeDate=as.Date('2002-2-15'),
                           settleDate=as.Date('2002-2-15'),
                           dt=0.25,
                           interpWhat='discount', interpHow='loglinear')
discountCurve.flat <- DiscountCurve(discountCurve.param, list(flat=0.05))

ZeroCouponBond(bond, discountCurve.flat, dateparams)


# The same bond with a discount curve constructed from market quotes
tsQuotes <- list(d1w  =0.0382,
                 d1m  =0.0372,
                 fut1=96.2875,
                 fut2=96.7875,
                 fut3=96.9875,
                 fut4=96.6875,
                 fut5=96.4875,
                 fut6=96.3875,
                 fut7=96.2875,
                 fut8=96.0875,
                 s3y  =0.0398,
                 s5y  =0.0443,
                 s10y =0.05165,
                 s15y =0.055175)
tsQuotes <- list("flat" = 0.02)		## While discount curve code is buggy

discountCurve <- DiscountCurve(discountCurve.param, tsQuotes)
ZeroCouponBond(bond, discountCurve, dateparams)


#examples with default arguments
ZeroCouponBond(bond, discountCurve)

bond <- list(issueDate=as.Date("2004-11-30"),
             maturityDate=as.Date("2008-11-30"))
dateparams <-list(settlementDays=1)
ZeroCouponBond(bond, discountCurve, dateparams)


ZeroPriceByYield(0.1478, 100, as.Date("1993-6-24"), as.Date("1993-11-1"))

ZeroYield(90, 100, as.Date("1993-6-24"), as.Date("1993-11-1"))

}
\keyword{misc}

\name{EuropeanOptionArrays}
\alias{EuropeanOptionArrays}
\alias{oldEuropeanOptionArrays}
\alias{plotOptionSurface}
\title{European Option evaluation using Closed-Form solution}
\description{
  The \code{EuropeanOptionArrays} function allows any two of the numerical
  input parameters to be a vector, and a list of matrices is
  returned for the option value as well as each of the 'greeks'. For
  each of the returned matrices, each element
  corresponds to an evaluation under the given set of parameters.
}
\usage{
EuropeanOptionArrays(type, underlying, strike, dividendYield,
                     riskFreeRate, maturity, volatility)
oldEuropeanOptionArrays(type, underlying, strike, dividendYield,
                        riskFreeRate, maturity, volatility)
plotOptionSurface(EOres, ylabel="", xlabel="", zlabel="", fov=60)
}
\arguments{
  \item{type}{A string with one of the values \code{call} or \code{put}}
  \item{underlying}{(Scalar or list) current price(s) of the underlying stock}
  \item{strike}{(Scalar or list) strike price(s) of the option}
  \item{dividendYield}{(Scalar or list) continuous dividend yield(s) (as a fraction) of the stock}
  \item{riskFreeRate}{(Scalar or list) risk-free rate(s)}
  \item{maturity}{(Scalar or list) time(s) to maturity (in fractional years)}
  \item{volatility}{(Scalar or list) volatilit(y|ies) of the underlying stock}
  \item{EOres}{result matrix produced by \code{EuropeanOptionArrays}}
  \item{ylabel}{label for y-axsis}
  \item{xlabel}{label for x-axsis}
  \item{zlabel}{label for z-axsis}
  \item{fov}{viewpoint for 3d rendering}
}
\value{
  The \code{EuropeanOptionArrays} function allows any two of the numerical
  input parameters to be a vector or sequence. A list of
  two-dimensional matrices is returned. Each cell corresponds to
  an evaluation under the given set of parameters.

  For these functions, the following components are returned:
  \item{value}{(matrix) value of option}
  \item{delta}{(matrix) change in value for a change in the underlying}
  \item{gamma}{(matrix) change in value for a change in delta}
  \item{vega}{(matrix) change in value for a change in the underlying's volatility}
  \item{theta}{(matrix) change in value for a change in delta}
  \item{rho}{(matrix) change in value for a change in time to maturity}
  \item{dividendRho}{(matrix) change in value for a change in delta}
  \item{parameters}{List with parameters with which object was created}

  The \code{oldEuropeanOptionArrays} function is an older implementation
  which vectorises this at the R level instead but allows more general
  multidimensional arrays.

}
\details{
  The well-known closed-form solution derived by Black, Scholes and
  Merton is used for valuation.

  Please see any decent Finance textbook for background reading, and the
  \code{QuantLib} documentation for details on the \code{QuantLib}
  implementation.
}
\references{\url{https://www.quantlib.org/} for details on \code{QuantLib}.}
\author{Dirk Eddelbuettel \email{edd@debian.org} for the \R interface;
  the QuantLib Group for \code{QuantLib}}
\note{The interface might change in future release as \code{QuantLib}
  stabilises its own API.}
\seealso{\code{\link{AmericanOption}},\code{\link{BinaryOption}}}

\examples{
\dontrun{
# define two vectos for the underlying and the volatility
und.seq <- seq(10,180,by=2)
vol.seq <- seq(0.1,0.9,by=0.1)
# evaluate them along with three scalar parameters
EOarr <- EuropeanOptionArrays("call", underlying=und.seq,
                              strike=100, dividendYield=0.01,
                              riskFreeRate=0.03,
                              maturity=1, volatility=vol.seq)
# and look at four of the result arrays: value, delta, gamma, vega
old.par <- par(no.readonly = TRUE)
par(mfrow=c(2,2),oma=c(5,0,0,0),mar=c(2,2,2,1))
plot(EOarr$parameters.underlying, EOarr$value[,1], type='n',
     main="option value", xlab="", ylab="")
topocol <- topo.colors(length(vol.seq))
for (i in 1:length(vol.seq))
  lines(EOarr$parameters.underlying, EOarr$value[,i], col=topocol[i])
plot(EOarr$parameters.underlying, EOarr$delta[,1],type='n',
     main="option delta", xlab="", ylab="")
for (i in 1:length(vol.seq))
  lines(EOarr$parameters.underlying, EOarr$delta[,i], col=topocol[i])
plot(EOarr$parameters.underlying, EOarr$gamma[,1],type='n',
     main="option gamma", xlab="", ylab="")
for (i in 1:length(vol.seq))
  lines(EOarr$parameters.underlying, EOarr$gamma[,i], col=topocol[i])
plot(EOarr$parameters.underlying, EOarr$vega[,1],type='n',
     main="option vega", xlab="", ylab="")
for (i in 1:length(vol.seq))
  lines(EOarr$parameters.underlying, EOarr$vega[,i], col=topocol[i])
mtext(text=paste("Strike is 100, maturity 1 year, riskless rate 0.03",
        "\nUnderlying price from", und.seq[1],"to", und.seq[length(und.seq)],
        "\nVolatility  from",vol.seq[1], "to",vol.seq[length(vol.seq)]),
      side=1,font=1,outer=TRUE,line=3)
par(old.par)
}
}
\keyword{misc}

\name{CallableBond}
\alias{CallableBond}
\alias{CallableBond.default}
\title{CallableBond evaluation}
\description{
  The \code{CallableBond} function sets up and evaluates a callable fixed rate bond using Hull-White model
  and a TreeCallableFixedBondEngine pricing engine. For more detail, see the source codes in quantlib's example folder, Examples/CallableBond/CallableBond.cpp
}
\usage{
\method{CallableBond}{default}(bondparams, hullWhite, coupon, dateparams)
}
\arguments{
\item{bondparams}{a named list whose elements are:
  \tabular{ll}{
    \code{issueDate}  \tab a Date, the bond's issue date \cr
    \code{maturityDate} \tab a Date, the bond's maturity date \cr
    \code{faceAmount} \tab (Optional) a double, face amount of the bond. \cr
    \code{}           \tab Default value is 100.\cr
    \code{redemption} \tab (Optional) a double, percentage of the initial face \cr
    \code{}           \tab amount that will be returned at maturity date. \cr
    \code{}           \tab Default value is 100. \cr
    \code{callSch} \tab (Optional) a data frame whose columns are "Price",\cr
    \code{}        \tab "Type" and "Date" corresponding to QuantLib's \cr
    \code{}        \tab CallabilitySchedule. Defaule is an empty frame, or no callability.\cr
  }
}
\item{hullWhite}{a named list whose elements are parameters needed to set up a HullWhite pricing engine in QuantLib:
  \tabular{ll}{
    \code{term}  \tab a double, to set up a flat rate yield term structure \cr
    \code{alpha} \tab a double, Hull-White model's alpha value \cr
    \code{sigma} \tab a double, Hull-White model's sigma value \cr
    \code{gridIntervals}. \tab a double, time intervals parameter to \cr
    \code{}              \tab set up the TreeCallableFixedBondEngine \cr
  }
  Currently, the codes only support a flat rate yield term structure. For more detail, see QuantLib's doc on HullWhite
  and TreeCallableFixedBondEngine.
}
\item{coupon}{a numeric vector of coupon rates}
\item{dateparams}{(Optional) a named list, QuantLib's date parameters of the bond.
  \tabular{ll}{
    \code{settlementDays} \tab (Optional) a double, settlement days. \cr
    \code{}        \tab Default value is 1.\cr
    \code{calendar} \tab (Optional) a string, either 'us' or 'uk' \cr
    \code{}        \tab corresponding to US Goverment Bond \cr
    \code{}		   \tab calendar and UK Exchange calendar.\cr
    \code{}        \tab  Default value is 'us'.\cr
    \code{dayCounter} \tab (Optional) a number or string, \cr
    \code{}        \tab day counter convention.\cr
    \code{}        \tab  See \link{Enum}. Default value is 'Thirty360' \cr
    \code{period}  \tab (Optional) a number or string, \cr
    \code{}       \tab  interest compounding interval. See \link{Enum}. \cr
    \code{}       \tab Default value is 'Semiannual'.\cr
    \code{businessDayConvention} \tab (Optional) a number or string, \cr
    \code{}                 \tab business day convention. \cr
    \code{} \tab See \link{Enum}. Default value is 'Following'. \cr
    \code{terminationDateConvention} \tab (Optional) a number or string \cr
    \code{}                         \tab termination day convention.\cr
    \code{}                         \tab See \link{Enum}. Default value is'Following'.\cr
  }
  See example below.
}
}
\value{
  The \code{CallableBond} function returns an object of class
  \code{CallableBond} (which inherits from class
  \code{Bond}). It contains a list with the following
  components:
  \item{NPV}{net present value of the bond}
  \item{cleanPrice}{price price of the bond}
  \item{dirtyPrice}{dirty price of the bond}
  \item{accruedAmount}{accrued amount of the bond}
  \item{yield}{yield of the bond}
  \item{cashFlows}{cash flows of the bond}
}
\details{
  Please see any decent Finance textbook for background reading, and the
  \code{QuantLib} documentation for details on the \code{QuantLib}
  implementation.
}
\references{\url{https://www.quantlib.org/} for details on \code{QuantLib}.}

\author{Khanh Nguyen \email{knguyen@cs.umb.edu} for the inplementation; Dirk Eddelbuettel \email{edd@debian.org} for the \R interface;
  the QuantLib Group for \code{QuantLib}}
\note{The interface might change in future release as \code{QuantLib}
  stabilises its own API.}
\examples{
if (interactive()) {  # the example is too computationally expensive for normal checks
    #set-up a HullWhite according to example from QuantLib
    HullWhite <- list(term = 0.055, alpha = 0.03, sigma = 0.01, gridIntervals = 40)

    #callability schedule dataframe
    Price <- rep(as.double(100),24)
    Type <- rep(as.character("C"), 24)
    Date <- seq(as.Date("2006-09-15"), by = '3 months', length = 24)
    callSch <- data.frame(Price, Type, Date)
    callSch$Type <- as.character(callSch$Type)

    bondparams <- list(faceAmount=100, issueDate = as.Date("2004-09-16"),
                       maturityDate=as.Date("2012-09-16"), redemption=100,
                       callSch = callSch)
    dateparams <- list(settlementDays=3, calendar="UnitedStates/GovernmentBond",
                       dayCounter = "ActualActual",
                       period="Quarterly",
                       businessDayConvention = "Unadjusted",
                       terminationDateConvention= "Unadjusted")
    coupon <- c(0.0465)
    setEvaluationDate(as.Date("2004-11-22"))

    CallableBond(bondparams, HullWhite, coupon, dateparams)
    #examples using default values
    CallableBond(bondparams, HullWhite, coupon)
    dateparams <- list(period="Quarterly",
                       businessDayConvention = "Unadjusted",
                       terminationDateConvention= "Unadjusted")
    CallableBond(bondparams, HullWhite, coupon, dateparams)

    bondparams <- list(issueDate = as.Date("2004-09-16"),
                       maturityDate=as.Date("2012-09-16"))
    CallableBond(bondparams, HullWhite, coupon, dateparams)
}
}
\keyword{misc}

\name{BermudanSwaption}
\alias{BermudanSwaption}
\alias{BermudanSwaption.default}
\alias{summary.G2Analytic}
\alias{summary.HWAnalytic}
\alias{summary.HWTree}
\alias{summary.BKTree}
\title{Bermudan swaption valuation using several short-rate models}
\description{
  \code{BermudanSwaption} prices a Bermudan swaption with specified
  strike and maturity (in years), after calibrating the selected
  short-rate model to an input swaption volatility matrix. Swaption
  maturities are in years down the rows, and swap tenors are in years
  along the columns, in the usual fashion. It is assumed that the
  Bermudan swaption is
  exercisable on each reset date of the underlying swaps.
}
\usage{
BermudanSwaption(params, ts, swaptionMaturities, swapTenors,
volMatrix)
}
\arguments{
  \item{params}{A list specifying the \code{tradeDate} (month/day/year),
    \code{settlementDate}, \code{startDate}, \code{maturity},  \code{payFixed}
    flag, \code{strike}, pricing \code{method}, and curve construction options
    (see \emph{Examples} section below). Curve construction options are
    \code{interpWhat} (possible values are \code{discount},
    \code{forward}, and \code{zero}) and
    \code{interpHow} (possible values are \code{linear},
    \code{loglinear}
    , and \code{spline}). Both \code{interpWhat} and \code{interpHow}
    are ignored when a flat yield curve is requested, but they must be
    present nevertheless.
    The pricing method can be one of the following (all short-rate models):
    \tabular{ll}{
      \code{G2Analytic} \tab G2 2-factor Gaussian model using analytic formulas.\cr
      \code{HWAnalytic} \tab Hull-White model using analytic formulas.\cr
      \code{HWTree} \tab Hull-White model using a tree.\cr
      \code{BKTree} \tab Black-Karasinski model using a tree.
      }
    }
  \item{ts}{A term structure built with DiscounCurve or market observables
     needed to construct the spot term
    structure of interest rates. A list of name/value pairs. See the
    help page for \code{\link{DiscountCurve}} for details.}
  \item{swaptionMaturities}{A vector containing the swaption maturities
    associated with the rows of the swaption volatility matrix.}
  \item{swapTenors}{A vector containing the underlying swap tenors
    associated with the columns of the swaption volatility matrix.}
  \item{volMatrix}{The swaption volatility matrix. Must be a 2D matrix
  stored by rows. See the example below.}
 }
\value{
  \code{BermudanSwaption} , if there are sufficient swaption vols to fit an affine model,
  returns a list containing calibrated model
  paramters (what parameters are returned depends on the model
  selected) along with:
  \item{price}{Price of swaption in basis points (actual price
  equals \code{price} times notional divided by 10,000)}
  \item{ATMStrike}{At-the-money strike}
  \item{params}{Input parameter list}
  If there are insufficient swaption vols to calibrate it throws a warning and returns \code{NULL}
}
\details{
  This function was update for \code{QuantLib} Version 1.7.1 or later. It
  introduces support for fixed-income instruments in \code{RQuantLib}. It implements the
  full function and should work in most cases as long as there are suuficient swaption vol
  data points to fit the affine model. At least 5 unique points are required. The data point
  search attempts to find 5 or more points with one being the closet match in terms in of
  expiration and maturity.

  See the \code{\link{SabrSwaption}} function for an alternative.
}
\references{
  Brigo, D. and Mercurio, F. (2001) \emph{Interest Rate Models: Theory and
    Practice}, Springer-Verlag, New York.

  For information about \code{QuantLib} see \url{https://www.quantlib.org/}.

  For information about \code{RQuantLib} see
\url{http://dirk.eddelbuettel.com/code/rquantlib.html}.

}
\author{Dominick Samperi}

\seealso{\code{\link{DiscountCurve}}, \code{\link{SabrSwaption}}}

\examples{
\dontrun{
# This data replicates sample code shipped with QuantLib 0.3.10 results
params <- list(tradeDate=as.Date('2002-2-15'),
               settleDate=as.Date('2002-2-19'),
               startDate=as.Date('2003-2-19'),
               maturity=as.Date('2008-2-19'),
               dt=.25,
               payFixed=TRUE,
               strike=.05,
               method="G2Analytic",
               interpWhat="discount",
               interpHow="loglinear")
setEvaluationDate(as.Date('2002-2-15'))
# Market data used to construct the term structure of interest rates
tsQuotes <- list(d1w  =0.05,
                 # d1m  =0.0372,
                 # fut1=96.2875,
                 # fut2=96.7875,
                 # fut3=96.9875,
                 # fut4=96.6875,
                 # fut5=96.4875,
                 # fut6=96.3875,
                 # fut7=96.2875,
                 # fut8=96.0875,
                 s3y  =0.05,
                 s5y  =0.05,
                 s10y =0.05,
                 s15y =0.05)

times=seq(0,14.75,.25)
swcurve=DiscountCurve(params,tsQuotes,times)
# Use this to compare with the Bermudan swaption example from QuantLib
#tsQuotes <- list(flat=0.04875825)

# Swaption volatility matrix with corresponding maturities and tenors
swaptionMaturities <- c(1,2,3,4,5)

swapTenors <- c(1,2,3,4,5)

volMatrix <- matrix(
    c(0.1490, 0.1340, 0.1228, 0.1189, 0.1148,
      0.1290, 0.1201, 0.1146, 0.1108, 0.1040,
      0.1149, 0.1112, 0.1070, 0.1010, 0.0957,
      0.1047, 0.1021, 0.0980, 0.0951, 0.1270,
      0.1000, 0.0950, 0.0900, 0.1230, 0.1160),
    ncol=5, byrow=TRUE)

volMatrix <- matrix(
    c(rep(.20,25)),
    ncol=5, byrow=TRUE)
# Price the Bermudan swaption
pricing <- BermudanSwaption(params, ts=.05,
                            swaptionMaturities, swapTenors, volMatrix)
summary(pricing)
}
}
\keyword{models}

% $Id$
\name{Schedule}
\alias{Schedule}
\alias{Schedule.default}
\title{Schedule generation}
\description{
  The \code{Schedule} function generates a schedule of dates conformant to
  a given convention in a given calendar.
}
\usage{
\method{Schedule}{default}(params)
}
\arguments{

\item{params}{a named list, QuantLib's parameters of the schedule. 
  \tabular{ll}{
    \code{effectiveDate}       \tab a Date, when the schedule becomes effective. \cr 
    \code{maturityDate}        \tab a Date, when the schedule matures. \cr 
    \code{period}              \tab (Optional) a number or string, the frequency of \cr
    \code{}                    \tab the schedule. Default value is 'Semiannual'. \cr
    \code{calendar}            \tab (Optional) a string, the calendar name. \cr 
    \code{}                    \tab Defaults to 'TARGET' \cr
    \code{businessDayConvention} \tab (Optional) a number or string, the \cr 
    \code{}                    \tab day convention to use. \cr
    \code{}                    \tab Defaults to 'Following'. \cr
    \code{terminationDateConvention} \tab (Optional) a number or string, the \cr 
    \code{}                    \tab day convention to use for the terminal date. \cr
    \code{}                    \tab Defaults to 'Following'. \cr
    \code{dateGeneration}      \tab (Optional) a number or string, the \cr 
    \code{}                    \tab date generation rule. \cr
    \code{}                    \tab Defaults to 'Backward'. \cr
    \code{endOfMonth}          \tab (Optional) 1 or 0, use End of Month rule for \cr
    \code{}                    \tab schedule dates. Defaults to 0 (false).
  }
  See example below.
}

}

\value{
  The \code{Schedule} function returns an object of class
  \code{Schedule}. It contains the list of dates in the schedule.
}
\details{
  Please see any decent Finance textbook for background reading, and the
  \code{QuantLib} documentation for details on the \code{QuantLib}
  implementation.  
}
\references{\url{https://www.quantlib.org/} for details on \code{QuantLib}.}
\author{
  Michele Salvadore \email{michele.salvadore@gmail.com} for the inplementation;
  Dirk Eddelbuettel \email{edd@debian.org} for the \R interface;
  the QuantLib Group for \code{QuantLib}
}

\seealso{
  \code{\link{FixedRateBond}}
}

\examples{

params <- list(effectiveDate=as.Date("2004-11-30"),
               maturityDate=as.Date("2008-11-30"),
               period='Semiannual',
               calendar='UnitedStates/GovernmentBond',
               businessDayConvention='Unadjusted',
               terminationDateConvention='Unadjusted',
               dateGeneration='Forward',
               endOfMonth=1)
Schedule(params)
}
\keyword{misc}

\name{BondUtilities}
\alias{matchBDC}
\alias{matchCompounding}
\alias{matchDateGen}
\alias{matchDayCounter}
\alias{matchFrequency}
\alias{matchParams}
\title{Bond parameter conversion utilities}
\description{
These functions are using internally to convert from the characters at
the R level to the \code{enum} types used at the C++ level. They are
documented here mostly to provide a means to look up some of the
possible values---the user is not expected to call these functions directly..}
\usage{
matchBDC(bdc = c("Following", "ModifiedFollowing", "Preceding",
                 "ModifiedPreceding", "Unadjusted",
                 "HalfMonthModifiedFollowing", "Nearest"))
matchCompounding(cp = c("Simple", "Compounded", "Continuous", "SimpleThenCompounded"))
matchDayCounter(daycounter = c("Actual360", "ActualFixed", "ActualActual", "Business252",
                              "OneDayCounter", "SimpleDayCounter", "Thirty360",
                              "Actual365NoLeap", "ActualActual.ISMA", "ActualActual.Bond",
                              "ActualActual.ISDA", "ActualActual.Historical",
                              "ActualActual.AFB", "ActualActual.Euro"))
matchDateGen(dg = c("Backward", "Forward", "Zero", "ThirdWednesday",
                    "Twentieth", "TwentiethIMM", "OldCDS", "CDS"))
matchFrequency(freq = c("NoFrequency","Once", "Annual", "Semiannual",
                        "EveryFourthMonth", "Quarterly", "Bimonthly",
                        "Monthly", "EveryFourthWeek", "Biweekly",
                        "Weekly", "Daily"))
matchParams(params)
}
\arguments{
  \item{bdc}{A string identifying one of the possible business day convention values.}
  \item{cp}{A string identifying one of the possible compounding frequency values.}
  \item{daycounter}{A string identifying one of the possible day counter scheme values.}
  \item{dg}{A string identifying one of the possible date generation scheme values.}
  \item{freq}{A string identifying one of the possible (dividend) frequency values.}
  \item{params}{A named vector containing the other parameters as components.}
}
\value{
  Each function converts the given character value into a corresponding
  numeric entry. For \code{matchParams}, an named vector of strings is
  converted into a named vector of numerics..
}
\details{
  The QuantLib documentation should be consulted for details.

  Note that \code{Actual365NoLeap} is soft deprecated as of QuantLib 1.11 and hard deprecated as of
  QuantLib 1.16. For users on QuantLib 1.16 or later, use of the RQuantLib daycounter enum with a
  value of severn will result in \code{Actual365Fixed(Actual365Fixed::NoLeap)} which is functionally
  equivalent to \code{Actual365NoLeap}. Previously RQuantLib allowed users to retain
  \code{Actual365NoLeap} via defining \code{RQUANTLIB_USE_ACTUAL365NOLEAP}, but this is no longer
  required.

  Also note that \code{ActualActual} without explicit convention specification is hard deprecated
  as of QuantLib 1.23. This is only soft-deprecated in RQuantLib by explicitly passing in the same
  default convention. Previously RQuantLib allowed users to define
  \code{RQUANTLIB_USE_ACTUALACTUAL}, but this is no longer required.
}
\references{\url{https://www.quantlib.org/} for details on \code{QuantLib}.}
\author{Khanh Nguyen \email{knguyen@cs.umb.edu} for the \R interface;
  the QuantLib Group for \code{QuantLib}}
\note{The interface might change in future release as \code{QuantLib}
  stabilises its own API.}
%\examples{}
\keyword{misc}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{getQuantLibVersion}
\alias{getQuantLibVersion}
\title{Return the QuantLib version number}
\usage{
getQuantLibVersion()
}
\value{
A character variable
}
\description{
This function returns the QuantLib version string as encoded in the header
file \code{config.hpp} and determined at compilation time of the QuantLib library.
}
\examples{
  getQuantLibVersion()
}
\references{
\url{https://www.quantlib.org} for details on \code{QuantLib}.
}
\author{
Dirk Eddelbuettel
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{getQuantLibCapabilities}
\alias{getQuantLibCapabilities}
\title{Return configuration options of the QuantLib library}
\usage{
getQuantLibCapabilities()
}
\value{
A named vector of logical variables
}
\description{
This function returns a named vector of boolean variables describing several
configuration options determined at compilation time of the QuantLib library.
}
\details{
Not all of these features are used (yet) by RQuantLib.
}
\examples{
  getQuantLibCapabilities()
}
\references{
\url{https://www.quantlib.org} for details on \code{QuantLib}.
}
\author{
Dirk Eddelbuettel
}

\name{BinaryOptionImpliedVolatility}
\alias{BinaryOptionImpliedVolatility}
\alias{BinaryOptionImpliedVolatility.default}
\title{Implied Volatility calculation for Binary Option}
\description{
  The \code{BinaryOptionImpliedVolatility} function solves for the
  (unobservable) implied volatility, given an option price as well as
  the other required parameters to value an option.}
\usage{
\method{BinaryOptionImpliedVolatility}{default}(type, value, underlying,
		strike, dividendYield, riskFreeRate, maturity, volatility,
		cashPayoff=1, dayCounter=0)
}
\arguments{
  \item{type}{A string with one of the values \code{call}, \code{put} or
    \code{straddle}}
  \item{value}{Value of the option (used only for ImpliedVolatility calculation)}
  \item{underlying}{Current price of the underlying stock}
  \item{strike}{Strike price of the option}
  \item{dividendYield}{Continuous dividend yield (as a fraction) of the stock}
  \item{riskFreeRate}{Risk-free rate}
  \item{maturity}{Time to maturity in either fractional years or as a
    \sQuote{Date} object specifying the expiry date)}
  \item{volatility}{Initial guess for the volatility of the underlying
    stock}
  \item{cashPayoff}{Binary payout if options is exercised, default is 1}
  \item{dayCounter}{Integer denoting the selected QuantLib day counter
    convention, default value is 0 for \sQuote{Actual360}}
}
\value{

  The \code{BinaryOptionImpliedVolatility} function returns an numeric
  variable with volatility implied by the given market prices.

}
\details{
  The Finite Differences method is used to value the Binary Option.
  Implied volatilities are then calculated numerically.

  Please see any decent Finance textbook for background reading, and the
  \code{QuantLib} documentation for details on the \code{QuantLib}
  implementation.
}
\references{\url{https://www.quantlib.org/} for details on \code{QuantLib}.}
\author{Dirk Eddelbuettel \email{edd@debian.org} for the \R interface;
  the QuantLib Group for \code{QuantLib}}
\note{The interface might change in future release as \code{QuantLib}
  stabilises its own API.}
\seealso{\code{\link{EuropeanOption}},\code{\link{AmericanOption}},\code{\link{BinaryOption}}}

\examples{
BinaryOptionImpliedVolatility("call", value=4.50, strike=100, 100, 0.02, 0.03, 0.5, 0.4, 10)
}
\keyword{misc}

\name{EuropeanOptionImpliedVolatility}
\alias{EuropeanOptionImpliedVolatility}
\alias{EuropeanOptionImpliedVolatility.default}
\title{Implied Volatility calculation for European Option}
\description{
  The \code{EuropeanOptionImpliedVolatility} function solves for the
  (unobservable) implied volatility, given an option price as well as
  the other required parameters to value an option.}
\usage{
\method{EuropeanOptionImpliedVolatility}{default}(type, value,
	underlying, strike, dividendYield, riskFreeRate, maturity, volatility)
}
\arguments{
  \item{type}{A string with one of the values \code{call} or \code{put}}
  \item{value}{Value of the option (used only for ImpliedVolatility calculation)}
  \item{underlying}{Current price of the underlying stock}
  \item{strike}{Strike price of the option}
  \item{dividendYield}{Continuous dividend yield (as a fraction) of the stock}
  \item{riskFreeRate}{Risk-free rate}
  \item{maturity}{Time to maturity (in fractional years)}
  \item{volatility}{Initial guess for the volatility of the underlying stock}
}
\value{
  The \code{EuropeanOptionImpliedVolatility} function returns an numeric
  variable with volatility implied by the given market prices and given parameters.
}
\details{
  The well-known closed-form solution derived by Black, Scholes and
  Merton is used for valuation. Implied volatilities are then calculated
  numerically.

  Please see any decent Finance textbook for background reading, and the
  \code{QuantLib} documentation for details on the \code{QuantLib}
  implementation.  
}
\references{\url{https://www.quantlib.org/} for details on \code{QuantLib}.}
\author{Dirk Eddelbuettel \email{edd@debian.org} for the \R interface;
  the QuantLib Group for \code{QuantLib}}
\note{The interface might change in future release as \code{QuantLib}
  stabilises its own API.}
\seealso{\code{\link{EuropeanOption}},\code{\link{AmericanOption}},\code{\link{BinaryOption}}}
\examples{
EuropeanOptionImpliedVolatility(type="call", value=11.10, underlying=100,
	strike=100, dividendYield=0.01, riskFreeRate=0.03,
	maturity=0.5, volatility=0.4)
}
\keyword{misc}

\name{BinaryOption}
\alias{BinaryOption}
\alias{BinaryOption.default}
\title{Binary Option evaluation using Closed-Form solution}
\description{
This function evaluations an Binary option on a common stock
using a closed-form solution. The option value as well as the common first
derivatives ("Greeks") are returned.}
\usage{
\method{BinaryOption}{default}(binType, type, excType, underlying,
        strike, dividendYield,
	riskFreeRate, maturity, volatility, cashPayoff)
}
\arguments{
  \item{binType}{A string with one of the values \code{cash},
    \code{asset} or \code{gap} to select CashOrNothing, AssetOrNothing
    or Gap payoff profiles}
  \item{type}{A string with one of the values \code{call} or \code{put}}
  \item{excType}{A string with one of the values \code{european} or
    \code{american} to denote the exercise type}
  \item{underlying}{Current price of the underlying stock}
  \item{strike}{Strike price of the option}
  \item{dividendYield}{Continuous dividend yield (as a fraction) of the stock}
  \item{riskFreeRate}{Risk-free rate}
  \item{maturity}{Time to maturity (in fractional years)}
  \item{volatility}{Volatility of the underlying stock}
  \item{cashPayoff}{Payout amount}
}
\value{
  An object of class \code{BinaryOption} (which inherits from class
  \code{\link{Option}}) is returned. It contains a list with the
  following components: 
  \item{value}{Value of option}
  \item{delta}{Sensitivity of the option value for a change in the underlying}
  \item{gamma}{Sensitivity of the option delta for a change in the underlying}
  \item{vega}{Sensitivity of the option value for a change in the
    underlying's volatility} 
  \item{theta}{Sensitivity of the option value for a change in t, the
    remaining time to maturity}
  \item{rho}{Sensitivity of the option value for a change in the
    risk-free interest rate}
  \item{dividendRho}{Sensitivity of the option value for a change in the
    dividend yield}
}
\details{
  A closed-form solution is used to value the Binary Option.
  
  Please see any decent Finance textbook for background reading, and
  the \code{QuantLib} documentation for details on the \code{QuantLib}
  implementation. 
}
\references{\url{https://www.quantlib.org/} for details on \code{QuantLib}.}
\author{Dirk Eddelbuettel \email{edd@debian.org} for the \R interface;
  the QuantLib Group for \code{QuantLib}}
\note{The interface might change in future release as \code{QuantLib}
  stabilises its own API.}
\seealso{\code{\link{AmericanOption}},\code{\link{EuropeanOption}}}

\examples{
BinaryOption(binType="asset", type="call", excType="european",
             underlying=100, strike=100, dividendYield=0.02,
             riskFreeRate=0.03, maturity=0.5, volatility=0.4, cashPayoff=10)
}
\keyword{misc}

% $Id$
\name{AsianOption}
\alias{AsianOption}
\alias{AsianOption.default}
\title{Asian Option evaluation using Closed-Form solution}
\description{
  The \code{AsianOption} function evaluates an Asian-style
  option on a common stock using an analytic solution for continuous
  geometric average price. The option value, the common first
  derivatives ("Greeks") as well as the calling parameters are returned.
}
\usage{
\method{AsianOption}{default}(averageType, type, underlying, strike,
	                  dividendYield, riskFreeRate, maturity, 
	                  volatility, first=0, length=11.0/12.0, fixings=26)
}
\arguments{
  \item{averageType}{Specifiy averaging type, either \dQuote{geometric} or \dQuote{arithmetic} }
  \item{type}{A string with one of the values \code{call} or \code{put}}
  \item{underlying}{Current price of the underlying stock}
  \item{strike}{Strike price of the option}
  \item{dividendYield}{Continuous dividend yield (as a fraction) of the stock}
  \item{riskFreeRate}{Risk-free rate}
  \item{maturity}{Time to maturity (in fractional years)}
  \item{volatility}{Volatility of the underlying stock}
  \item{first}{(Only for arithmetic averaging) Time step to first
    average, can be zero}
  \item{length}{(Only for arithmetic averaging) Total time length for
    averaging period}
  \item{fixings}{(Only for arithmetic averaging) Total number of averaging fixings}
}
\value{
  The \code{AsianOption} function returns an object of class
  \code{AsianOption} (which inherits from class 
  \code{\link{Option}}). It contains a list with the following
  components:
  \item{value}{Value of option}
  \item{delta}{Sensitivity of the option value for a change in the underlying}
  \item{gamma}{Sensitivity of the option delta for a change in the underlying}
  \item{vega}{Sensitivity of the option value for a change in the
    underlying's volatility} 
  \item{theta}{Sensitivity of the option value for a change in t, the
    remaining time to maturity}
  \item{rho}{Sensitivity of the option value for a change in the
    risk-free interest rate}
  \item{dividendRho}{Sensitivity of the option value for a change in the
    dividend yield}
}
\details{
  
  When "arithmetic" evaluation is used, only the NPV() is returned.
  
  The well-known closed-form solution derived by Black, Scholes and
  Merton is used for valuation. Implied volatilities are calculated
  numerically.

  Please see any decent Finance textbook for background reading, and the
  \code{QuantLib} documentation for details on the \code{QuantLib}
  implementation.  
}
\references{\url{https://www.quantlib.org/} for details on \code{QuantLib}.}
\author{Dirk Eddelbuettel \email{edd@debian.org} for the \R interface;
  the QuantLib Group for \code{QuantLib}}
\note{The interface might change in future release as \code{QuantLib}
  stabilises its own API.}
\examples{
# simple call with some explicit parameters, and slightly increased vol:
AsianOption("geometric", "put", underlying=80, strike=85, div=-0.03,
            riskFree=0.05, maturity=0.25, vol=0.2)
}
\keyword{misc}

\name{FloatingRateBond}
\alias{FloatingRateBond}
\alias{FloatingRateBond.default}
\title{Floating rate bond pricing}
\description{
  The \code{FloatingRateBond} function evaluates a floating rate bond using discount curve. 
  More specificly, the calculation is done by DiscountingBondEngine from QuantLib.
  The NPV, clean price, dirty price, accrued interest, yield and cash flows of the bond is returned. 
  For more detail, see the source codes in quantlib's test-suite. test-suite/bond.cpp
}
\usage{
\method{FloatingRateBond}{default}(bond, gearings, spreads,
                                   caps, floors, index, 
                                   curve, dateparams )
}
\arguments{


\item{bond}{bond parameters, a named list whose elements are: 
  \tabular{ll}{
    \code{issueDate}     \tab a Date, the bond's issue date\cr
    \code{maturityDate}  \tab a Date, the bond's maturity date\cr
    \code{faceAmount}    \tab (Optional) a double, face amount of the bond.\cr
    \code{}        \tab  Default value is 100. \cr
    \code{redemption}    \tab (Optional) a double, percentage of the initial \cr
    \code{}        \tab 	face amount that will be returned at maturity \cr
    \code{}        \tab    date. Default value is 100.\cr
    \code{effectiveDate}  \tab (Optinal) a Date, the bond's effective date. Default value is issueDate\cr
  }
}
\item{gearings}{(Optional) a numeric vector, bond's gearings. See quantlib's doc on
  FloatingRateBond for more detail. Default value is an empty vector c(). }
\item{spreads}{(Optional) a numeric vector, bond's spreads. See quantlib's doc on FloatingRateBond for more detail.Default value is an empty vector c() }
\item{caps}{(Optional) a numeric vector, bond's caps. See quantlib's doc on
  FloatingRateBond for more detail. Default value is an empty vector c() }
\item{floors}{(Optional) a numeric vector, bond's floors. See quantlib's doc on
  FloatingRateBond for more detail. Default value is an empty vector c() }


\item{curve}{Can be one of the following:
  \tabular{ll}{
    \code{a DiscountCurve} \tab  a object of DiscountCurve class \cr
    \code{}                \tab For more detail, see example or \cr 
    \code{}                \tab the discountCurve function \cr
    \code{A 2 items list} \tab specifies a flat curve in two \cr 
    \code{}               \tab values "todayDate" and "rate"   \cr
    \code{A 3 items list} \tab specifies three values to construct a \cr
    \code{}                \tab DiscountCurve object, "params" , \cr 
    \code{} \tab "tsQuotes", "times". \cr
    \code{}                \tab For more detail, see example or \cr 
    \code{}                \tab the discountCurve function \cr
  }
}

\item{index}{a named list whose elements are parameters of an IborIndex term structure. 
  \tabular{ll}{
    \code{type} \tab a string, currently support only "USDLibor" \cr
    \code{length} \tab an integer, length of the index \cr
    \code{inTermOf} \tab a string, period unit, currently support only 'Month' \cr
    \code{term} \tab a DiscountCurve object, the term structure of the index\cr
  }
}
\item{dateparams}{(Optional) a named list, QuantLib's date parameters of the bond. 
  \tabular{ll}{
    \code{settlementDays} \tab (Optional) a double, settlement days. \cr 
    \code{}        \tab Default value is 1.\cr
    \code{calendar} \tab (Optional) a string, either 'us' or 'uk' \cr 
    \code{}        \tab corresponding to US Goverment Bond \cr
    \code{}		   \tab calendar and UK Exchange calendar.\cr
    \code{}        \tab  Default value is 'us'.\cr
    \code{dayCounter} \tab (Optional) a number or string, \cr 
    \code{}        \tab day counter convention.\cr
    \code{}        \tab  See \link{Enum}. Default value is 'Thirty360' \cr 
    \code{period}  \tab (Optional) a number or string, \cr
    \code{}       \tab  interest compounding interval. See \link{Enum}. \cr
    \code{}       \tab Default value is 'Semiannual'.\cr
    \code{businessDayConvention} \tab (Optional) a number or string, \cr 
    \code{}                 \tab business day convention. \cr 
    \tab See \link{Enum}. Default value is 'Following'. \cr
    \code{terminationDateConvention} \tab (Optional) a number or string, \cr 
    \code{}                 \tab termination day convention. \cr 
    \tab See \link{Enum}. Default value is 'Following'. \cr
    \code{endOfMonth} \tab (Optional) a numeric with value 1 or 0. \cr 
    \code{}           \tab End of Month rule. Default value is 0.\cr
    \code{dateGeneration} \tab (Optional) a numeric, date generation method. \cr 
    \code{} \tab See \link{Enum}. Default value is 'Backward' \cr
  }
  See example below.
}
}
\value{
  The \code{FloatingRateBond} function returns an object of class
  \code{FloatingRateBond} (which inherits from class 
  \code{Bond}). It contains a list with the following
  components:
  \item{NPV}{net present value of the bond}
  \item{cleanPrice}{clean price of the bond}
  \item{dirtyPrice}{dirty price of the bond}
  \item{accruedAmount}{accrued amount of the bond}
  \item{yield}{yield of the bond}
  \item{cashFlows}{cash flows of the bond}
}
\details{
  A discount curve is built to calculate the bond value. 

  Please see any decent Finance textbook for background reading, and the
  \code{QuantLib} documentation for details on the \code{QuantLib}
  implementation.  
}
\references{\url{https://www.quantlib.org/} for details on \code{QuantLib}.}

\author{Khanh Nguyen \email{knguyen@cs.umbno.edu} for the inplementation; Dirk Eddelbuettel \email{edd@debian.org} for the \R interface;
  the QuantLib Group for \code{QuantLib}}
\note{The interface might change in future release as \code{QuantLib}
  stabilises its own API.}


\examples{

bond <- list(faceAmount=100, issueDate=as.Date("2004-11-30"),
             maturityDate=as.Date("2008-11-30"), redemption=100, 
             effectiveDate=as.Date("2004-11-30"))
dateparams <- list(settlementDays=1, calendar="UnitedStates/GovernmentBond",
                   dayCounter = 'ActualActual', period=2, 
                   businessDayConvention = 1, terminationDateConvention=1,
                   dateGeneration=0, endOfMonth=0, fixingDays = 1)

gearings <- spreads <- caps <- floors <- vector()

params <- list(tradeDate=as.Date('2002-2-15'),
               settleDate=as.Date('2002-2-19'),
               dt=.25,
               interpWhat="discount",
               interpHow="loglinear")
setEvaluationDate(as.Date("2004-11-22"))

tsQuotes <- list(d1w  =0.0382,
                 d1m  =0.0372,
                 fut1=96.2875,
                 fut2=96.7875,
                 fut3=96.9875,
                 fut4=96.6875,
                 fut5=96.4875,
                 fut6=96.3875,
                 fut7=96.2875,
                 fut8=96.0875,
                 s3y  =0.0398,
                 s5y  =0.0443,
                 s10y =0.05165,
                 s15y =0.055175)
tsQuotes <- list("flat" = 0.02)		## While discount curve code is buggy

## when both discount and libor curves are flat.

discountCurve.flat <- DiscountCurve(params, list(flat=0.05))
termstructure <- DiscountCurve(params, list(flat=0.03))
iborIndex.params <- list(type="USDLibor", length=6, 
                  inTermOf="Month", term=termstructure)                      
FloatingRateBond(bond, gearings, spreads, caps, floors, 
                 iborIndex.params, discountCurve.flat, dateparams)


## discount curve is constructed from market quotes
## and a flat libor curve
discountCurve <- DiscountCurve(params, tsQuotes)
termstructure <- DiscountCurve(params, list(flat=0.03))
iborIndex.params <- list(type="USDLibor", length=6, 
                  inTermOf="Month", term = termstructure)                      
FloatingRateBond(bond, gearings, spreads, caps, floors, 
                 iborIndex.params, discountCurve, dateparams)

#example using default values
FloatingRateBond(bond=bond, index=iborIndex.params, curve=discountCurve)

}
\keyword{misc}

\name{DiscountCurve}
\alias{DiscountCurve}
\alias{DiscountCurve.default}
\alias{plot.DiscountCurve}
\title{Returns the discount curve (with zero rates and forwards) given times}
\description{
  \code{DiscountCurve} constructs the spot term structure of interest
  rates based on input market data including the settlement date,
  deposit rates, futures prices, FRA rates, or swap rates, in various
  combinations. It returns the corresponding discount factors, zero
  rates, and forward rates for a vector of times that is specified
  as input.
}
\usage{
DiscountCurve(params, tsQuotes, times, legparams)
}
\arguments{
  \item{params}{A list specifying the \code{tradeDate} (month/day/year),
    \code{settleDate}, forward rate
    time span \code{dt}, and two curve
    construction options: \code{interpWhat} (with possible values
    \code{discount}, \code{forward}, and \code{zero}) and
    \code{interpHow} (with possible values \code{linear},
    \code{loglinear}, and \code{spline}). \code{spline} here means cubic spline
    interpolation of the \code{interpWhat} value.
  }
  \item{tsQuotes}{Market quotes used to construct the spot term
    structure of interest rates. Must be a list of name/value pairs,
    where the currently recognized names are:
    \tabular{ll}{
      \code{flat} \tab rate for a flat yield curve\cr
      \code{d1w}  \tab 1-week deposit rate\cr
      \code{d1m}  \tab 1-month deposit rate\cr
      \code{d3m}  \tab 3-month deposit rate\cr
      \code{d6m}  \tab 6-month deposit rate\cr
      \code{d9m}  \tab 9-month deposit rate\cr
      \code{d1y}  \tab 1-year deposit rate\cr
      \code{s2y}  \tab 2-year swap rate\cr
      \code{s3y}  \tab 3-year swap rate\cr
      \code{s4y}  \tab 4-year swap rate\cr
      \code{s5y}  \tab 5-year swap rate\cr
      \code{s6y}  \tab 6-year swap rate\cr
      \code{s7y}  \tab 7-year swap rate\cr
      \code{s8y}  \tab 8-year swap rate\cr
      \code{s9y}  \tab 9-year swap rate\cr
      \code{s10y} \tab 10-year swap rate\cr
      \code{s12y} \tab 12-year swap rate\cr
      \code{s15y} \tab 15-year swap rate\cr
      \code{s20y} \tab 20-year swap rate\cr
      \code{s25y}  \tab 25-year swap rate\cr
      \code{s30y} \tab 30-year swap rate\cr
      \code{s40y} \tab 40-year swap rate\cr
      \code{s50y} \tab 50-year swap rate\cr
      \code{s60y} \tab 60-year swap rate\cr
      \code{s70y} \tab 70-year swap rate\cr
      \code{s80y} \tab 80-year swap rate\cr
      \code{s90y} \tab 90-year swap rate\cr
      \code{s100y} \tab 100-year swap rate\cr
      \code{fut1}--\code{fut8} \tab 3-month futures contracts\cr
      \code{fra3x6} \tab 3x6 FRA\cr
      \code{fra6x9} \tab 6x9 FRA\cr
      \code{fra6x12}\tab 6x12 FRA
    }
    Here rates are expected as fractions (so 5\% means .05).
    If \code{flat} is specified it must be the first and only item in
    the list. The eight futures correspond to the first eight IMM
    dates. The maturity dates of the instruments specified need not be
    ordered, but they must be distinct.
    }
  \item{times}{A vector of times at which to return the discount
    factors, forward rates, and zero rates. Times must be specified such
    that the largest time plus \code{dt} does not exceed the longest
    maturity of the instruments used for calibration (no extrapolation).
    }
  \item{legparams}{A list specifying the \code{dayCounter} the day count convention for the
   fixed leg (default is Thirty360), and  \code{fixFreq}, fixed coupon frequecny (defualt is Annual), \code{floatFreq},
   floating leg reset frequency (default is Semiannual).
  }

}
\value{
  \code{DiscountCurve} returns a list containing:
  \item{times}{Vector of input times}
  \item{discounts}{Corresponding discount factors}
  \item{forwards}{Corresponding forward rates with time span \code{dt}}
  \item{zerorates}{Corresponding zero coupon rates}
  \item{flatQuotes}{True if a flat quote was used, False otherwise}
  \item{params}{The input parameter list}
}
\details{
  This function is based on \code{QuantLib} Version 0.3.10. It
  introduces support for fixed-income instruments in \code{RQuantLib}.

  Forward rates and zero rates are computed assuming continuous
  compounding, so the forward rate \eqn{f} over the
  period from \eqn{t_1}{t1} to \eqn{t_2}{t2} is determined by the
  relation
  \deqn{d_1/d_2 = e^{f (t_2 - t_1)},}{d1/d2 = exp(f(t2 - t1)),}
  where \eqn{d_1}{d1} and \eqn{d_2}{d2} are discount factors
  corresponding to the two times. In the case of the zero rate
  \eqn{t_1}{t1} is the current time (the spot date).

  Curve construction can be a delicate problem and the algorithms may
  fail for some input data sets and/or some combinations of the
  values for \code{interpWhat} and \code{interpHow}.
  Fortunately, the C++ exception mechanism seems to work well with the R
  interface, and \code{QuantLib} exceptions are propagated back to the
  R user, usually with a message that indicates what went wrong. (The
  first part of the message contains technical information about the
  precise location of the problem in the \code{QuantLib} code. Scroll to
  the end to find information that is meaningful to the R user.)

}
\references{
  Brigo, D. and Mercurio, F. (2001) \emph{Interest Rate Models: Theory and
    Practice}, Springer-Verlag, New York.

  For information about \code{QuantLib} see \url{https://www.quantlib.org/}.

  For information about \code{RQuantLib} see
\url{http://dirk.eddelbuettel.com/code/rquantlib.html}.

}
\author{Dominick Samperi}

\seealso{\code{\link{BermudanSwaption}}}

\examples{
\dontrun{
savepar <- par(mfrow=c(3,3), mar=c(4,4,2,0.5))

## This data is taken from sample code shipped with QuantLib 0.9.7
## from the file Examples/Swap/swapvaluation
params <- list(tradeDate=as.Date('2004-09-20'),
               settleDate=as.Date('2004-09-22'),
               dt=.25,
               interpWhat="discount",
               interpHow="loglinear")
setEvaluationDate(as.Date("2004-09-20"))

## We get numerical issue for the spline interpolation if we add
## any on of these three extra futures -- the original example
## creates different curves based on different deposit, fra, futures
## and swap data
## Removing s2y helps, as kindly pointed out by Luigi Ballabio
tsQuotes <- list(d1w = 0.0382,
                 d1m = 0.0372,
                 d3m = 0.0363,
                 d6m = 0.0353,
                 d9m = 0.0348,
                 d1y = 0.0345,
                 fut1=96.2875,
                 fut2=96.7875,
                 fut3=96.9875,
                 fut4=96.6875,
                 fut5=96.4875,
                 fut6=96.3875,
                 fut7=96.2875,
                 fut8=96.0875,
#                 s2y = 0.037125,
                 s3y = 0.0398,
                 s5y = 0.0443,
                 s10y = 0.05165,
                 s15y = 0.055175)

times <- seq(0,10,.1)

# Loglinear interpolation of discount factors
curves <- DiscountCurve(params, tsQuotes, times)
plot(curves,setpar=FALSE)

# Linear interpolation of discount factors
params$interpHow="linear"
curves <- DiscountCurve(params, tsQuotes, times)
plot(curves,setpar=FALSE)

# Spline interpolation of discount factors
params$interpHow="spline"
curves <- DiscountCurve(params, tsQuotes, times)
plot(curves,setpar=FALSE)

par(savepar)
}
}
\keyword{models}

\name{EuropeanOption}
\alias{EuropeanOption}
\alias{EuropeanOption.default}
\title{European Option evaluation using Closed-Form solution}
\description{
  The \code{EuropeanOption} function evaluations an European-style
  option on a common stock using the Black-Scholes-Merton solution. The
  option value, the common first derivatives ("Greeks") as well as the
  calling parameters are returned.
}
\usage{
\method{EuropeanOption}{default}(type, underlying, strike,
	dividendYield, riskFreeRate, maturity, volatility,
	discreteDividends, discreteDividendsTimeUntil, dayCounter)
}
\arguments{
  \item{type}{A string with one of the values \code{call} or \code{put}}
  \item{underlying}{Current price of the underlying stock}
  \item{strike}{Strike price of the option}
  \item{dividendYield}{Continuous dividend yield (as a fraction) of the stock}
  \item{riskFreeRate}{Risk-free rate}
  \item{maturity}{Time to maturity in either fractional years or as a
    \sQuote{Date} object specifying the expiry date)}
  \item{volatility}{Volatility of the underlying stock}
  \item{discreteDividends}{Vector of discrete dividends (optional)}
  \item{discreteDividendsTimeUntil}{Vector of times to discrete dividends
    (in fractional years, optional)}
  \item{dayCounter}{Integer denoting the selected QuantLib day counter
    convention, default value is 0 for \sQuote{Actual360}}
}
\value{
  The \code{EuropeanOption} function returns an object of class
  \code{EuropeanOption} (which inherits from class
  \code{\link{Option}}). It contains a list with the following
  components:
  \item{value}{Value of option}
  \item{delta}{Sensitivity of the option value for a change in the underlying}
  \item{gamma}{Sensitivity of the option delta for a change in the underlying}
  \item{vega}{Sensitivity of the option value for a change in the
    underlying's volatility}
  \item{theta}{Sensitivity of the option value for a change in t, the
    remaining time to maturity}
  \item{rho}{Sensitivity of the option value for a change in the
    risk-free interest rate}
  \item{dividendRho}{Sensitivity of the option value for a change in the
    dividend yield}

}
\details{
  The well-known closed-form solution derived by Black, Scholes and
  Merton is used for valuation. Implied volatilities are calculated
  numerically.

  Please see any decent Finance textbook for background reading, and the
  \code{QuantLib} documentation for details on the \code{QuantLib}
  implementation.
}
\references{\url{https://www.quantlib.org/} for details on \code{QuantLib}.}
\author{Dirk Eddelbuettel \email{edd@debian.org} for the \R interface;
  the QuantLib Group for \code{QuantLib}}
\note{The interface might change in future release as \code{QuantLib}
  stabilises its own API.}
\seealso{\code{\link{EuropeanOptionImpliedVolatility}},
  \code{\link{EuropeanOptionArrays}},
  \code{\link{AmericanOption}},\code{\link{BinaryOption}}}

\examples{
## simple call with unnamed parameters
EuropeanOption("call", 100, 100, 0.01, 0.03, 0.5, 0.4)
## simple call with some explicit parameters, and slightly increased vol:
EuropeanOption(type="call", underlying=100, strike=100, dividendYield=0.01,
riskFreeRate=0.03, maturity=0.5, volatility=0.5)
## simple call with slightly shorter maturity: QuantLib 1.7 compiled with
## intra-day time calculation support with create slightly changed values
EuropeanOption(type="call", underlying=100, strike=100, dividendYield=0.01,
riskFreeRate=0.03, maturity=0.499, volatility=0.5)
}
\keyword{misc}

\name{Bond}
\alias{Bond}
\alias{plot.Bond}
\alias{print.Bond}
\alias{print.FixedRateBond}
\alias{summary.Bond}
\title{Base class for Bond price evalution}
\description{
  This class forms the basis from which the more specific classes are
  derived. }
\usage{
\method{print}{Bond}(x, digits=5, ...)
\method{print}{FixedRateBond}(x, digits=5, ...)
\method{plot}{Bond}(x, ...)
\method{summary}{Bond}(object, digits=5, ...)
}
\arguments{
  \item{x}{Any Bond object derived from this base class}
  \item{object}{Any Bond object derived from this base class}
  \item{digits}{Number of digits of precision shown}
  \item{...}{Further arguments}
}
\value{
  None, but side effects of displaying content.
}
\details{
         Please see any decent Finance textbook for background reading, and the
  \code{QuantLib} documentation for details on the \code{QuantLib}
  implementation.
}
\references{\url{https://www.quantlib.org/} for details on \code{QuantLib}.}
\author{Khanh Nguyen \email{knguyen@cs.umb.edu}; Dirk Eddelbuettel \email{edd@debian.org} for the \R interface;
  the QuantLib Group for \code{QuantLib}}
\note{The interface might change in future release as \code{QuantLib}
  stabilises its own API.}

\examples{
\dontrun{

## This data is taken from sample code shipped with QuantLib 0.9.7
## from the file Examples/Swap/swapvaluation
params <- list(tradeDate=as.Date('2004-09-20'),
               settleDate=as.Date('2004-09-22'),
               dt=.25,
               interpWhat="discount",
               interpHow="loglinear")
setEvaluationDate(as.Date("2004-09-20"))

## We got numerical issues for the spline interpolation if we add
## any on of these three extra futures, at least with QuantLib 0.9.7
## The curve data comes from QuantLib's Examples/Swap/swapvaluation.cpp
## Removing s2y helps, as kindly pointed out by Luigi Ballabio
tsQuotes <- list(d1w = 0.0382,
                 d1m = 0.0372,
                 fut1=96.2875,
                 fut2=96.7875,
                 fut3=96.9875,
                 fut4=96.6875,
                 fut5=96.4875,
                 fut6=96.3875,
                 fut7=96.2875,
                 fut8=96.0875,
                 # s2y = 0.037125,  ## s2y perturbs
                 s3y = 0.0398,
                 s5y = 0.0443,
                 s10y = 0.05165,
                 s15y = 0.055175)
times <- seq(0,10,.1)

setEvaluationDate(params$tradeDate)
discountCurve <- DiscountCurve(params, tsQuotes, times)

# price a zero coupon bond
bondparams <- list(faceAmount=100, issueDate=as.Date("2004-11-30"),
                   maturityDate=as.Date("2008-11-30"), redemption=100 )
dateparams <-list(settlementDays=1,
                  calendar="UnitedStates/GovernmentBond",
                  businessDayConvention=4)
ZeroCouponBond(bondparams, discountCurve, dateparams)

# price a fixed rate coupon bond

bond <- list(settlementDays=1, issueDate=as.Date("2004-11-30"),
             faceAmount=100, dayCounter='Thirty360',
             paymentConvention='Unadjusted')
schedule <- list(effectiveDate=as.Date("2004-11-30"),
                 maturityDate=as.Date("2008-11-30"),
                 period='Semiannual',
                 calendar='UnitedStates/GovernmentBond',
                 businessDayConvention='Unadjusted',
                 terminationDateConvention='Unadjusted',
                 dateGeneration='Forward',
                 endOfMonth=1)
calc=list(dayCounter='Actual360', compounding='Compounded',
          freq='Annual', durationType='Modified')
rates <- c(0.02875)
FixedRateBond(bond, rates, schedule, calc, discountCurve=discountCurve)

# price a fixed rate coupon bond from yield

yield <- 0.050517
FixedRateBond(bond, rates, schedule, calc, yield=yield)

# calculate the same bond from the clean price

price <- 92.167
FixedRateBond(bond, rates, schedule, calc, price=price)

# price a floating rate bond
bondparams <- list(faceAmount=100, issueDate=as.Date("2004-11-30"),
                   maturityDate=as.Date("2008-11-30"), redemption=100,
                   effectiveDate=as.Date("2004-12-01"))

dateparams <- list(settlementDays=1, calendar="UnitedStates/GovernmentBond",
                   dayCounter = 1, period=3, businessDayConvention = 1,
                   terminationDateConvention=1, dateGeneration=0, endOfMonth=0,
                   fixingDays = 1)

gearings <- spreads <- caps <- floors <- vector()

iborCurve <- DiscountCurve(params,list(flat=0.05), times)
ibor <- list(type="USDLibor", length=6, inTermOf="Month",
             term=iborCurve)
FloatingRateBond(bondparams, gearings, spreads, caps, floors,
                 ibor, discountCurve, dateparams)
}
}
\keyword{misc}

\name{AmericanOption}
\alias{AmericanOption}
\alias{AmericanOption.default}
\title{American Option evaluation using Finite Differences}
\description{
This function evaluations an American-style option on a common stock
using finite differences. The option value as well as the common first
derivatives ("Greeks") are returned.}
\usage{
\method{AmericanOption}{default}(type, underlying, strike,
		dividendYield, riskFreeRate, maturity, volatility,
		timeSteps=150, gridPoints=149, engine="BaroneAdesiWhaley",
		discreteDividends, discreteDividendsTimeUntil, dayCounter)
}
\arguments{
  \item{type}{A string with one of the values \code{call} or \code{put}}
  \item{underlying}{Current price of the underlying stock}
  \item{strike}{Strike price of the option}
  \item{dividendYield}{Continuous dividend yield (as a fraction) of the stock}
  \item{riskFreeRate}{Risk-free rate}
  \item{maturity}{Time to maturity in either fractional years or as a
    \sQuote{Date} object specifying the expiry date)}
  \item{volatility}{Volatility of the underlying stock}
  \item{timeSteps}{Time steps for the \dQuote{CrankNicolson} finite
    differences method engine, default value is 150}
  \item{gridPoints}{Grid points for the \dQuote{CrankNicolson} finite differences method,
    default value is 149}
  \item{engine}{String selecting pricing engine, currently supported are
    \dQuote{BaroneAdesiWhaley} and \dQuote{CrankNicolson}}
  \item{discreteDividends}{Vector of discrete dividends (optional)}
  \item{discreteDividendsTimeUntil}{Vector of times to discrete dividends
    (in fractional years, optional)}
  \item{dayCounter}{Integer denoting the selected QuantLib day counter
    convention, default value is 0 for \sQuote{Actual360}}
}
\value{
  An object of class \code{AmericanOption} (which inherits from class
  \code{\link{Option}}) is returned. It contains a list with the
  following components:
  \item{value}{Value of option}
  \item{delta}{Sensitivity of the option value for a change in the underlying}
  \item{gamma}{Sensitivity of the option delta for a change in the underlying}
  \item{vega}{Sensitivity of the option value for a change in the
    underlying's volatility}
  \item{theta}{Sensitivity of the option value for a change in t, the
    remaining time to maturity}
  \item{rho}{Sensitivity of the option value for a change in the
    risk-free interest rate}
  \item{dividendRho}{Sensitivity of the option value for a change in the
    dividend yield}

  Note that under the new pricing framework used in QuantLib,
  pricers do not provide analytics for all 'Greeks'. When
  \dQuote{CrankNicolson} is selected, then at least delta, gamma and
  vega are available. With the default pricing engine of
  \dQuote{BaroneAdesiWhaley}, no greeks are returned.

  The \dQuote{CrankNicolson} engine needs to be used when setting
  discrete dividends.
}
\details{
  The Finite Differences method is used to value the American Option.

  Please see any decent Finance textbook for background reading, and
  the \code{QuantLib} documentation for details on the \code{QuantLib}
  implementation.
}
\references{\url{https://www.quantlib.org/} for details on \code{QuantLib}.}
\author{Dirk Eddelbuettel \email{edd@debian.org} for the \R interface;
  the QuantLib Group for \code{QuantLib}}
\note{The interface might change in future release as \code{QuantLib}
  stabilises its own API.}
\seealso{\code{\link{EuropeanOption}}}

\examples{
# simple call with unnamed parameters
AmericanOption("call", 100, 100, 0.02, 0.03, 0.5, 0.4)
# simple call with some explicit parameters
AmericanOption("put", strike=100, volatility=0.4, 100, 0.02, 0.03, 0.5)
# simple call with unnamed parameters, using Crank-Nicolons
AmericanOption("put", strike=100, volatility=0.4, 100, 0.02, 0.03, 0.5, engine="CrankNicolson")
}
\keyword{misc}

\name{ImpliedVolatility}
\alias{ImpliedVolatility}
\alias{print.ImpliedVolatility}
\alias{summary.ImpliedVolatility}
\title{Base class for option-price implied volatility evalution}
\description{
  This class forms the basis from which the more specific classes are
  derived. }
\usage{
\method{print}{ImpliedVolatility}(x, digits=3, ...)
\method{summary}{ImpliedVolatility}(object, digits=3, ...)
}
\arguments{
  \item{x}{Any option-price implied volatility object derived
    from this base class} 
  \item{object}{Any option-price implied volatility object derived
    from this base class} 
  \item{digits}{Number of digits of precision shown} 
  \item{...}{Further arguments} 
}
\value{
  None, but side effects of displaying content.
}
\details{
  Please see any decent Finance textbook for background reading, and the
  \code{QuantLib} documentation for details on the \code{QuantLib}
  implementation.  
}
\references{\url{https://www.quantlib.org/} for details on \code{QuantLib}.}
\author{Dirk Eddelbuettel \email{edd@debian.org} for the \R interface;
  the QuantLib Group for \code{QuantLib}}
\note{The interface might change in future release as \code{QuantLib}
  stabilises its own API.}

\seealso{\code{\link{AmericanOptionImpliedVolatility}},
  \code{\link{EuropeanOptionImpliedVolatility}},
  \code{\link{AmericanOption}},\code{\link{EuropeanOption}},
  \code{\link{BinaryOption}}}

\examples{
impVol<-EuropeanOptionImpliedVolatility("call", value=11.10, strike=100,
                                        volatility=0.4, 100, 0.01, 0.03, 0.5)
print(impVol)
summary(impVol)
}
\keyword{misc}

\name{BarrierOption}
\alias{BarrierOption}
\alias{BarrierOption.default}
\title{Barrier Option evaluation using Closed-Form solution}
\description{
This function evaluations an Barrier option on a common stock
using a closed-form solution. The option value as well as the common first
derivatives ("Greeks") are returned.}
\usage{
\method{BarrierOption}{default}(barrType, type, underlying, strike,
                                dividendYield, riskFreeRate, maturity,
                                volatility, barrier, rebate=0.0, dayCounter=0)
}
\arguments{
  \item{barrType}{A string with one of the values \code{downin},
    \code{downout}, \code{upin} or \code{upout}}
  \item{type}{A string with one of the values \code{call} or \code{put}}
  \item{underlying}{Current price of the underlying stock}
  \item{strike}{Strike price of the option}
  \item{dividendYield}{Continuous dividend yield (as a fraction) of the stock}
  \item{riskFreeRate}{Risk-free rate}
  \item{maturity}{Time to maturity in either fractional years or as a
    \sQuote{Date} object specifying the expiry date)}
  \item{volatility}{Volatility of the underlying stock}
  \item{barrier}{Option barrier value}
  \item{rebate}{Optional option rebate, defaults to 0.0}
  \item{dayCounter}{Integer denoting the selected QuantLib day counter
    convention, default value is 0 for \sQuote{Actual360}}
}
\value{
  An object of class \code{BarrierOption} (which inherits from class
  \code{\link{Option}}) is returned. It contains a list with the
  following components:
  \item{value}{Value of option}
  \item{delta}{Sensitivity of the option value for a change in the underlying}
  \item{gamma}{Sensitivity of the option delta for a change in the underlying}
  \item{vega}{Sensitivity of the option value for a change in the
    underlying's volatility}
  \item{theta}{Sensitivity of the option value for a change in t, the
    remaining time to maturity}
  \item{rho}{Sensitivity of the option value for a change in the
    risk-free interest rate}
  \item{dividendRho}{Sensitivity of the option value for a change in the
    dividend yield}.

  Note that under the new pricing framework used in QuantLib, binary
  pricers do not provide analytics for 'Greeks'. This is expected to be
  addressed in future releases of QuantLib.
}
\details{
  A closed-form solution is used to value the Barrier Option. In the
  case of Barrier options, the calculations are from Haug's "Option
  pricing formulas" book (McGraw-Hill).

  Please see any decent Finance textbook for background reading, and
  the \code{QuantLib} documentation for details on the \code{QuantLib}
  implementation.
}
\references{\url{https://www.quantlib.org/} for details on \code{QuantLib}.}
\author{Dirk Eddelbuettel \email{edd@debian.org} for the \R interface;
  the QuantLib Group for \code{QuantLib}}
\note{The interface might change in future release as \code{QuantLib}
  stabilises its own API.}
\seealso{\code{\link{AmericanOption}},\code{\link{EuropeanOption}}}

\examples{
BarrierOption(barrType="downin", type="call", underlying=100,
	strike=100, dividendYield=0.02, riskFreeRate=0.03,
	maturity=0.5, volatility=0.4, barrier=90)
}
\keyword{misc}

\name{FittedBondCurve}
\alias{FittedBondCurve}
\alias{FittedBondCurve.default}
\alias{plot.FittedBondCurve}
\title{Returns the discount curve (with zero rates and forwards) given set of bonds}
\description{
  \code{FittedBondCurve} fits a term structure to a set of bonds
    using three different fitting methodologies. For more detail,
    see QuantLib/Example/FittedBondCurve.
}

\usage{
FittedBondCurve(curveparams, lengths, coupons, marketQuotes, dateparams)
}

\arguments{
\item{curveparams}{curve parameters
  \tabular{ll}{
    \code{method} \tab a string, fitting methods: "ExponentialSplinesFitting", \cr
    \code{} \tab  "SimplePolynomialFitting", "NelsonSiegelFitting"\cr
    \code{origDate} \tab a Date, starting date of the curve \cr
  }
}
\item{lengths}{an numeric vector, length of the bonds in year}
\item{coupons}{a numeric vector, coupon rate of the bonds}
\item{marketQuotes}{a numeric vector, market price of the bonds}
\item{dateparams}{(Optional) a named list, QuantLib's date parameters of the bond.
  \tabular{ll}{
    \code{settlementDays} \tab (Optional) a double, settlement days. \cr
    \code{}        \tab Default value is 1.\cr
    \code{dayCounter} \tab (Optional) a number or string, \cr
    \code{}        \tab day counter convention.\cr
    \code{}        \tab  See \link{Enum}. Default value is 'Thirty360' \cr
    \code{period}  \tab (Optional) a number or string, \cr
    \code{}       \tab  interest compounding interval. See \link{Enum}. \cr
    \code{}       \tab Default value is 'Semiannual'.\cr
    \code{businessDayConvention} \tab (Optional) a number or string, \cr
    \code{}                 \tab business day convention. \cr
    \tab See \link{Enum}. Default value is 'Following'. \cr
  }
  See example below.
}
}
\value{
  \code{table}, a three columns "date - zeroRate - discount" data frame
}

\details{
  Please see any decent Finance textbook for background reading, and the
  \code{QuantLib} documentation for details on the \code{QuantLib}
  implementation.
}
\references{
  \url{https://www.quantlib.org/} for details on \code{QuantLib}.
}
\author{Khanh Nguyen \email{knguyen@cs.umb.edu} for the inplementation; Dirk Eddelbuettel \email{edd@debian.org} for the \R interface;
  the QuantLib Group for \code{QuantLib}
}


\examples{
# commented-out as it runs longer than CRAN likes
\dontrun{
lengths <- c(2,4,6,8,10,12,14,16,18,20,22,24,26,28,30)
coupons <- c( 0.0200, 0.0225, 0.0250, 0.0275, 0.0300,
              0.0325, 0.0350, 0.0375, 0.0400, 0.0425,
              0.0450, 0.0475, 0.0500, 0.0525, 0.0550 )
marketQuotes <- rep(100, length(lengths))
dateparams <- list(settlementDays=0, period="Annual",
                   dayCounter="ActualActual",
                  businessDayConvention ="Unadjusted")
curveparams <- list(method="ExponentialSplinesFitting",
                    origDate = Sys.Date())
curve <- FittedBondCurve(curveparams, lengths, coupons, marketQuotes, dateparams)
z <- zoo::zoo(curve$table$zeroRates, order.by=curve$table$date)
plot(z)
}
}

\name{AmericanOptionImpliedVolatility}
\alias{AmericanOptionImpliedVolatility}
\alias{AmericanOptionImpliedVolatility.default}
\title{Implied Volatility calculation for American Option}
\description{
  The \code{AmericanOptionImpliedVolatility} function solves for the
  (unobservable) implied volatility, given an option price as well as
  the other required parameters to value an option.}
\usage{
\method{AmericanOptionImpliedVolatility}{default}(type, value,
		underlying, strike,dividendYield, riskFreeRate, maturity, volatility,
	        timeSteps=150, gridPoints=151, dayCounter=0)
}
\arguments{
  \item{type}{A string with one of the values \code{call} or \code{put}}
  \item{value}{Value of the option (used only for ImpliedVolatility calculation)}
  \item{underlying}{Current price of the underlying stock}
  \item{strike}{Strike price of the option}
  \item{dividendYield}{Continuous dividend yield (as a fraction) of the stock}
  \item{riskFreeRate}{Risk-free rate}
  \item{maturity}{Time to maturity in either fractional years or as a
    \sQuote{Date} object specifying the expiry date)}
  \item{volatility}{Initial guess for the volatility of the underlying
    stock}
  \item{timeSteps}{Time steps for the Finite Differences method, default
    value is 150}
  \item{gridPoints}{Grid points for the Finite Differences method,
    default value is 151}
  \item{dayCounter}{Integer denoting the selected QuantLib day counter
    convention, default value is 0 for \sQuote{Actual360}}
}
\value{
  The \code{AmericanOptionImpliedVolatility} function returns an numeric
  variable with volatility implied by the given market prices and given parameters.
}
\details{
  The Finite Differences method is used to value the American Option.
  Implied volatilities are then calculated numerically.

  Please see any decent Finance textbook for background reading, and the
  \code{QuantLib} documentation for details on the \code{QuantLib}
  implementation.
}
\references{\url{https://www.quantlib.org/} for details on \code{QuantLib}.}
\author{Dirk Eddelbuettel \email{edd@debian.org} for the \R interface;
  the QuantLib Group for \code{QuantLib}}
\note{The interface might change in future release as \code{QuantLib}
  stabilises its own API.}
\seealso{\code{\link{EuropeanOption}},\code{\link{AmericanOption}},\code{\link{BinaryOption}}}

\examples{
AmericanOptionImpliedVolatility(type="call", value=11.10, underlying=100,
	strike=100, dividendYield=0.01, riskFreeRate=0.03,
	maturity=0.5, volatility=0.4, dayCounter=1)
}
\keyword{misc}
